@@ -1,25 +1,26 @@
 #include <sirius.hpp>
 
 using namespace sirius;
-using namespace sddk;
 
-int test1(int size)
+int
+test1(int size)
 {
-    double t = -utils::wtime();
-    sddk::mdarray<char, 1> buf(size);
+    double t = -wtime();
+    mdarray<char, 1> buf({size});
     buf.zero();
 
     for (int r = 0; r < mpi::Communicator::world().size(); r++) {
         mpi::Communicator::world().bcast(buf.at(memory_t::host), size, r);
     }
-    t += utils::wtime();
+    t += wtime();
     if (mpi::Communicator::world().rank() == 0) {
         printf("time : %f sec.\n", t);
     }
     return 0;
 }
 
-int main(int argn, char** argv)
+int
+main(int argn, char** argv)
 {
     cmd_args args;
     args.register_key("--size=", "buffer size in bytes");
