@@ -1,6 +1,9 @@
 #include <sirius.hpp>
 
-int main(int argn, char** argv)
+using namespace sirius;
+
+int
+main(int argn, char** argv)
 {
     sirius::initialize(true);
 
@@ -9,7 +12,7 @@ int main(int argn, char** argv)
     std::cout << mpi::Communicator::world().size() << " " << mpi::Communicator::world().rank() << std::endl;
 
     auto blacs_handler = la::linalg_base::create_blacs_handler(mpi::Communicator::self().native());
-    blacs_handler = la::linalg_base::create_blacs_handler(mpi::Communicator::world().native());
+    blacs_handler      = la::linalg_base::create_blacs_handler(mpi::Communicator::world().native());
     std::cout << blacs_handler << std::endl;
 
     sirius::finalize(true);
