@@ -2,19 +2,19 @@
 
 using namespace sirius;
 
-void f(mdarray<int, 1> const& a)
+void
+f(mdarray<int, 1> const& a)
 {
     a.allocate_on_device();
-
 }
 
-int main(int argn, char** argv)
+int
+main(int argn, char** argv)
 {
     cmd_args args;
 
     args.parse_args(argn, argv);
-    if (args.exist("help"))
-    {
+    if (args.exist("help")) {
         printf("Usage: %s [options]\n", argv[0]);
         args.print_help();
         exit(0);
