@@ -2,7 +2,8 @@
 
 using namespace sirius;
 
-void test_nested_gemm()
+void
+test_nested_gemm()
 {
     int nt = omp_get_max_threads();
 
@@ -15,21 +16,23 @@ void test_nested_gemm()
     mdarray<double_complex, 2> c(m, n);
 
     for (int i = 0; i < m; i++)
-        for (int j = 0; j < k; j++) a(j, i) = 0.1;
+        for (int j = 0; j < k; j++)
+            a(j, i) = 0.1;
 
     for (int i = 0; i < n; i++)
-        for (int j = 0; j < n; j++) b(j, i) = 0.1;
+        for (int j = 0; j < n; j++)
+            b(j, i) = 0.1;
 
     c.zero();
-    
+
     /* warmup */
     linalg<device_t::CPU>::gemm(2, 0, m, n, k, a, b, c);
 
     double t = omp_get_wtime();
-    
+
     linalg<device_t::CPU>::gemm(2, 0, m, n, k, a, b, c);
 
-    double t0 = omp_get_wtime() - t;
+    double t0   = omp_get_wtime() - t;
     double perf = 8e-9 * m * n * k / t0;
 
     printf("performance (all threads): %f Gflops\n", perf);
@@ -38,11 +41,10 @@ void test_nested_gemm()
 
     #pragma omp parallel num_threads(1)
     {
-        for (int i = 1; i < nt; i++)
-        {
-            #ifdef __MKL
+        for (int i = 1; i < nt; i++) {
+#ifdef __MKL
             mkl_set_num_threads_local(i);
-            #endif
+#endif
             omp_set_num_threads(i);
 
             double t = omp_get_wtime();
@@ -54,20 +56,19 @@ void test_nested_gemm()
     }
 
     omp_set_nested(0);
-    #ifdef __MKL
+#ifdef __MKL
     mkl_set_num_threads_local(0);
-    #endif
+#endif
     omp_set_num_threads(nt);
 }
 
-
-int main(int argn, char** argv)
+int
+main(int argn, char** argv)
 {
     cmd_args args;
 
     args.parse_args(argn, argv);
-    if (args.exist("help"))
-    {
+    if (args.exist("help")) {
         printf("Usage: %s [options]\n", argv[0]);
         args.print_help();
         return 0;
