@@ -3,11 +3,11 @@
 /* test FFT: tranfrom random function to real space, transform back and compare with the original function */
 
 using namespace sirius;
-using namespace sddk;
 using namespace mpi;
 
 template <typename T>
-int test_fft_complex(cmd_args& args, device_t fft_pu__)
+int
+test_fft_complex(cmd_args& args, device_t fft_pu__)
 {
     double cutoff = args.value<double>("cutoff", 40);
 
@@ -27,20 +27,20 @@ int test_fft_complex(cmd_args& args, device_t fft_pu__)
     fft::Gvec_fft gvp(gvec, Communicator::world(), Communicator::self());
 
     fft::spfft_grid_type<T> spfft_grid(fft_grid[0], fft_grid[1], fft_grid[2], gvp.zcol_count(), spl_z.local_size(),
-                           SPFFT_PU_HOST, -1, Communicator::world().native(), SPFFT_EXCH_DEFAULT);
+                                       SPFFT_PU_HOST, -1, Communicator::world().native(), SPFFT_EXCH_DEFAULT);
 
     const auto fft_type = gvec.reduced() ? SPFFT_TRANS_R2C : SPFFT_TRANS_C2C;
 
     auto const& gv = gvp.gvec_array();
-    fft::spfft_transform_type<T> spfft(spfft_grid.create_transform(SPFFT_PU_HOST, fft_type, fft_grid[0], fft_grid[1], fft_grid[2],
-        spl_z.local_size(), gvp.count(), SPFFT_INDEX_TRIPLETS,
-        gv.at(memory_t::host)));
+    fft::spfft_transform_type<T> spfft(spfft_grid.create_transform(SPFFT_PU_HOST, fft_type, fft_grid[0], fft_grid[1],
+                                                                   fft_grid[2], spl_z.local_size(), gvp.count(),
+                                                                   SPFFT_INDEX_TRIPLETS, gv.at(memory_t::host)));
 
-    sddk::mdarray<std::complex<T>, 1> f(gvp.count());
+    mdarray<std::complex<T>, 1> f({gvp.count()});
     for (int ig = 0; ig < gvp.count(); ig++) {
-        f[ig] = utils::random<std::complex<T>>();
+        f[ig] = random<std::complex<T>>();
     }
-    sddk::mdarray<std::complex<T>, 1> g(gvp.count());
+    mdarray<std::complex<T>, 1> g({gvp.count()});
 
     spfft.backward(reinterpret_cast<T const*>(&f[0]), spfft.processing_unit());
     spfft.forward(spfft.processing_unit(), reinterpret_cast<T*>(&g[0]), SPFFT_FULL_SCALING);
@@ -59,7 +59,8 @@ int test_fft_complex(cmd_args& args, device_t fft_pu__)
 }
 
 template <typename T>
-int run_test(cmd_args& args)
+int
+run_test(cmd_args& args)
 {
     int result = test_fft_complex<T>(args, device_t::CPU);
 #ifdef SIRIUS_GPU
@@ -68,11 +69,12 @@ int run_test(cmd_args& args)
     return result;
 }
 
-int main(int argn, char **argv)
+int
+main(int argn, char** argv)
 {
     cmd_args args;
     args.register_key("--cutoff=", "{double} cutoff radius in G-space");
-    args.register_key("--fp32",    "run in FP32 arithmetics");
+    args.register_key("--fp32", "run in FP32 arithmetics");
 
     args.parse_args(argn, argv);
     if (args.exist("help")) {
@@ -85,7 +87,7 @@ int main(int argn, char **argv)
     printf("running %-30s : ", argv[0]);
     int result{0};
     if (args.exist("fp32")) {
-#if defined(USE_FP32)
+#if defined(SIRIUS_USE_FP32)
         result = run_test<float>(args);
 #else
         RTE_THROW("not compiled with FP32 support");
@@ -94,9 +96,15 @@ int main(int argn, char **argv)
         result = run_test<double>(args);
     }
     if (result) {
-        printf("\x1b[31m" "Failed" "\x1b[0m" "\n");
+        printf("\x1b[31m"
+               "Failed"
+               "\x1b[0m"
+               "\n");
     } else {
-        printf("\x1b[32m" "OK" "\x1b[0m" "\n");
+        printf("\x1b[32m"
+               "OK"
+               "\x1b[0m"
+               "\n");
     }
     sirius::finalize();
 
