@@ -2,7 +2,8 @@
 
 using namespace sirius;
 
-int run_test(cmd_args& args)
+int
+run_test(cmd_args& args)
 {
     Simulation_context ctx;
     ctx.import(args);
@@ -10,7 +11,8 @@ int run_test(cmd_args& args)
     return 0;
 }
 
-int main(int argn, char** argv)
+int
+main(int argn, char** argv)
 {
     cmd_args args;
     args.register_key("--control.verbosity=", "{int} verbosity level");
@@ -26,9 +28,15 @@ int main(int argn, char** argv)
     printf("running %-30s : ", argv[0]);
     int result = run_test(args);
     if (result) {
-        printf("\x1b[31m" "Failed" "\x1b[0m" "\n");
+        printf("\x1b[31m"
+               "Failed"
+               "\x1b[0m"
+               "\n");
     } else {
-        printf("\x1b[32m" "OK" "\x1b[0m" "\n");
+        printf("\x1b[32m"
+               "OK"
+               "\x1b[0m"
+               "\n");
     }
     sirius::finalize();
 
