@@ -2,16 +2,16 @@
 #include "testing.hpp"
 
 using namespace sirius;
-using namespace sddk;
 
-int test1()
+int
+test1()
 {
     for (int num_ranks = 1; num_ranks < 20; num_ranks++) {
         for (int N = 1; N < 1130; N++) {
-            splindex<splindex_t::block> spl(N, num_ranks, 0);
+            splindex_block<> spl(N, n_blocks(num_ranks), block_id(0));
             int sz = 0;
             for (int i = 0; i < num_ranks; i++) {
-                sz += (int)spl.local_size(i);
+                sz += spl.local_size(block_id(i));
             }
             if (sz != N) {
                 std::stringstream s;
@@ -20,21 +20,21 @@ int test1()
                 s << "computed global index size: " << sz << std::endl;
                 s << "number of ranks: " << num_ranks << std::endl;
                 for (int i = 0; i < num_ranks; i++) {
-                    s << "i, local_size(i): " << i << ", " << spl.local_size(i) << std::endl;
+                    s << "i, local_size(i): " << i << ", " << spl.local_size(block_id(i)) << std::endl;
                 }
                 throw std::runtime_error(s.str());
             }
             for (int i = 0; i < N; i++) {
-                int rank = spl.local_rank(i);
-                int offset = (int)spl.local_index(i);
-                if (i != (int)spl.global_index(offset, rank)) {
+                int rank   = spl.location(block_id(i)).ib;
+                int offset = spl.location(block_id(i)).index_local;
+                if (i != (int)spl.global_index(offset, block_id(rank))) {
                     std::stringstream s;
                     s << "test1: wrong index." << std::endl;
                     s << "global index size: " << N << std::endl;
                     s << "number of ranks: " << num_ranks << std::endl;
                     s << "global index: " << i << std::endl;
                     s << "rank, offset: " << rank << ", " << offset << std::endl;
-                    s << "computed global index: " << spl.global_index(offset, rank) << std::endl;
+                    s << "computed global index: " << spl.global_index(offset, block_id(rank)) << std::endl;
                     throw std::runtime_error(s.str());
                 }
             }
@@ -43,27 +43,34 @@ int test1()
     return 0;
 }
 
-int test2()
+int
+test2()
 {
     for (int bs = 1; bs < 17; bs++) {
         for (int num_ranks = 1; num_ranks < 13; num_ranks++) {
             for (int N = 1; N < 1113; N++) {
-                splindex<splindex_t::block_cyclic> spl(N, num_ranks, 0, bs);
+                splindex_block_cyclic<> spl(N, n_blocks(num_ranks), block_id(0), bs);
                 int sz = 0;
                 for (int i = 0; i < num_ranks; i++) {
-                    sz += (int)spl.local_size(i);
+                    sz += (int)spl.local_size(block_id(i));
                 }
                 if (sz != N) {
                     std::stringstream s;
 
-                    s << "test2: wrong sum of local sizes" << std::endl;
+                    s << "test2: wrong sum of local sizes" << std::endl
+                      << "N : " << N << std::endl
+                      << "num_ranks :" << num_ranks << std::endl
+                      << "block size : " << bs << std::endl;
+                    for (int i = 0; i < num_ranks; i++) {
+                        s << "rank, local_size : " << i << ", " << spl.local_size(block_id(i)) << std::endl;
+                    }
                     throw std::runtime_error(s.str());
                 }
 
                 for (int i = 0; i < N; i++) {
-                    int rank = spl.local_rank(i);
-                    int offset = (int)spl.local_index(i);
-                    if (i != (int)spl.global_index(offset, rank)) {
+                    int rank   = spl.location(block_id(i)).ib;
+                    int offset = spl.location(block_id(i)).index_local;
+                    if (i != (int)spl.global_index(offset, block_id(rank))) {
                         std::stringstream s;
                         s << "test2: wrong index" << std::endl;
                         s << "bs = " << bs << std::endl
@@ -72,7 +79,7 @@ int test2()
                           << "idx = " << i << std::endl
                           << "rank = " << rank << std::endl
                           << "offset = " << offset << std::endl
-                          << "computed index = " << spl.global_index(offset, rank) << std::endl;
+                          << "computed index = " << spl.global_index(offset, block_id(rank)) << std::endl;
                         throw std::runtime_error(s.str());
                     }
                 }
@@ -82,18 +89,19 @@ int test2()
     return 0;
 }
 
-int test3()
+int
+test3()
 {
     for (int num_ranks = 1; num_ranks < 20; num_ranks++) {
         for (int N = 1; N < 1130; N++) {
-            splindex<splindex_t::block> spl_tmp(N, num_ranks, 0);
+            splindex_block<> spl_tmp(N, n_blocks(num_ranks), block_id(0));
 
-            splindex<splindex_t::chunk> spl(N, num_ranks, 0, spl_tmp.counts());
+            splindex_chunk<> spl(N, n_blocks(num_ranks), block_id(0), spl_tmp.counts());
 
             for (int i = 0; i < N; i++) {
-                int rank = spl.local_rank(i);
-                int offset = spl.local_index(i);
-                if (i != spl.global_index(offset, rank)) {
+                int rank   = spl.location(block_id(i)).ib;
+                int offset = spl.location(block_id(i)).index_local;
+                if (i != spl.global_index(offset, block_id(rank))) {
                     std::stringstream s;
                     s << "test3: wrong index" << std::endl;
                     throw std::runtime_error(s.str());
@@ -104,7 +112,8 @@ int test3()
     return 0;
 }
 
-int main(int argn, char** argv)
+int
+main(int argn, char** argv)
 {
     int err{0};
     err += call_test("test block index", test1);
