@@ -1,13 +1,13 @@
 from .helpers import *
+import json
 from .coefficient_array import CoefficientArray, PwCoeffs
 from .py_sirius import *
-from .py_sirius import smearing as smearing
 from .py_sirius import K_point_set, Density
 from .logger import Logger
-from .operators import S_operator
+from .operators import S_operator, Sinv_operator, US_Precond
 import numpy as np
 from numpy import array, zeros
-__all__ = ["ot", "baarman", "bands", "edft", "smearing"]
+__all__ = ["ot", "baarman", "bands", "edft"]
 
 
 class OccupancyDescriptor(object):
@@ -40,6 +40,7 @@ class PWDescriptor(object):
     """
     Accessor for wave-function coefficients
     """
+
     def __set__(self, instance, value):
         from .helpers import store_pw_coeffs
         store_pw_coeffs(instance, value)
@@ -52,6 +53,7 @@ class KPointWeightDescriptor(object):
     """
     Accessor for k-point weights
     """
+
     def __get__(self, instance, owner):
 
         out = CoefficientArray(dtype=np.double, ctype=np.array)
@@ -67,6 +69,7 @@ class BandEnergiesDescriptor(object):
     """
     Accessor for band energies
     """
+
     def __get__(self, instance, owner):
 
         out = CoefficientArray(dtype=np.double, ctype=np.array)
@@ -77,7 +80,6 @@ class BandEnergiesDescriptor(object):
                 out[key] = np.array(instance[k].band_energies(ispn))
         return out
 
-
     def __set__(self, instance, value):
         for key, val in value._data.items():
             k, ispn = key
@@ -106,3 +108,27 @@ Density.rho = DensityDescriptor(0)
 Density.mx = DensityDescriptor(1)
 Density.my = DensityDescriptor(2)
 Density.mz = DensityDescriptor(3)
+
+
+# __repr__ for matrix / vector types
+def _show_array(self):
+    return np.array(self).__repr__()
+
+
+from .py_sirius import vector3d_double, matrix3d, vector3d_int
+
+vector3d_double.__repr__ = _show_array
+vector3d_int.__repr__ = _show_array
+matrix3d.__repr__ = _show_array
+matrix3di.__repr__ = _show_array
+
+def gvec_array(gvec):
+    return np.vstack([np.array(gvec.gvec(i)) for i in range(gvec.count())])
+
+Gvec.__array__ = gvec_array
+
+
+def timings():
+    """Return timings from SIRIUS internal profiler."""
+    from .py_sirius import _timings
+    return json.loads(_timings())
