@@ -1,9 +1,9 @@
 from .free_energy import FreeEnergy
-from .neugebaur import CG as NeugebaurCG
+from .neugebauer import CG as NeugebauerCG
 from .marzari import CG as MarzariCG
 from .smearing import (GaussianSplineSmearing,
                        FermiDiracSmearing,
                        make_fermi_dirac_smearing,
                        make_gaussian_spline_smearing)
-from .neugebaur import kb
+from .neugebauer import kb
 from .preconditioner import make_kinetic_precond, make_kinetic_precond2
