@@ -61,4 +61,8 @@ class FreeEnergy:
         self.energy.kpointset.fn = fn
         E, HX = self.energy.compute(cn)
         entropy = self.smearing.entropy(fn)
+
+        self.entropy = entropy
+        self.ks_energy = E
+
         return E + entropy, HX
