@@ -1,4 +1,5 @@
 from mpi4py import MPI
+import sys
 
 class Singleton(type):
     _instances = {}
@@ -13,9 +14,8 @@ class Singleton(type):
 class Logger:
     __metaclass__ = Singleton
 
-    def __init__(self, fout=None, comm=None, all_print=False):
+    def __init__(self, fout=None, comm=None):
         self.fout = fout
-        self._all_print = all_print
         if self.fout is not None:
             with open(self.fout, 'w'):
                 print('')
@@ -24,19 +24,20 @@ class Logger:
         else:
             self.comm = comm
 
-    def log(self, arg1, *args):
+    def log(self, arg1, *args, all_print=False):
         """
 
         """
-        if self.comm.rank == 0 or self._all_print:
+        if self.comm.rank == 0 or all_print:
             if self.fout is not None:
                 with open(self.fout, 'a') as fh:
-                    print(arg1, *args, file=fh)
-                    print(arg1, *args, file=sys.stdout)
+                    print(arg1, *args, file=fh, flush=True)
+                    print(arg1, *args, file=sys.stdout, flush=True)
             else:
                 print(arg1, *args)
-        elif self._all_print:
+        elif all_print:
             print(arg1, *args)
+        sys.stdout.flush()
 
-    def __call__(self, *args):
-        self.log(*args)
+    def __call__(self, *args, all_print=False):
+        self.log(*args, all_print=all_print)
