@@ -23,7 +23,7 @@
  */
 
 #include "simulation_parameters.hpp"
-#include "mpi/communicator.hpp"
+#include "core/mpi/communicator.hpp"
 #include "context/input_schema.hpp"
 
 #include <unordered_set>
@@ -114,7 +114,7 @@ compose_json(nlohmann::json const& schema__, nlohmann::json const& in__, nlohman
         std::stringstream ss;
         ss << "The following configuration parameters were not recognized and ignored: ";
         std::copy(visited.begin(), visited.end(), std::ostream_iterator<std::string>(ss, " "));
-        WARNING(ss)
+        RTE_WARNING(ss)
     }
 }
 
@@ -154,7 +154,7 @@ get_section_options(std::string const& section__)
 void
 Simulation_parameters::import(std::string const& str__)
 {
-    auto json = utils::read_json_from_file_or_string(str__);
+    auto json = read_json_from_file_or_string(str__);
     import(json);
 }
 
@@ -170,9 +170,9 @@ Simulation_parameters::import(cmd_args const& args__)
     cfg_.control().processing_unit(args__.value("control.processing_unit", cfg_.control().processing_unit()));
     cfg_.control().mpi_grid_dims(args__.value("control.mpi_grid_dims", cfg_.control().mpi_grid_dims()));
     cfg_.control().std_evp_solver_name(
-        args__.value("control.std_evp_solver_name", cfg_.control().std_evp_solver_name()));
+            args__.value("control.std_evp_solver_name", cfg_.control().std_evp_solver_name()));
     cfg_.control().gen_evp_solver_name(
-        args__.value("control.gen_evp_solver_name", cfg_.control().gen_evp_solver_name()));
+            args__.value("control.gen_evp_solver_name", cfg_.control().gen_evp_solver_name()));
     cfg_.control().fft_mode(args__.value("control.fft_mode", cfg_.control().fft_mode()));
     cfg_.control().verbosity(args__.value("control.verbosity", cfg_.control().verbosity()));
     cfg_.control().verification(args__.value("control.verification", cfg_.control().verification()));
@@ -180,9 +180,10 @@ Simulation_parameters::import(cmd_args const& args__)
     cfg_.parameters().ngridk(args__.value("parameters.ngridk", cfg_.parameters().ngridk()));
     cfg_.parameters().gamma_point(args__.value("parameters.gamma_point", cfg_.parameters().gamma_point()));
     cfg_.parameters().pw_cutoff(args__.value("parameters.pw_cutoff", cfg_.parameters().pw_cutoff()));
+    cfg_.parameters().gk_cutoff(args__.value("parameters.gk_cutoff", cfg_.parameters().gk_cutoff()));
 
     cfg_.iterative_solver().early_restart(
-        args__.value("iterative_solver.early_restart", cfg_.iterative_solver().early_restart()));
+            args__.value("iterative_solver.early_restart", cfg_.iterative_solver().early_restart()));
     cfg_.mixer().beta(args__.value("mixer.beta", cfg_.mixer().beta()));
     cfg_.mixer().type(args__.value("mixer.type", cfg_.mixer().type()));
 }
@@ -213,7 +214,7 @@ Simulation_parameters::processing_unit(std::string name__)
         }
     }
     cfg_.control().processing_unit(name__);
-    processing_unit_ = sddk::get_device_t(name__);
+    processing_unit_ = get_device_t(name__);
 }
 
 void
@@ -229,13 +230,13 @@ Simulation_parameters::electronic_structure_method(std::string name__)
     cfg_.parameters().electronic_structure_method(name__);
 
     std::map<std::string, electronic_structure_method_t> m = {
-        {"full_potential_lapwlo", electronic_structure_method_t::full_potential_lapwlo},
-        {"pseudopotential", electronic_structure_method_t::pseudopotential}};
+            {"full_potential_lapwlo", electronic_structure_method_t::full_potential_lapwlo},
+            {"pseudopotential", electronic_structure_method_t::pseudopotential}};
 
     if (m.count(name__) == 0) {
         std::stringstream s;
         s << "wrong type of electronic structure method: " << name__;
-        TERMINATE(s);
+        RTE_THROW(s);
     }
     electronic_structure_method_ = m[name__];
 }
