@@ -110,45 +110,57 @@ namespace sirius {
  *      \frac{N_{el}^2}{4 \lambda}
  *  \f]
  */
-double ewald_energy(const Simulation_context& ctx, const fft::Gvec& gvec, const Unit_cell& unit_cell);
+double
+ewald_energy(const Simulation_context& ctx, const fft::Gvec& gvec, const Unit_cell& unit_cell);
 
 /// Returns exchange correlation potential.
-double energy_vxc(Density const& density, Potential const& potential);
+double
+energy_vxc(Density const& density, Potential const& potential);
 
 /// Returns exchange correlation energy.
-double energy_exc(Density const& density, Potential const& potential);
+double
+energy_exc(Density const& density, Potential const& potential);
 
 /// Returns Hatree potential.
-double energy_vha(Potential const& potential);
+double
+energy_vha(Potential const& potential);
 
 /// TODO doc
-double energy_bxc(const Density& density, const Potential& potential);
+double
+energy_bxc(const Density& density, const Potential& potential);
 
 /// Return nucleus energy in the electrostatic field.
 /** Compute energy of nucleus in the electrostatic potential generated by the total (electrons + nuclei)
 j*  charge density. Diverging self-interaction term z*z/|r=0| is excluded. */
-double energy_enuc(Simulation_context const& ctx, Potential const& potential);
+double
+energy_enuc(Simulation_context const& ctx, Potential const& potential);
 
 /// TODO doc
-double energy_vloc(Density const& density, Potential const& potential);
+double
+energy_vloc(Density const& density, Potential const& potential);
 
 /// Return eigen-value sum of core states.
-double core_eval_sum(Unit_cell const& unit_cell);
+double
+core_eval_sum(Unit_cell const& unit_cell);
 
 /// TODO doc
-double valence_eval_sum(K_point_set const& kset);
+double
+valence_eval_sum(K_point_set const& kset);
 
 /// TODO doc
-double eval_sum(Unit_cell const& unit_cell, K_point_set const& kset);
+double
+eval_sum(Unit_cell const& unit_cell, K_point_set const& kset);
 
 /// TODO doc
-double energy_veff(Density const& density, Potential const& potential);
+double
+energy_veff(Density const& density, Potential const& potential);
 
 /// Return kinetic energy
-double energy_kin(Simulation_context const& ctx, K_point_set const& kset, Density const& density,
-                  Potential const& potential);
+double
+energy_kin(Simulation_context const& ctx, K_point_set const& kset, Density const& density, Potential const& potential);
 
-double energy_potential(Density const& density, Potential const& potential);
+double
+energy_potential(Density const& density, Potential const& potential);
 
 /// Total energy of the electronic subsystem.
 /** <b> Full potential total energy </b>
@@ -249,31 +261,46 @@ double energy_potential(Density const& density, Potential const& potential);
  *     E^{XC}[\rho + \rho_{core}, |{\bf m}|]
  *  \f]
  */
-double total_energy(Simulation_context const& ctx, K_point_set const& kset, Density const& density,
-                    Potential const& potential, double ewald_energy);
+double
+total_energy(Simulation_context const& ctx, K_point_set const& kset, Density const& density, Potential const& potential,
+             double ewald_energy);
 
-double one_electron_energy(Density const& density, Potential const& potential);
+double
+ks_energy(Simulation_context const& ctx, K_point_set const& kset, Density const& density, Potential const& potential,
+          double ewald_energy);
 
-double one_electron_energy_hubbard(Density const& density, Potential const& potential);
-double hubbard_energy(Density const& density);
+double
+ks_energy(Simulation_context const& ctx, const std::map<std::string, double>& energies);
+
+std::map<std::string, double>
+total_energy_components(Simulation_context const& ctx, const K_point_set& kset, Density const& density,
+                        Potential const& potential, double ewald_energy);
+
+double
+one_electron_energy(Density const& density, Potential const& potential);
+
+double
+one_electron_energy_hubbard(Density const& density, Potential const& potential);
+double
+hubbard_energy(Density const& density);
 
 inline auto
 energy_dict(Simulation_context const& ctx__, K_point_set const& kset__, Density const& density__,
-    Potential const& potential__, double ewald_energy__, double scf_correction__ = 0)
+            Potential const& potential__, double ewald_energy__, double scf_correction__ = 0)
 {
     nlohmann::json dict;
 
-    dict["energy"] =  nlohmann::json::object();
+    dict["energy"] = nlohmann::json::object();
 
-    dict["energy"]["total"]          = total_energy(ctx__, kset__, density__, potential__, ewald_energy__) + scf_correction__;
-    dict["energy"]["vha"]            = energy_vha(potential__);
-    dict["energy"]["vxc"]            = energy_vxc(density__, potential__);
-    dict["energy"]["exc"]            = energy_exc(density__, potential__);
-    dict["energy"]["bxc"]            = energy_bxc(density__, potential__);
-    dict["energy"]["veff"]           = energy_veff(density__, potential__);
-    dict["energy"]["eval_sum"]       = eval_sum(ctx__.unit_cell(), kset__);
-    dict["energy"]["kin"]            = energy_kin(ctx__, kset__, density__, potential__);
-    dict["energy"]["ewald"]          = ewald_energy__;
+    dict["energy"]["total"]    = total_energy(ctx__, kset__, density__, potential__, ewald_energy__) + scf_correction__;
+    dict["energy"]["vha"]      = energy_vha(potential__);
+    dict["energy"]["vxc"]      = energy_vxc(density__, potential__);
+    dict["energy"]["exc"]      = energy_exc(density__, potential__);
+    dict["energy"]["bxc"]      = energy_bxc(density__, potential__);
+    dict["energy"]["veff"]     = energy_veff(density__, potential__);
+    dict["energy"]["eval_sum"] = eval_sum(ctx__.unit_cell(), kset__);
+    dict["energy"]["kin"]      = energy_kin(ctx__, kset__, density__, potential__);
+    dict["energy"]["ewald"]    = ewald_energy__;
     dict["energy"]["scf_correction"] = scf_correction__;
     dict["energy"]["entropy_sum"]    = kset__.entropy_sum();
     dict["efermi"]                   = kset__.energy_fermi();
