@@ -22,13 +22,13 @@
  *  \brief Contains implementation of sirius::Stress class.
  */
 
-#include "linalg/r3.hpp"
+#include "core/r3/r3.hpp"
+#include "core/profiler.hpp"
 #include "k_point/k_point.hpp"
-#include "stress.hpp"
 #include "non_local_functor.hpp"
-#include "utils/profiler.hpp"
 #include "dft/energy.hpp"
-#include "symmetry/crystal_symmetry.hpp"
+#include "symmetry/symmetrize_stress_tensor.hpp"
+#include "stress.hpp"
 
 namespace sirius {
 
@@ -38,23 +38,22 @@ Stress::calc_stress_nonloc_aux()
 {
     PROFILE("sirius::Stress|nonloc");
 
-    sddk::mdarray<real_type<F>, 2> collect_result(9, ctx_.unit_cell().num_atoms());
+    mdarray<real_type<F>, 2> collect_result({9, ctx_.unit_cell().num_atoms()});
     collect_result.zero();
 
     stress_nonloc_.zero();
 
     /* if there are no beta projectors then get out there */
-    if (ctx_.unit_cell().mt_lo_basis_size() == 0) {
+    if (ctx_.unit_cell().max_mt_basis_size() == 0) {
         return;
     }
 
     print_memory_usage(ctx_.out(), FILE_LINE);
 
-    for (int ikloc = 0; ikloc < kset_.spl_num_kpoints().local_size(); ikloc++) {
-        int ik  = kset_.spl_num_kpoints(ikloc);
-        auto kp = kset_.get<T>(ik);
-        auto mem = ctx_.processing_unit() == sddk::device_t::CPU ? sddk::memory_t::host : sddk::memory_t::device;
-        auto mg = kp->spinor_wave_functions().memory_guard(mem, wf::copy_to::device);
+    for (auto it : kset_.spl_num_kpoints()) {
+        auto kp  = kset_.get<T>(it.i);
+        auto mem = ctx_.processing_unit_memory_t();
+        auto mg  = kp->spinor_wave_functions().memory_guard(mem, wf::copy_to::device);
         Beta_projectors_strain_deriv<T> bp_strain_deriv(ctx_, kp->gkvec());
 
         add_k_point_contribution_nonlocal<T, F>(ctx_, bp_strain_deriv, *kp, collect_result);
@@ -81,7 +80,7 @@ Stress::calc_stress_nonloc_aux()
 
     stress_nonloc_ *= (1.0 / ctx_.unit_cell().omega());
 
-    symmetrize(stress_nonloc_);
+    symmetrize_stress_tensor(ctx_.unit_cell().symmetry(), stress_nonloc_);
 }
 
 r3::matrix<double>
@@ -118,8 +117,8 @@ Stress::calc_stress_hubbard()
     auto r = ctx_.unit_cell().num_hubbard_wf();
     /* if there are no beta projectors then get out there */
     /* TODO : Need to fix the case where pp have no beta projectors */
-    if (ctx_.unit_cell().mt_lo_basis_size() == 0) {
-        TERMINATE("Hubbard forces : Your pseudo potentials do not have beta projectors. This need a proper fix");
+    if (ctx_.unit_cell().max_mt_basis_size() == 0) {
+        RTE_THROW("Hubbard forces : Your pseudo potentials do not have beta projectors. This need a proper fix");
         return stress_hubbard_;
     }
 
@@ -127,18 +126,17 @@ Stress::calc_stress_hubbard()
 
     auto nhwf = ctx_.unit_cell().num_hubbard_wf().first;
 
-    sddk::mdarray<std::complex<double>, 4> dn(nhwf, nhwf, 2, 9);
+    mdarray<std::complex<double>, 4> dn({nhwf, nhwf, 2, 9});
     if (is_device_memory(ctx_.processing_unit_memory_t())) {
         dn.allocate(ctx_.processing_unit_memory_t());
     }
-    for (int ikloc = 0; ikloc < kset_.spl_num_kpoints().local_size(); ikloc++) {
-        int ik  = kset_.spl_num_kpoints(ikloc);
-        auto kp = kset_.get<double>(ik);
+    for (auto it : kset_.spl_num_kpoints()) {
+        auto kp = kset_.get<double>(it.i);
         dn.zero();
         if (is_device_memory(ctx_.processing_unit_memory_t())) {
             dn.zero(ctx_.processing_unit_memory_t());
         }
-        kp->beta_projectors().prepare();
+        // kp->beta_projectors().prepare();
         auto mg1 = kp->spinor_wave_functions().memory_guard(ctx_.processing_unit_memory_t(), wf::copy_to::device);
         auto mg2 = kp->hubbard_wave_functions_S().memory_guard(ctx_.processing_unit_memory_t(), wf::copy_to::device);
         auto mg3 = kp->atomic_wave_functions().memory_guard(ctx_.processing_unit_memory_t(), wf::copy_to::device);
@@ -164,10 +162,10 @@ Stress::calc_stress_hubbard()
                             for (int m1 = 0; m1 < lmax_at; m1++) {
                                 for (int m2 = 0; m2 < lmax_at; m2++) {
                                     stress_hubbard_(dir1, dir2) -=
-                                        (potential_.hubbard_potential().local(at_lvl)(m2, m1, ispn) *
-                                         dn(offset + m1, offset + m2, ispn, dir1 + 3 * dir2))
-                                        .real() /
-                                        ctx_.unit_cell().omega();
+                                            (potential_.hubbard_potential().local(at_lvl)(m2, m1, ispn) *
+                                             dn(offset + m1, offset + m2, ispn, dir1 + 3 * dir2))
+                                                    .real() /
+                                            ctx_.unit_cell().omega();
                                 }
                             }
                         }
@@ -177,7 +175,7 @@ Stress::calc_stress_hubbard()
                 double d = 0;
                 for (int i = 0; i < ctx_.cfg().hubbard().nonlocal().size(); i++) {
                     auto nl = ctx_.cfg().hubbard().nonlocal(i);
-                    int ia = nl.atom_pair()[0];
+                    int ia  = nl.atom_pair()[0];
                     int ja  = nl.atom_pair()[1];
                     int il  = nl.l()[0];
                     int jl  = nl.l()[1];
@@ -194,8 +192,8 @@ Stress::calc_stress_hubbard()
                     for (int is = 0; is < ctx_.num_spins(); is++) {
                         for (int m2 = 0; m2 < 2 * jl + 1; m2++) {
                             for (int m1 = 0; m1 < 2 * il + 1; m1++) {
-                              auto result1_ = z1 * std::conj(dn(offset2 + m2, offset1 + m1, is, dir1 + 3 * dir2)) *
-                                    potential_.hubbard_potential().nonlocal(i)(m1, m2, is);
+                                auto result1_ = z1 * std::conj(dn(offset2 + m2, offset1 + m1, is, dir1 + 3 * dir2)) *
+                                                potential_.hubbard_potential().nonlocal(i)(m1, m2, is);
                                 d += std::real(result1_);
                             }
                         }
@@ -204,12 +202,11 @@ Stress::calc_stress_hubbard()
                 stress_hubbard_(dir1, dir2) -= d / ctx_.unit_cell().omega();
             }
         }
-        kp->beta_projectors().dismiss();
     }
 
     /* global reduction */
     kset_.comm().allreduce(&stress_hubbard_(0, 0), 9);
-    symmetrize(stress_hubbard_);
+    symmetrize_stress_tensor(ctx_.unit_cell().symmetry(), stress_hubbard_);
 
     stress_hubbard_ = -1.0 * stress_hubbard_;
 
@@ -237,28 +234,29 @@ Stress::calc_stress_core()
         return stress_core_;
     }
 
-    potential_.xc_potential().fft_transform(-1);
+    potential_.xc_potential().rg().fft_transform(-1);
 
-    auto q     = ctx_.gvec().shells_len();
-    auto ff    = ctx_.ps_core_ri_djl().values(q, ctx_.comm());
-    auto drhoc = ctx_.make_periodic_function<sddk::index_domain_t::local>(ff);
+    auto q        = ctx_.gvec().shells_len();
+    auto const ff = ctx_.ri().ps_core_djl_->values(q, ctx_.comm());
+    auto drhoc =
+            make_periodic_function<index_domain_t::local>(ctx_.unit_cell(), ctx_.gvec(), ctx_.phase_factors_t(), ff);
 
     double sdiag{0};
     int ig0 = ctx_.gvec().skip_g0();
 
     for (int igloc = ig0; igloc < ctx_.gvec().count(); igloc++) {
-        auto G = ctx_.gvec().gvec_cart<sddk::index_domain_t::local>(igloc);
+        auto G = ctx_.gvec().gvec_cart<index_domain_t::local>(igloc);
         auto g = G.length();
 
         for (int mu : {0, 1, 2}) {
             for (int nu : {0, 1, 2}) {
                 stress_core_(mu, nu) -=
-                    std::real(std::conj(potential_.xc_potential().f_pw_local(igloc)) * drhoc[igloc]) *
-                    G[mu] * G[nu] / g;
+                        std::real(std::conj(potential_.xc_potential().rg().f_pw_local(igloc)) * drhoc[igloc]) * G[mu] *
+                        G[nu] / g;
             }
         }
 
-        sdiag += std::real(std::conj(potential_.xc_potential().f_pw_local(igloc)) *
+        sdiag += std::real(std::conj(potential_.xc_potential().rg().f_pw_local(igloc)) *
                            density_.rho_pseudo_core().f_pw_local(igloc));
     }
 
@@ -267,8 +265,8 @@ Stress::calc_stress_core()
         sdiag *= 2;
     }
     if (ctx_.comm().rank() == 0) {
-        sdiag +=
-            std::real(std::conj(potential_.xc_potential().f_pw_local(0)) * density_.rho_pseudo_core().f_pw_local(0));
+        sdiag += std::real(std::conj(potential_.xc_potential().rg().f_pw_local(0)) *
+                           density_.rho_pseudo_core().f_pw_local(0));
     }
 
     for (int mu : {0, 1, 2}) {
@@ -277,7 +275,7 @@ Stress::calc_stress_core()
 
     ctx_.comm().allreduce(&stress_core_(0, 0), 9);
 
-    symmetrize(stress_core_);
+    symmetrize_stress_tensor(ctx_.unit_cell().symmetry(), stress_core_);
 
     return stress_core_;
 }
@@ -304,8 +302,8 @@ Stress::calc_stress_xc()
         if (ctx_.num_spins() == 1) {
             Smooth_periodic_function<double> rhovc(ctx_.spfft<double>(), ctx_.gvec_fft_sptr());
             rhovc.zero();
-            rhovc.add(density_.rho());
-            rhovc.add(density_.rho_pseudo_core());
+            rhovc += density_.rho().rg();
+            rhovc += density_.rho_pseudo_core();
 
             /* transform to PW domain */
             rhovc.fft_transform(-1);
@@ -321,8 +319,8 @@ Stress::calc_stress_xc()
             for (int irloc = 0; irloc < ctx_.spfft<double>().local_slice_size(); irloc++) {
                 for (int mu = 0; mu < 3; mu++) {
                     for (int nu = 0; nu < 3; nu++) {
-                        t(mu, nu) +=
-                            2 * grad_rho[mu].f_rg(irloc) * grad_rho[nu].f_rg(irloc) * potential_.vsigma(0).f_rg(irloc);
+                        t(mu, nu) += 2 * grad_rho[mu].value(irloc) * grad_rho[nu].value(irloc) *
+                                     potential_.vsigma(0).value(irloc);
                     }
                 }
             }
@@ -348,13 +346,13 @@ Stress::calc_stress_xc()
             for (int irloc = 0; irloc < ctx_.spfft<double>().local_slice_size(); irloc++) {
                 for (int mu = 0; mu < 3; mu++) {
                     for (int nu = 0; nu < 3; nu++) {
-                        t(mu, nu) += grad_rho_up[mu].f_rg(irloc) * grad_rho_up[nu].f_rg(irloc) * 2 *
-                                         potential_.vsigma(0).f_rg(irloc) +
-                                     (grad_rho_up[mu].f_rg(irloc) * grad_rho_dn[nu].f_rg(irloc) +
-                                      grad_rho_dn[mu].f_rg(irloc) * grad_rho_up[nu].f_rg(irloc)) *
-                                         potential_.vsigma(1).f_rg(irloc) +
-                                     grad_rho_dn[mu].f_rg(irloc) * grad_rho_dn[nu].f_rg(irloc) * 2 *
-                                         potential_.vsigma(2).f_rg(irloc);
+                        t(mu, nu) += grad_rho_up[mu].value(irloc) * grad_rho_up[nu].value(irloc) * 2 *
+                                             potential_.vsigma(0).value(irloc) +
+                                     (grad_rho_up[mu].value(irloc) * grad_rho_dn[nu].value(irloc) +
+                                      grad_rho_dn[mu].value(irloc) * grad_rho_up[nu].value(irloc)) *
+                                             potential_.vsigma(1).value(irloc) +
+                                     grad_rho_dn[mu].value(irloc) * grad_rho_dn[nu].value(irloc) * 2 *
+                                             potential_.vsigma(2).value(irloc);
                     }
                 }
             }
@@ -364,7 +362,7 @@ Stress::calc_stress_xc()
         stress_xc_ += t;
     }
 
-    symmetrize(stress_xc_);
+    symmetrize_stress_tensor(ctx_.unit_cell().symmetry(), stress_xc_);
 
     return stress_xc_;
 }
@@ -377,27 +375,27 @@ Stress::calc_stress_us()
     stress_us_.zero();
 
     /* check if we have beta projectors. Only for pseudo potentials */
-    if (ctx_.unit_cell().mt_lo_basis_size() == 0) {
+    if (ctx_.unit_cell().max_mt_basis_size() == 0) {
         return stress_us_;
     }
 
     potential_.fft_transform(-1);
 
     la::lib_t la{la::lib_t::none};
-    sddk::memory_t qmem{sddk::memory_t::none};
+    memory_t qmem{memory_t::none};
 
-    sddk::memory_pool* mp{nullptr};
+    memory_pool* mp{nullptr};
     switch (ctx_.processing_unit()) {
-        case sddk::device_t::CPU: {
-            mp   = &get_memory_pool(sddk::memory_t::host);
+        case device_t::CPU: {
+            mp   = &get_memory_pool(memory_t::host);
             la   = la::lib_t::blas;
-            qmem = sddk::memory_t::host;
+            qmem = memory_t::host;
             break;
         }
-        case sddk::device_t::GPU: {
-            mp   = &get_memory_pool(sddk::memory_t::host_pinned);
+        case device_t::GPU: {
+            mp   = &get_memory_pool(memory_t::host_pinned);
             la   = la::lib_t::spla;
-            qmem = sddk::memory_t::host;
+            qmem = memory_t::host;
             break;
         }
     }
@@ -408,15 +406,16 @@ Stress::calc_stress_us()
             continue;
         }
 
-        Augmentation_operator q_deriv(ctx_.unit_cell().atom_type(iat), ctx_.gvec(), ctx_.aug_ri(), ctx_.aug_ri_djl());
+        Augmentation_operator q_deriv(ctx_.unit_cell().atom_type(iat), ctx_.gvec(), *ctx_.ri().aug_,
+                                      *ctx_.ri().aug_djl_);
 
         auto nbf = atom_type.mt_basis_size();
 
         /* get auxiliary density matrix */
-        auto dm = density_.density_matrix_aux(density_.density_matrix(), iat);
+        auto dm = density_.density_matrix_aux(atom_type);
 
-        sddk::mdarray<std::complex<double>, 2> phase_factors(atom_type.num_atoms(), ctx_.gvec().count(),
-                                                       get_memory_pool(sddk::memory_t::host));
+        mdarray<std::complex<double>, 2> phase_factors({atom_type.num_atoms(), ctx_.gvec().count()},
+                                                       get_memory_pool(memory_t::host));
 
         PROFILE_START("sirius::Stress|us|phase_fac");
         #pragma omp parallel for
@@ -428,8 +427,8 @@ Stress::calc_stress_us()
         }
         PROFILE_STOP("sirius::Stress|us|phase_fac");
 
-        sddk::mdarray<double, 2> v_tmp(atom_type.num_atoms(), ctx_.gvec().count() * 2, *mp);
-        sddk::mdarray<double, 2> tmp(nbf * (nbf + 1) / 2, atom_type.num_atoms(), *mp);
+        mdarray<double, 2> v_tmp({atom_type.num_atoms(), ctx_.gvec().count() * 2}, *mp);
+        mdarray<double, 2> tmp({nbf * (nbf + 1) / 2, atom_type.num_atoms()}, *mp);
         /* over spin components, can be from 1 to 4 */
         for (int nu = 0; nu < 3; nu++) {
             /* generate dQ(G)/dG */
@@ -446,11 +445,11 @@ Stress::calc_stress_us()
                     }
                     #pragma omp parallel for
                     for (int igloc = igloc0; igloc < ctx_.gvec().count(); igloc++) {
-                        auto gvc = ctx_.gvec().gvec_cart<sddk::index_domain_t::local>(igloc);
+                        auto gvc = ctx_.gvec().gvec_cart<index_domain_t::local>(igloc);
                         double g = gvc.length();
 
                         for (int ia = 0; ia < atom_type.num_atoms(); ia++) {
-                            auto z = phase_factors(ia, igloc) * potential_.component(ispin).f_pw_local(igloc) *
+                            auto z = phase_factors(ia, igloc) * potential_.component(ispin).rg().f_pw_local(igloc) *
                                      (-gvc[mu] / g);
                             v_tmp(ia, 2 * igloc)     = z.real();
                             v_tmp(ia, 2 * igloc + 1) = z.imag();
@@ -460,9 +459,9 @@ Stress::calc_stress_us()
 
                     PROFILE_START("sirius::Stress|us|gemm");
                     la::wrap(la).gemm('N', 'T', nbf * (nbf + 1) / 2, atom_type.num_atoms(), 2 * ctx_.gvec().count(),
-                                    &la::constant<double>::one(), q_deriv.q_pw().at(qmem), q_deriv.q_pw().ld(),
-                                    v_tmp.at(sddk::memory_t::host), v_tmp.ld(), &la::constant<double>::zero(),
-                                    tmp.at(sddk::memory_t::host), tmp.ld());
+                                      &la::constant<double>::one(), q_deriv.q_pw().at(qmem), q_deriv.q_pw().ld(),
+                                      v_tmp.at(memory_t::host), v_tmp.ld(), &la::constant<double>::zero(),
+                                      tmp.at(memory_t::host), tmp.ld());
                     PROFILE_STOP("sirius::Stress|us|gemm");
 
                     for (int ia = 0; ia < atom_type.num_atoms(); ia++) {
@@ -482,7 +481,7 @@ Stress::calc_stress_us()
 
     stress_us_ *= (1.0 / ctx_.unit_cell().omega());
 
-    symmetrize(stress_us_);
+    symmetrize_stress_tensor(ctx_.unit_cell().symmetry(), stress_us_);
 
     return stress_us_;
 }
@@ -502,7 +501,7 @@ Stress::calc_stress_ewald()
     for (int igloc = ig0; igloc < ctx_.gvec().count(); igloc++) {
         int ig = ctx_.gvec().offset() + igloc;
 
-        auto G          = ctx_.gvec().gvec_cart<sddk::index_domain_t::local>(igloc);
+        auto G          = ctx_.gvec().gvec_cart<index_domain_t::local>(igloc);
         double g2       = std::pow(G.length(), 2);
         double g2lambda = g2 / 4.0 / lambda;
 
@@ -553,7 +552,7 @@ Stress::calc_stress_ewald()
         }
     }
 
-    symmetrize(stress_ewald_);
+    symmetrize_stress_tensor(ctx_.unit_cell().symmetry(), stress_ewald_);
 
     return stress_ewald_;
 }
@@ -564,9 +563,7 @@ Stress::print_info(std::ostream& out__, int verbosity__) const
     auto print_stress = [&](std::string label__, r3::matrix<double> const& s) {
         out__ << "=== " << label__ << " ===" << std::endl;
         for (int mu : {0, 1, 2}) {
-           out__ << utils::ffmt(12, 6) << s(mu, 0)
-                 << utils::ffmt(12, 6) << s(mu, 1)
-                 << utils::ffmt(12, 6) << s(mu, 2) << std::endl;
+            out__ << ffmt(12, 6) << s(mu, 0) << ffmt(12, 6) << s(mu, 1) << ffmt(12, 6) << s(mu, 2) << std::endl;
         }
     };
 
@@ -619,9 +616,9 @@ Stress::calc_stress_har()
 
     int ig0 = ctx_.gvec().skip_g0();
     for (int igloc = ig0; igloc < ctx_.gvec().count(); igloc++) {
-        auto G    = ctx_.gvec().gvec_cart<sddk::index_domain_t::local>(igloc);
+        auto G    = ctx_.gvec().gvec_cart<index_domain_t::local>(igloc);
         double g2 = std::pow(G.length(), 2);
-        auto z    = density_.rho().f_pw_local(igloc);
+        auto z    = density_.rho().rg().f_pw_local(igloc);
         double d  = twopi * (std::pow(z.real(), 2) + std::pow(z.imag(), 2)) / g2;
 
         for (int mu : {0, 1, 2}) {
@@ -640,7 +637,7 @@ Stress::calc_stress_har()
 
     ctx_.comm().allreduce(&stress_har_(0, 0), 9);
 
-    symmetrize(stress_har_);
+    symmetrize_stress_tensor(ctx_.unit_cell().symmetry(), stress_har_);
 
     return stress_har_;
 }
@@ -651,12 +648,11 @@ Stress::calc_stress_kin_aux()
 {
     stress_kin_.zero();
 
-    for (int ikloc = 0; ikloc < kset_.spl_num_kpoints().local_size(); ikloc++) {
-        int ik  = kset_.spl_num_kpoints(ikloc);
-        auto kp = kset_.get<T>(ik);
+    for (auto it : kset_.spl_num_kpoints()) {
+        auto kp = kset_.get<T>(it.i);
 
         double fact = kp->gkvec().reduced() ? 2.0 : 1.0;
-        fact *=  kp->weight();
+        fact *= kp->weight();
 
         #pragma omp parallel
         {
@@ -665,7 +661,7 @@ Stress::calc_stress_kin_aux()
                 #pragma omp for
                 for (int i = 0; i < kp->num_occupied_bands(ispin); i++) {
                     for (int igloc = 0; igloc < kp->num_gkvec_loc(); igloc++) {
-                        auto Gk = kp->gkvec().template gkvec_cart<sddk::index_domain_t::local>(igloc);
+                        auto Gk = kp->gkvec().template gkvec_cart<index_domain_t::local>(igloc);
 
                         double f = kp->band_occupancy(i, ispin);
                         auto z = kp->spinor_wave_functions().pw_coeffs(igloc, wf::spin_index(ispin), wf::band_index(i));
@@ -687,7 +683,7 @@ Stress::calc_stress_kin_aux()
 
     stress_kin_ *= (-1.0 / ctx_.unit_cell().omega());
 
-    symmetrize(stress_kin_);
+    symmetrize_stress_tensor(ctx_.unit_cell().symmetry(), stress_kin_);
 }
 
 r3::matrix<double>
@@ -695,7 +691,7 @@ Stress::calc_stress_kin()
 {
     PROFILE("sirius::Stress|kin");
     if (ctx_.cfg().parameters().precision_wf() == "fp32") {
-#if defined(USE_FP32)
+#if defined(SIRIUS_USE_FP32)
         this->calc_stress_kin_aux<float>();
 #endif
     } else {
@@ -704,28 +700,6 @@ Stress::calc_stress_kin()
     return stress_kin_;
 }
 
-void
-Stress::symmetrize(r3::matrix<double>& mtrx__) const
-{
-    if (!ctx_.use_symmetry()) {
-        return;
-    }
-
-    r3::matrix<double> result;
-
-    for (int i = 0; i < ctx_.unit_cell().symmetry().size(); i++) {
-        auto R = ctx_.unit_cell().symmetry()[i].spg_op.Rcp;
-        result = result + dot(dot(transpose(R), mtrx__), R);
-    }
-
-    mtrx__ = result * (1.0 / ctx_.unit_cell().symmetry().size());
-
-    std::vector<std::array<int, 2>> idx = {{0, 1}, {0, 2}, {1, 2}};
-    for (auto e : idx) {
-        mtrx__(e[0], e[1]) = mtrx__(e[1], e[0]) = 0.5 * (mtrx__(e[0], e[1]) + mtrx__(e[1], e[0]));
-    }
-}
-
 r3::matrix<double>
 Stress::calc_stress_vloc()
 {
@@ -733,28 +707,30 @@ Stress::calc_stress_vloc()
 
     stress_vloc_.zero();
 
-    auto q          = ctx_.gvec().shells_len();
-    auto ri_vloc    = ctx_.vloc_ri().values(q, ctx_.comm());
-    auto ri_vloc_dg = ctx_.vloc_ri_djl().values(q, ctx_.comm());
+    auto q                = ctx_.gvec().shells_len();
+    auto const ri_vloc    = ctx_.ri().vloc_->values(q, ctx_.comm());
+    auto const ri_vloc_dg = ctx_.ri().vloc_djl_->values(q, ctx_.comm());
 
-    auto v  = ctx_.make_periodic_function<sddk::index_domain_t::local>(ri_vloc);
-    auto dv = ctx_.make_periodic_function<sddk::index_domain_t::local>(ri_vloc_dg);
+    auto v  = make_periodic_function<index_domain_t::local>(ctx_.unit_cell(), ctx_.gvec(), ctx_.phase_factors_t(),
+                                                           ri_vloc);
+    auto dv = make_periodic_function<index_domain_t::local>(ctx_.unit_cell(), ctx_.gvec(), ctx_.phase_factors_t(),
+                                                            ri_vloc_dg);
 
     double sdiag{0};
 
     int ig0 = ctx_.gvec().skip_g0();
     for (int igloc = ig0; igloc < ctx_.gvec().count(); igloc++) {
 
-        auto G = ctx_.gvec().gvec_cart<sddk::index_domain_t::local>(igloc);
+        auto G = ctx_.gvec().gvec_cart<index_domain_t::local>(igloc);
 
         for (int mu : {0, 1, 2}) {
             for (int nu : {0, 1, 2}) {
                 stress_vloc_(mu, nu) +=
-                    std::real(std::conj(density_.rho().f_pw_local(igloc)) * dv[igloc]) * G[mu] * G[nu];
+                        std::real(std::conj(density_.rho().rg().f_pw_local(igloc)) * dv[igloc]) * G[mu] * G[nu];
             }
         }
 
-        sdiag += std::real(std::conj(density_.rho().f_pw_local(igloc)) * v[igloc]);
+        sdiag += std::real(std::conj(density_.rho().rg().f_pw_local(igloc)) * v[igloc]);
     }
 
     if (ctx_.gvec().reduced()) {
@@ -762,7 +738,7 @@ Stress::calc_stress_vloc()
         sdiag *= 2;
     }
     if (ctx_.comm().rank() == 0) {
-        sdiag += std::real(std::conj(density_.rho().f_pw_local(0)) * v[0]);
+        sdiag += std::real(std::conj(density_.rho().rg().f_pw_local(0)) * v[0]);
     }
 
     for (int mu : {0, 1, 2}) {
@@ -771,7 +747,7 @@ Stress::calc_stress_vloc()
 
     ctx_.comm().allreduce(&stress_vloc_(0, 0), 9);
 
-    symmetrize(stress_vloc_);
+    symmetrize_stress_tensor(ctx_.unit_cell().symmetry(), stress_vloc_);
 
     return stress_vloc_;
 }
@@ -780,7 +756,7 @@ r3::matrix<double>
 Stress::calc_stress_nonloc()
 {
     if (ctx_.cfg().parameters().precision_wf() == "fp32") {
-#if defined(USE_FP32)
+#if defined(SIRIUS_USE_FP32)
         if (ctx_.gamma_point()) {
             calc_stress_nonloc_aux<float, float>();
         } else {
