@@ -40,7 +40,7 @@ Hamiltonian0<T>::Hamiltonian0(Potential& potential__, bool precompute_lapw__)
     PROFILE("sirius::Hamiltonian0");
 
     local_op_ = std::unique_ptr<Local_operator<T>>(
-        new Local_operator<T>(ctx_, ctx_.spfft_coarse<T>(), ctx_.gvec_coarse_fft_sptr(), &potential__));
+            new Local_operator<T>(ctx_, ctx_.spfft_coarse<T>(), ctx_.gvec_coarse_fft_sptr(), &potential__));
 
     if (!ctx_.full_potential()) {
         d_op_ = std::unique_ptr<D_operator<T>>(new D_operator<T>(ctx_));
@@ -53,7 +53,7 @@ Hamiltonian0<T>::Hamiltonian0(Potential& potential__, bool precompute_lapw__)
             ctx_.unit_cell().generate_radial_functions(ctx_.out());
             ctx_.unit_cell().generate_radial_integrals();
         }
-        hmt_ = std::vector<sddk::mdarray<std::complex<T>, 2>>(ctx_.unit_cell().num_atoms());
+        hmt_    = std::vector<mdarray<std::complex<T>, 2>>(ctx_.unit_cell().num_atoms());
         auto pu = ctx_.processing_unit();
         #pragma omp parallel
         {
@@ -65,26 +65,26 @@ Hamiltonian0<T>::Hamiltonian0(Potential& potential__, bool precompute_lapw__)
 
                 int nmt = type.mt_basis_size();
 
-                hmt_[ia] = sddk::mdarray<std::complex<T>, 2>(nmt, nmt, sddk::memory_t::host, "hmt");
+                hmt_[ia] = mdarray<std::complex<T>, 2>({nmt, nmt}, mdarray_label("hmt"));
 
                 /* compute muffin-tin Hamiltonian */
                 for (int j2 = 0; j2 < nmt; j2++) {
                     int lm2    = type.indexb(j2).lm;
                     int idxrf2 = type.indexb(j2).idxrf;
                     for (int j1 = 0; j1 <= j2; j1++) {
-                        int lm1    = type.indexb(j1).lm;
-                        int idxrf1 = type.indexb(j1).idxrf;
-                        hmt_[ia](j1, j2) = atom.radial_integrals_sum_L3<spin_block_t::nm>(idxrf1, idxrf2,
-                                                                                type.gaunt_coefs().gaunt_vector(lm1, lm2));
+                        int lm1          = type.indexb(j1).lm;
+                        int idxrf1       = type.indexb(j1).idxrf;
+                        hmt_[ia](j1, j2) = atom.radial_integrals_sum_L3<spin_block_t::nm>(
+                                idxrf1, idxrf2, type.gaunt_coefs().gaunt_vector(lm1, lm2));
                         hmt_[ia](j2, j1) = std::conj(hmt_[ia](j1, j2));
                     }
                 }
-                if (pu == sddk::device_t::GPU) {
-                    hmt_[ia].allocate(sddk::memory_t::device).copy_to(sddk::memory_t::device, stream_id(tid));
+                if (pu == device_t::GPU) {
+                    hmt_[ia].allocate(memory_t::device).copy_to(memory_t::device, acc::stream_id(tid));
                 }
             }
-            if (pu == sddk::device_t::GPU) {
-                acc::sync_stream(stream_id(tid));
+            if (pu == device_t::GPU) {
+                acc::sync_stream(acc::stream_id(tid));
             }
         }
     }
@@ -95,17 +95,18 @@ Hamiltonian0<T>::~Hamiltonian0()
 {
 }
 
-template <typename T> template <spin_block_t sblock>
+template <typename T>
+template <spin_block_t sblock>
 void
-Hamiltonian0<T>::apply_hmt_to_apw(Atom const& atom__, int ngv__, sddk::mdarray<std::complex<T>, 2>& alm__,
-                               sddk::mdarray<std::complex<T>, 2>& halm__) const
+Hamiltonian0<T>::apply_hmt_to_apw(Atom const& atom__, int ngv__, mdarray<std::complex<T>, 2>& alm__,
+                                  mdarray<std::complex<T>, 2>& halm__) const
 {
     auto& type = atom__.type();
 
     // TODO: this is k-independent and can in principle be precomputed together with radial integrals if memory is
     // available
     // TODO: for spin-collinear case hmt is Hermitian; compute upper triangular part and use zhemm
-    sddk::mdarray<std::complex<T>, 2> hmt(type.mt_aw_basis_size(), type.mt_aw_basis_size());
+    mdarray<std::complex<T>, 2> hmt({type.mt_aw_basis_size(), type.mt_aw_basis_size()});
     /* compute the muffin-tin Hamiltonian */
     for (int j2 = 0; j2 < type.mt_aw_basis_size(); j2++) {
         int lm2    = type.indexb(j2).lm;
@@ -113,19 +114,19 @@ Hamiltonian0<T>::apply_hmt_to_apw(Atom const& atom__, int ngv__, sddk::mdarray<s
         for (int j1 = 0; j1 < type.mt_aw_basis_size(); j1++) {
             int lm1    = type.indexb(j1).lm;
             int idxrf1 = type.indexb(j1).idxrf;
-            hmt(j1, j2) = atom__.radial_integrals_sum_L3<sblock>(idxrf1, idxrf2,
-                                                                 type.gaunt_coefs().gaunt_vector(lm1, lm2));
+            hmt(j1, j2) =
+                    atom__.radial_integrals_sum_L3<sblock>(idxrf1, idxrf2, type.gaunt_coefs().gaunt_vector(lm1, lm2));
         }
     }
     la::wrap(la::lib_t::blas)
-        .gemm('N', 'T', ngv__, type.mt_aw_basis_size(), type.mt_aw_basis_size(), &la::constant<std::complex<T>>::one(),
-              alm__.at(sddk::memory_t::host), alm__.ld(), hmt.at(sddk::memory_t::host), hmt.ld(),
-              &la::constant<std::complex<T>>::zero(), halm__.at(sddk::memory_t::host), halm__.ld());
+            .gemm('N', 'T', ngv__, type.mt_aw_basis_size(), type.mt_aw_basis_size(),
+                  &la::constant<std::complex<T>>::one(), alm__.at(memory_t::host), alm__.ld(), hmt.at(memory_t::host),
+                  hmt.ld(), &la::constant<std::complex<T>>::zero(), halm__.at(memory_t::host), halm__.ld());
 }
 
 template <typename T>
 void
-Hamiltonian0<T>::add_o1mt_to_apw(Atom const& atom__, int num_gkvec__, sddk::mdarray<std::complex<T>, 2>& alm__) const
+Hamiltonian0<T>::add_o1mt_to_apw(Atom const& atom__, int num_gkvec__, mdarray<std::complex<T>, 2>& alm__) const
 {
     // TODO: optimize for the loop layout using blocks of G-vectors
     auto& type = atom__.type();
@@ -136,12 +137,12 @@ Hamiltonian0<T>::add_o1mt_to_apw(Atom const& atom__, int num_gkvec__, sddk::mdar
             alm[j] = oalm[j] = alm__(ig, j);
         }
         for (int j = 0; j < type.mt_aw_basis_size(); j++) {
-            int l     = type.indexb(j).l;
+            int l     = type.indexb(j).am.l();
             int lm    = type.indexb(j).lm;
             int idxrf = type.indexb(j).idxrf;
             for (int order = 0; order < type.aw_order(l); order++) {
                 int j1     = type.indexb().index_by_lm_order(lm, order);
-                int idxrf1 = type.indexr().index_by_l_order(l, order);
+                int idxrf1 = type.indexr().index_of(angular_momentum(l), order);
                 oalm[j] += static_cast<const T>(atom__.symmetry_class().o1_radial_integral(idxrf, idxrf1)) * alm[j1];
             }
         }
@@ -155,12 +156,12 @@ template <typename T>
 void
 Hamiltonian0<T>::apply_bmt(wf::Wave_functions<T>& psi__, std::vector<wf::Wave_functions<T>>& bpsi__) const
 {
-    sddk::mdarray<std::complex<T>, 3> zm(unit_cell_.max_mt_basis_size(), unit_cell_.max_mt_basis_size(), ctx_.num_mag_dims());
+    mdarray<std::complex<T>, 3> zm(
+            {unit_cell_.max_mt_basis_size(), unit_cell_.max_mt_basis_size(), ctx_.num_mag_dims()});
 
-    for (int ialoc = 0; ialoc < psi__.spl_num_atoms().local_size(); ialoc++) {
-        int ia            = psi__.spl_num_atoms()[ialoc];
+    for (auto it : psi__.spl_num_atoms()) {
+        auto ia           = it.i;
         auto& atom        = unit_cell_.atom(ia);
-        //int offset        = psi__.offset_mt_coeffs(ialoc);
         int mt_basis_size = atom.type().mt_basis_size();
 
         zm.zero();
@@ -176,16 +177,17 @@ Hamiltonian0<T>::apply_bmt(wf::Wave_functions<T>& psi__, std::vector<wf::Wave_fu
                     int lm1    = atom.type().indexb(xi1).lm;
                     int idxrf1 = atom.type().indexb(xi1).idxrf;
 
-                    zm(xi1, xi2, i) = atom.type().gaunt_coefs().sum_L3_gaunt(lm1, lm2, atom.b_radial_integrals(idxrf1, idxrf2, i));
+                    zm(xi1, xi2, i) = atom.type().gaunt_coefs().sum_L3_gaunt(
+                            lm1, lm2, atom.b_radial_integrals(idxrf1, idxrf2, i));
                 }
             }
         }
         /* compute bwf = B_z*|wf_j> */
-        la::wrap(la::lib_t::blas).hemm(
-            'L', 'U', mt_basis_size, ctx_.num_fv_states(), &la::constant<std::complex<T>>::one(),
-            zm.at(sddk::memory_t::host), zm.ld(), &psi__.mt_coeffs(0, wf::atom_index(ialoc), wf::spin_index(0), wf::band_index(0)),
-            psi__.ld(), &la::constant<std::complex<T>>::zero(),
-            &bpsi__[0].mt_coeffs(0, wf::atom_index(ialoc), wf::spin_index(0), wf::band_index(0)), bpsi__[0].ld());
+        la::wrap(la::lib_t::blas)
+                .hemm('L', 'U', mt_basis_size, ctx_.num_fv_states(), &la::constant<std::complex<T>>::one(),
+                      zm.at(memory_t::host), zm.ld(), &psi__.mt_coeffs(0, it.li, wf::spin_index(0), wf::band_index(0)),
+                      psi__.ld(), &la::constant<std::complex<T>>::zero(),
+                      &bpsi__[0].mt_coeffs(0, it.li, wf::spin_index(0), wf::band_index(0)), bpsi__[0].ld());
 
         /* compute bwf = (B_x - iB_y)|wf_j> */
         if (bpsi__.size() == 3) {
@@ -202,11 +204,12 @@ Hamiltonian0<T>::apply_bmt(wf::Wave_functions<T>& psi__, std::vector<wf::Wave_fu
                 }
             }
 
-            la::wrap(la::lib_t::blas).gemm(
-               'N', 'N', mt_basis_size, ctx_.num_fv_states(), mt_basis_size, &la::constant<std::complex<T>>::one(),
-               zm.at(sddk::memory_t::host), zm.ld(), &psi__.mt_coeffs(0, wf::atom_index(ialoc), wf::spin_index(0), wf::band_index(0)),
-               psi__.ld(), &la::constant<std::complex<T>>::zero(),
-               &bpsi__[2].mt_coeffs(0, wf::atom_index(ialoc), wf::spin_index(0), wf::band_index(0)), bpsi__[2].ld());
+            la::wrap(la::lib_t::blas)
+                    .gemm('N', 'N', mt_basis_size, ctx_.num_fv_states(), mt_basis_size,
+                          &la::constant<std::complex<T>>::one(), zm.at(memory_t::host), zm.ld(),
+                          &psi__.mt_coeffs(0, it.li, wf::spin_index(0), wf::band_index(0)), psi__.ld(),
+                          &la::constant<std::complex<T>>::zero(),
+                          &bpsi__[2].mt_coeffs(0, it.li, wf::spin_index(0), wf::band_index(0)), bpsi__[2].ld());
         }
     }
 }
@@ -219,14 +222,14 @@ Hamiltonian0<T>::apply_so_correction(wf::Wave_functions<T>& psi__, std::vector<w
 
     wf::spin_index s(0);
 
-    for (int ialoc = 0; ialoc < psi__.spl_num_atoms().local_size(); ialoc++) {
-        int ia     = psi__.spl_num_atoms()[ialoc];
+    for (auto it : psi__.spl_num_atoms()) {
+        auto ia    = it.i;
         auto& atom = unit_cell_.atom(ia);
-        wf::atom_index a(ialoc);
+        auto a     = it.li;
 
         for (int l = 0; l <= atom.type().lmax_apw(); l++) {
             /* number of radial functions for this l */
-            int nrf = atom.type().indexr().num_rf(l);
+            int nrf = atom.type().indexr().max_order(l);
 
             for (int order1 = 0; order1 < nrf; order1++) {
                 for (int order2 = 0; order2 < nrf; order2++) {
@@ -247,8 +250,8 @@ Hamiltonian0<T>::apply_so_correction(wf::Wave_functions<T>& psi__, std::vector<w
                             hpsi__[1].mt_coeffs(idx1, a, s, b) -= z1;
                             /* apply L_{-} operator; u-d part */
                             if (m + l) {
-                                hpsi__[2].mt_coeffs(idx1, a, s, b) += psi__.mt_coeffs(idx3, a, s, b) * sori *
-                                    std::sqrt(T(l * (l + 1) - m * (m - 1)));
+                                hpsi__[2].mt_coeffs(idx1, a, s, b) +=
+                                        psi__.mt_coeffs(idx3, a, s, b) * sori * std::sqrt(T(l * (l + 1) - m * (m - 1)));
                             }
                             /* for the d-u part */
                             ///* apply L_{+} operator */
@@ -267,18 +270,18 @@ Hamiltonian0<T>::apply_so_correction(wf::Wave_functions<T>& psi__, std::vector<w
 
 template class Hamiltonian0<double>;
 
-template
-void
-Hamiltonian0<double>::apply_hmt_to_apw<spin_block_t::nm>(Atom const& atom__, int ngv__, sddk::mdarray<std::complex<double>, 2>& alm__,
-                                                         sddk::mdarray<std::complex<double>, 2>& halm__) const;
+template void
+Hamiltonian0<double>::apply_hmt_to_apw<spin_block_t::nm>(Atom const& atom__, int ngv__,
+                                                         mdarray<std::complex<double>, 2>& alm__,
+                                                         mdarray<std::complex<double>, 2>& halm__) const;
 
-#ifdef USE_FP32
+#ifdef SIRIUS_USE_FP32
 template class Hamiltonian0<float>;
 
-template
-void
-Hamiltonian0<float>::apply_hmt_to_apw<spin_block_t::nm>(Atom const& atom__, int ngv__, sddk::mdarray<std::complex<float>, 2>& alm__,
-                                                        sddk::mdarray<std::complex<float>, 2>& halm__) const;
+template void
+Hamiltonian0<float>::apply_hmt_to_apw<spin_block_t::nm>(Atom const& atom__, int ngv__,
+                                                        mdarray<std::complex<float>, 2>& alm__,
+                                                        mdarray<std::complex<float>, 2>& halm__) const;
 #endif
 
-} // namespace
+} // namespace sirius
