@@ -27,12 +27,10 @@
 
 #include <memory>
 #include <complex>
-#include "SDDK/memory.hpp"
-#include "SDDK/type_definition.hpp"
-#include "SDDK/wave_functions.hpp"
-#include "linalg/dmatrix.hpp"
-#include "typedefs.hpp"
-#include "fft/fft.hpp"
+#include "core/wf/wave_functions.hpp"
+#include "core/typedefs.hpp"
+#include "core/fft/fft.hpp"
+#include "core/la/dmatrix.hpp"
 #include "local_operator.hpp"
 #include "non_local_operator.hpp"
 
@@ -93,12 +91,13 @@ class Hamiltonian0
     /// Q operator (non-local part of S-operator).
     std::unique_ptr<Q_operator<T>> q_op_;
 
-    std::vector<sddk::mdarray<std::complex<T>, 2>> hmt_;
+    std::vector<mdarray<std::complex<T>, 2>> hmt_;
 
     /* copy constructor is forbidden */
     Hamiltonian0(Hamiltonian0<T> const& src) = delete;
     /* copy assignment operator is forbidden */
-    Hamiltonian0<T>& operator=(Hamiltonian0<T> const& src) = delete;
+    Hamiltonian0<T>&
+    operator=(Hamiltonian0<T> const& src) = delete;
 
   public:
     /// Constructor.
@@ -110,34 +109,41 @@ class Hamiltonian0
     Hamiltonian0(Hamiltonian0<T>&& src) = default;
 
     /// Return a Hamiltonian for the given k-point.
-    inline Hamiltonian_k<T> operator()(K_point<T>& kp__);
+    inline Hamiltonian_k<T>
+    operator()(K_point<T>& kp__) const;
 
-    Simulation_context& ctx() const
+    Simulation_context&
+    ctx() const
     {
         return ctx_;
     }
 
-    Potential& potential() const
+    Potential&
+    potential() const
     {
         return *potential_;
     }
 
-    Local_operator<T>& local_op() const
+    Local_operator<T>&
+    local_op() const
     {
         return *local_op_;
     }
 
-    inline Q_operator<T>& Q() const
+    inline Q_operator<T>&
+    Q() const
     {
         return *q_op_;
     }
 
-    inline D_operator<T>& D() const
+    inline D_operator<T>&
+    D() const
     {
         return *d_op_;
     }
 
-    auto const& hmt(int ia__) const
+    auto const&
+    hmt(int ia__) const
     {
         return hmt_[ia__];
     }
@@ -152,15 +158,18 @@ class Hamiltonian0
      *  \f]
      */
     template <spin_block_t sblock>
-    void apply_hmt_to_apw(Atom const& atom__, int ngv__, sddk::mdarray<std::complex<T>, 2>& alm__,
-                          sddk::mdarray<std::complex<T>, 2>& halm__) const;
+    void
+    apply_hmt_to_apw(Atom const& atom__, int ngv__, mdarray<std::complex<T>, 2>& alm__,
+                     mdarray<std::complex<T>, 2>& halm__) const;
 
     /// Add correction to LAPW overlap arising in the infinite-order relativistic approximation (IORA).
-    void add_o1mt_to_apw(Atom const& atom__, int num_gkvec__,
-                         sddk::mdarray<std::complex<T>, 2>& alm__) const; // TODO: documentation
+    void
+    add_o1mt_to_apw(Atom const& atom__, int num_gkvec__,
+                    mdarray<std::complex<T>, 2>& alm__) const; // TODO: documentation
 
     /// Apply muffin-tin part of magnetic filed to the wave-functions.
-    void apply_bmt(wf::Wave_functions<T>& psi__, std::vector<wf::Wave_functions<T>>& bpsi__) const;
+    void
+    apply_bmt(wf::Wave_functions<T>& psi__, std::vector<wf::Wave_functions<T>>& bpsi__) const;
 
     /// Apply SO correction to the first-variational LAPW wave-functions.
     /** Raising and lowering operators:
@@ -168,14 +177,16 @@ class Hamiltonian0
      *      L_{\pm} Y_{\ell m}= (L_x \pm i L_y) Y_{\ell m}  = \sqrt{\ell(\ell+1) - m(m \pm 1)} Y_{\ell m \pm 1}
      *  \f]
      */
-    void apply_so_correction(wf::Wave_functions<T>& psi__, std::vector<wf::Wave_functions<T>>& hpsi__) const;
+    void
+    apply_so_correction(wf::Wave_functions<T>& psi__, std::vector<wf::Wave_functions<T>>& hpsi__) const;
 };
 
 template <typename T>
 class Hamiltonian_k
 {
   private:
-    Hamiltonian0<T>& H0_;
+    /// K-point independent part of Hamiltonian.
+    Hamiltonian0<T> const& H0_;
     K_point<T>& kp_;
     /// Hubbard correction.
     /** In general case it is a k-dependent matrix */
@@ -185,37 +196,32 @@ class Hamiltonian_k
     Hamiltonian_k(Hamiltonian_k<T> const& src__) = delete;
 
     /// Assignment operator is forbidden.
-    Hamiltonian_k<T>& operator=(Hamiltonian_k<T> const& src__) = delete;
+    Hamiltonian_k<T>&
+    operator=(Hamiltonian_k<T> const& src__) = delete;
 
   public:
-    Hamiltonian_k(Hamiltonian0<T>& H0__, K_point<T>& kp__);
+    Hamiltonian_k(Hamiltonian0<T> const& H0__, K_point<T>& kp__);
 
     Hamiltonian_k(Hamiltonian_k<T>&& src__);
 
     ~Hamiltonian_k();
 
-    Hamiltonian0<T> const& H0() const
+    Hamiltonian0<T> const&
+    H0() const
     {
         return H0_;
     }
 
-    auto& kp()
-    {
-        return kp_;
-    }
-
-    auto const& kp() const
-    {
-        return kp_;
-    }
-
     template <typename F, int what>
-    std::pair<sddk::mdarray<T, 2>, sddk::mdarray<T, 2>> get_h_o_diag_pw() const;
+    std::pair<mdarray<T, 2>, mdarray<T, 2>>
+    get_h_o_diag_pw() const;
 
     template <int what>
-    std::pair<sddk::mdarray<T, 2>, sddk::mdarray<T, 2>> get_h_o_diag_lapw() const;
+    std::pair<mdarray<T, 2>, mdarray<T, 2>>
+    get_h_o_diag_lapw() const;
 
-    auto& U()
+    auto
+    U() const -> U_operator<T> const&
     {
         return *u_op_;
     }
@@ -345,17 +351,15 @@ class Hamiltonian_k
      *
      *  \param [in]  apw_only   True if only APW-APW block of H and O are applied.
      *  \param [in]  phi_is_lo  True if input wave-functions are pure local orbitals.
-     *  \param [in]  N          Starting index of wave-functions.
-     *  \param [in]  n          Number of wave-functions to which H and S are applied.
+     *  \param [in]  b          Range of bands.
      *  \param [in]  phi        Input wave-functions.
      *  \param [out] hphi       Result of Hamiltonian, applied to wave-functions.
      *  \param [out] ophi       Result of overlap operator, applied to wave-functions.
      */
-    //void apply_fv_h_o(bool apw_only__, bool phi_is_lo__, int N__, int n__, sddk::Wave_functions<T>& phi__,
-    //                  sddk::Wave_functions<T>* hphi__, sddk::Wave_functions<T>* ophi__);
+    void
+    apply_fv_h_o(bool apw_only__, bool phi_is_lo__, wf::band_range b__, wf::Wave_functions<T>& phi__,
+                 wf::Wave_functions<T>* hphi__, wf::Wave_functions<T>* ophi__) const;
 
-    void apply_fv_h_o(bool apw_only__, bool phi_is_lo__, wf::band_range b__, wf::Wave_functions<T>& phi__,
-                      wf::Wave_functions<T>* hphi__, wf::Wave_functions<T>* ophi__);
     /// Setup the Hamiltonian and overlap matrices in APW+lo basis
     /** The Hamiltonian matrix has the following expression:
      *  \f[
@@ -423,23 +427,27 @@ class Hamiltonian_k
      *      \delta_{\ell_{j'} \ell_j} \delta_{m_{j'} m_j}
      *  \f]
      */
-    void set_fv_h_o(la::dmatrix<std::complex<T>>& h__, la::dmatrix<std::complex<T>>& o__) const;
+    void
+    set_fv_h_o(la::dmatrix<std::complex<T>>& h__, la::dmatrix<std::complex<T>>& o__) const;
 
     /// Add interstitial contribution to apw-apw block of Hamiltonian and overlap.
-    void set_fv_h_o_it(la::dmatrix<std::complex<T>>& h__, la::dmatrix<std::complex<T>>& o__) const;
+    void
+    set_fv_h_o_it(la::dmatrix<std::complex<T>>& h__, la::dmatrix<std::complex<T>>& o__) const;
 
     /// Setup lo-lo block of Hamiltonian and overlap matrices.
-    void set_fv_h_o_lo_lo(la::dmatrix<std::complex<T>>& h__, la::dmatrix<std::complex<T>>& o__) const;
+    void
+    set_fv_h_o_lo_lo(la::dmatrix<std::complex<T>>& h__, la::dmatrix<std::complex<T>>& o__) const;
 
     /// Setup apw-lo and lo-apw blocks of LAPW Hamiltonian and overlap matrices.
-    void set_fv_h_o_apw_lo(Atom const& atom, int ia, sddk::mdarray<std::complex<T>, 2>& alm_row,
-                           sddk::mdarray<std::complex<T>, 2>& alm_col, sddk::mdarray<std::complex<T>, 2>& h,
-                           sddk::mdarray<std::complex<T>, 2>& o) const;
+    void
+    set_fv_h_o_apw_lo(Atom const& atom, int ia, mdarray<std::complex<T>, 2>& alm_row,
+                      mdarray<std::complex<T>, 2>& alm_col, mdarray<std::complex<T>, 2>& h,
+                      mdarray<std::complex<T>, 2>& o) const;
 
     /// Apply pseudopotential H and S operators to the wavefunctions.
-    /** \param [in]  spins Spin index range
-     *  \param [in]  N     Starting index of wave-functions.
-     *  \param [in]  n     Number of wave-functions to which H and S are applied.
+    /** \tparam F    Type of the subspace matrix.
+     *  \param [in]  spins Range of spins.
+     *  \param [in]  br    Range of bands.
      *  \param [in]  phi   Input wave-functions [storage: CPU && GPU].
      *  \param [out] hphi  Result of Hamiltonian, applied to wave-functions [storage: CPU || GPU].
      *  \param [out] sphi  Result of S-operator, applied to wave-functions [storage: CPU || GPU].
@@ -447,16 +455,10 @@ class Hamiltonian_k
      *  In non-collinear case (spins in [0,1]) the Hamiltonian and S operator are applied to both components of spinor
      *  wave-functions. Otherwise they are applied to a single component.
      */
-    //template <typename F, typename = std::enable_if_t<std::is_same<T, real_type<F>>::value>>
-    //void apply_h_s(sddk::spin_range spins__, int N__, int n__, sddk::Wave_functions<T>& phi__,
-    //               sddk::Wave_functions<T>* hphi__, sddk::Wave_functions<T>* sphi__);
-
-    /** \tparam F  Type of the subspace matrix.
-     */
     template <typename F>
     std::enable_if_t<std::is_same<T, real_type<F>>::value, void>
     apply_h_s(wf::spin_range spins__, wf::band_range br__, wf::Wave_functions<T> const& phi__,
-              wf::Wave_functions<T>* hphi__, wf::Wave_functions<T>* sphi__)
+              wf::Wave_functions<T>* hphi__, wf::Wave_functions<T>* sphi__) const
     {
         PROFILE("sirius::Hamiltonian_k::apply_h_s");
 
@@ -464,48 +466,51 @@ class Hamiltonian_k
 
         if (hphi__ != nullptr) {
             /* apply local part of Hamiltonian */
-            H0().local_op().apply_h(reinterpret_cast<fft::spfft_transform_type<T>&>(kp().spfft_transform()),
-                                    kp().gkvec_fft_sptr(), spins__, phi__, *hphi__, br__);
+            H0().local_op().apply_h(reinterpret_cast<fft::spfft_transform_type<T>&>(kp_.spfft_transform()),
+                                    kp_.gkvec_fft_sptr(), spins__, phi__, *hphi__, br__);
         }
 
         auto mem = H0().ctx().processing_unit_memory_t();
 
         if (pcs) {
             auto cs = phi__.checksum(mem, br__);
-            utils::print_checksum("phi", cs, RTE_OUT(H0().ctx().out()));
+            print_checksum("phi", cs, RTE_OUT(H0().ctx().out()));
             if (hphi__) {
                 auto cs = hphi__->checksum(mem, br__);
-                utils::print_checksum("hloc_phi", cs, RTE_OUT(H0().ctx().out()));
+                print_checksum("hloc_phi", cs, RTE_OUT(H0().ctx().out()));
             }
         }
 
         /* set initial sphi */
         if (sphi__ != nullptr) {
-            for (auto s = spins__.begin(); s!= spins__.end(); s++) {
+            for (auto s = spins__.begin(); s != spins__.end(); s++) {
                 auto sp = phi__.actual_spin_index(s);
                 wf::copy(mem, phi__, sp, br__, *sphi__, sp, br__);
             }
         }
 
         /* return if there are no beta-projectors */
-        if (H0().ctx().unit_cell().mt_lo_basis_size()) {
-            apply_non_local_D_Q<T, F>(mem, spins__, br__, kp().beta_projectors(), phi__, &H0().D(), hphi__, &H0().Q(), sphi__);
+        if (H0().ctx().unit_cell().max_mt_basis_size()) {
+            auto bp_generator = kp_.beta_projectors().make_generator();
+            auto beta_coeffs  = bp_generator.prepare();
+            apply_non_local_D_Q<T, F>(mem, spins__, br__, bp_generator, beta_coeffs, phi__, &H0().D(), hphi__,
+                                      &H0().Q(), sphi__);
         }
 
         /* apply the hubbard potential if relevant */
         if (H0().ctx().hubbard_correction() && !H0().ctx().gamma_point() && hphi__) {
             /* apply the hubbard potential */
-            apply_U_operator(H0().ctx(), spins__, br__, kp().hubbard_wave_functions_S(), phi__, this->U(), *hphi__);
+            apply_U_operator(H0().ctx(), spins__, br__, kp_.hubbard_wave_functions_S(), phi__, this->U(), *hphi__);
         }
 
         if (pcs) {
             if (hphi__) {
                 auto cs = hphi__->checksum(mem, br__);
-                utils::print_checksum("hphi", cs, RTE_OUT(H0().ctx().out()));
+                print_checksum("hphi", cs, RTE_OUT(H0().ctx().out()));
             }
             if (sphi__) {
                 auto cs = sphi__->checksum(mem, br__);
-                utils::print_checksum("hsphi", cs, RTE_OUT(H0().ctx().out()));
+                print_checksum("hsphi", cs, RTE_OUT(H0().ctx().out()));
             }
         }
     }
@@ -513,22 +518,44 @@ class Hamiltonian_k
     template <typename F>
     std::enable_if_t<!std::is_same<T, real_type<F>>::value, void>
     apply_h_s(wf::spin_range spins__, wf::band_range br__, wf::Wave_functions<T> const& phi__,
-              wf::Wave_functions<T>* hphi__, wf::Wave_functions<T>* sphi__)
+              wf::Wave_functions<T>* hphi__, wf::Wave_functions<T>* sphi__) const
     {
-        RTE_THROW("implementat this");
+        RTE_THROW("implement this");
+    }
+
+    /// apply S operator
+    /** \tparam F    Type of the subspace matrix.
+     *  \param [in]  spins Range of spins.
+     *  \param [in]  br    Range of bands.
+     *  \param [in]  phi   Input wave-functions [storage: CPU && GPU].
+     *  \param [out] sphi  Result of S-operator, applied to wave-functions [storage: CPU || GPU].
+     *
+     *  In non-collinear case (spins in [0,1]) the S operator is applied to both components of spinor
+     *  wave-functions. Otherwise they are applied to a single component.
+     */
+    template <typename F>
+    std::enable_if_t<std::is_same<T, real_type<F>>::value, void>
+    apply_s(wf::spin_range spin__, wf::band_range br__, wf::Wave_functions<T> const& phi__,
+            wf::Wave_functions<T>& sphi__) const
+    {
+        auto mem       = H0().ctx().processing_unit_memory_t();
+        auto bp_gen    = kp_.beta_projectors().make_generator();
+        auto bp_coeffs = bp_gen.prepare();
+        apply_S_operator<T, F>(mem, spin__, br__, bp_gen, bp_coeffs, phi__, &H0().Q(), sphi__);
     }
 
     /// Apply magnetic field to first-variational LAPW wave-functions.
-    void apply_b(wf::Wave_functions<T>& psi__, std::vector<wf::Wave_functions<T>>& bpsi__);
+    void
+    apply_b(wf::Wave_functions<T>& psi__, std::vector<wf::Wave_functions<T>>& bpsi__) const;
 };
 
 template <typename T>
 Hamiltonian_k<T>
-Hamiltonian0<T>::operator()(K_point<T>& kp__)
+Hamiltonian0<T>::operator()(K_point<T>& kp__) const
 {
     return Hamiltonian_k<T>(*this, kp__);
 }
 
-}
+} // namespace sirius
 
 #endif
