@@ -23,18 +23,20 @@
  */
 
 #include "potential.hpp"
-#include "utils/profiler.hpp"
+#include "core/profiler.hpp"
+#include "lapw/sum_fg_fl_yg.hpp"
 
 namespace sirius {
 
-double density_residual_hartree_energy(Density const& rho1__, Density const& rho2__)
+double
+density_residual_hartree_energy(Density const& rho1__, Density const& rho2__)
 {
     double eh{0};
     auto const& gv = rho1__.ctx().gvec();
     #pragma omp parallel for reduction(+:eh)
     for (int igloc = gv.skip_g0(); igloc < gv.count(); igloc++) {
-        auto z = rho1__.component(0).f_pw_local(igloc) - rho2__.component(0).f_pw_local(igloc);
-        double g = gv.gvec_len<sddk::index_domain_t::local>(igloc);
+        auto z   = rho1__.component(0).rg().f_pw_local(igloc) - rho2__.component(0).rg().f_pw_local(igloc);
+        double g = gv.gvec_len<index_domain_t::local>(igloc);
         eh += (std::pow(z.real(), 2) + std::pow(z.imag(), 2)) / std::pow(g, 2);
     }
     gv.comm().allreduce(&eh, 1);
@@ -45,14 +47,14 @@ double density_residual_hartree_energy(Density const& rho1__, Density const& rho
     return eh;
 }
 
-void Potential::poisson_add_pseudo_pw(sddk::mdarray<std::complex<double>, 2>& qmt__,
-                                      sddk::mdarray<std::complex<double>, 2>& qit__,
-                                      std::complex<double>* rho_pw__)
+void
+Potential::poisson_add_pseudo_pw(mdarray<std::complex<double>, 2>& qmt__, mdarray<std::complex<double>, 2>& qit__,
+                                 std::complex<double>* rho_pw__)
 {
     PROFILE("sirius::Potential::poisson_add_pseudo_pw");
 
     int lmmax = ctx_.lmmax_rho();
-    int ngv = ctx_.gvec().count();
+    int ngv   = ctx_.gvec().count();
 
     /* The following term is added to the plane-wave coefficients of the charge density:
      * Integrate[SphericalBesselJ[l,a*x]*p[x,R]*x^2,{x,0,R},Assumptions->{l>=0,n>=0,R>0,a>0}] /
@@ -63,31 +65,31 @@ void Potential::poisson_add_pseudo_pw(sddk::mdarray<std::complex<double>, 2>& qm
      */
     for (int iat = 0; iat < unit_cell_.num_atom_types(); iat++) {
         double R = unit_cell_.atom_type(iat).mt_radius();
-        int na = unit_cell_.atom_type(iat).num_atoms();
+        int na   = unit_cell_.atom_type(iat).num_atoms();
 
-        sddk::mdarray<std::complex<double>, 2> pf;
-        sddk::mdarray<std::complex<double>, 2> qa;
-        sddk::mdarray<std::complex<double>, 2> qapf;
+        mdarray<std::complex<double>, 2> pf;
+        mdarray<std::complex<double>, 2> qa;
+        mdarray<std::complex<double>, 2> qapf;
 
         switch (ctx_.processing_unit()) {
-            case sddk::device_t::CPU: {
-                auto& mp = get_memory_pool(sddk::memory_t::host);
-                pf = sddk::mdarray<std::complex<double>, 2>(ngv, na, mp);
-                qa = sddk::mdarray<std::complex<double>, 2>(lmmax, na, mp);
-                qapf = sddk::mdarray<std::complex<double>, 2>(lmmax, ngv, mp);
+            case device_t::CPU: {
+                auto& mp = get_memory_pool(memory_t::host);
+                pf       = mdarray<std::complex<double>, 2>({ngv, na}, mp);
+                qa       = mdarray<std::complex<double>, 2>({lmmax, na}, mp);
+                qapf     = mdarray<std::complex<double>, 2>({lmmax, ngv}, mp);
                 break;
             }
-            case sddk::device_t::GPU: {
-                auto& mp = get_memory_pool(sddk::memory_t::host);
-                auto& mpd = get_memory_pool(sddk::memory_t::device);
+            case device_t::GPU: {
+                auto& mp  = get_memory_pool(memory_t::host);
+                auto& mpd = get_memory_pool(memory_t::device);
                 /* allocate on GPU */
-                pf = sddk::mdarray<std::complex<double>, 2>(nullptr, ngv, na);
+                pf = mdarray<std::complex<double>, 2>({ngv, na}, nullptr);
                 pf.allocate(mpd);
                 /* allocate on CPU & GPU */
-                qa = sddk::mdarray<std::complex<double>, 2>(lmmax, na, mp);
+                qa = mdarray<std::complex<double>, 2>({lmmax, na}, mp);
                 qa.allocate(mpd);
                 /* allocate on CPU & GPU */
-                qapf = sddk::mdarray<std::complex<double>, 2>(lmmax, ngv, mp);
+                qapf = mdarray<std::complex<double>, 2>({lmmax, ngv}, mp);
                 qapf.allocate(mpd);
                 break;
             }
@@ -103,20 +105,22 @@ void Potential::poisson_add_pseudo_pw(sddk::mdarray<std::complex<double>, 2>& qm
         }
 
         switch (ctx_.processing_unit()) {
-            case sddk::device_t::CPU: {
-                la::wrap(la::lib_t::blas).gemm('N', 'C', ctx_.lmmax_rho(), ctx_.gvec().count(),
-                    unit_cell_.atom_type(iat).num_atoms(), &la::constant<std::complex<double>>::one(),
-                    qa.at(sddk::memory_t::host), qa.ld(), pf.at(sddk::memory_t::host), pf.ld(),
-                    &la::constant<std::complex<double>>::zero(), qapf.at(sddk::memory_t::host), qapf.ld());
+            case device_t::CPU: {
+                la::wrap(la::lib_t::blas)
+                        .gemm('N', 'C', ctx_.lmmax_rho(), ctx_.gvec().count(), unit_cell_.atom_type(iat).num_atoms(),
+                              &la::constant<std::complex<double>>::one(), qa.at(memory_t::host), qa.ld(),
+                              pf.at(memory_t::host), pf.ld(), &la::constant<std::complex<double>>::zero(),
+                              qapf.at(memory_t::host), qapf.ld());
                 break;
             }
-            case sddk::device_t::GPU: {
-                qa.copy_to(sddk::memory_t::device);
-                la::wrap(la::lib_t::gpublas).gemm('N', 'C', ctx_.lmmax_rho(), ctx_.gvec().count(),
-                    unit_cell_.atom_type(iat).num_atoms(), &la::constant<std::complex<double>>::one(),
-                    qa.at(sddk::memory_t::device), qa.ld(), pf.at(sddk::memory_t::device), pf.ld(),
-                    &la::constant<std::complex<double>>::zero(), qapf.at(sddk::memory_t::device), qapf.ld());
-                qapf.copy_to(sddk::memory_t::host);
+            case device_t::GPU: {
+                qa.copy_to(memory_t::device);
+                la::wrap(la::lib_t::gpublas)
+                        .gemm('N', 'C', ctx_.lmmax_rho(), ctx_.gvec().count(), unit_cell_.atom_type(iat).num_atoms(),
+                              &la::constant<std::complex<double>>::one(), qa.at(memory_t::device), qa.ld(),
+                              pf.at(memory_t::device), pf.ld(), &la::constant<std::complex<double>>::zero(),
+                              qapf.at(memory_t::device), qapf.ld());
+                qapf.copy_to(memory_t::host);
                 break;
             }
         }
@@ -127,7 +131,7 @@ void Potential::poisson_add_pseudo_pw(sddk::mdarray<std::complex<double>, 2>& qm
          * multipole moments in the muffin-tins */
         #pragma omp parallel for schedule(static)
         for (int igloc = ctx_.gvec().skip_g0(); igloc < ctx_.gvec().count(); igloc++) {
-            double gR = ctx_.gvec().gvec_len<sddk::index_domain_t::local>(igloc) * R;
+            double gR  = ctx_.gvec().gvec_len<index_domain_t::local>(igloc) * R;
             double gRn = std::pow(2.0 / gR, pseudo_density_order_ + 1);
 
             std::complex<double> rho_G(0, 0);
@@ -154,7 +158,8 @@ void Potential::poisson_add_pseudo_pw(sddk::mdarray<std::complex<double>, 2>& qm
     }
 }
 
-void Potential::poisson(Periodic_function<double> const& rho)
+void
+Potential::poisson(Periodic_function<double> const& rho)
 {
     PROFILE("sirius::Potential::poisson");
 
@@ -164,22 +169,22 @@ void Potential::poisson(Periodic_function<double> const& rho)
         /* true multipole moments */
         auto qmt = poisson_vmt(rho);
 
-        if (ctx_.cfg().control().print_checksum()) {
-            utils::print_checksum("qmt", qmt.checksum(), ctx_.out());
+        if (env::print_checksum()) {
+            print_checksum("qmt", qmt.checksum(), ctx_.out());
         }
 
         /* compute multipoles of interstitial density in MT region */
-        auto qit = ctx_.sum_fg_fl_yg(ctx_.lmax_rho(), &rho.f_pw_local(0), sbessel_mom_, gvec_ylm_);
+        auto qit = sum_fg_fl_yg(ctx_, ctx_.lmax_rho(), &rho.rg().f_pw_local(0), sbessel_mom_, gvec_ylm_);
 
-        if (ctx_.cfg().control().print_checksum()) {
-            utils::print_checksum("qit", qit.checksum(), ctx_.out());
+        if (env::print_checksum()) {
+            print_checksum("qit", qit.checksum(), ctx_.out());
         }
 
         /* add contribution from the pseudo-charge */
-        poisson_add_pseudo_pw(qmt, qit, const_cast<std::complex<double>*>(&rho.f_pw_local(0)));
+        poisson_add_pseudo_pw(qmt, qit, const_cast<std::complex<double>*>(&rho.rg().f_pw_local(0)));
 
         if (ctx_.cfg().control().verification() >= 2) {
-            auto qit = ctx_.sum_fg_fl_yg(ctx_.lmax_rho(), &rho.f_pw_local(0), sbessel_mom_, gvec_ylm_);
+            auto qit = sum_fg_fl_yg(ctx_, ctx_.lmax_rho(), &rho.rg().f_pw_local(0), sbessel_mom_, gvec_ylm_);
 
             double d = 0.0;
             for (int ia = 0; ia < unit_cell_.num_atoms(); ia++) {
@@ -195,13 +200,14 @@ void Potential::poisson(Periodic_function<double> const& rho)
 
     /* compute pw coefficients of Hartree potential */
     if (ctx_.gvec().comm().rank() == 0) {
-        hartree_potential_->f_pw_local(0) = 0.0;
+        hartree_potential_->rg().f_pw_local(0) = 0.0;
     }
     if (!ctx_.molecule()) {
         #pragma omp parallel for
         for (int igloc = ctx_.gvec().skip_g0(); igloc < ctx_.gvec().count(); igloc++) {
-            hartree_potential_->f_pw_local(igloc) = fourpi * rho.f_pw_local(igloc) /
-                std::pow(ctx_.gvec().gvec_len<sddk::index_domain_t::local>(igloc), 2);
+            hartree_potential_->rg().f_pw_local(igloc) =
+                    fourpi * rho.rg().f_pw_local(igloc) /
+                    std::pow(ctx_.gvec().gvec_len<index_domain_t::local>(igloc), 2);
         }
     } else {
         /* reference paper:
@@ -214,27 +220,28 @@ void Potential::poisson(Periodic_function<double> const& rho)
         double R_cut = 0.5 * std::pow(unit_cell_.omega(), 1.0 / 3);
         #pragma omp parallel for
         for (int igloc = ctx_.gvec().skip_g0(); igloc < ctx_.gvec().count(); igloc++) {
-            auto glen = ctx_.gvec().gvec_len<sddk::index_domain_t::local>(igloc);
-            hartree_potential_->f_pw_local(igloc) = (fourpi * rho.f_pw_local(igloc) / std::pow(glen, 2)) *
-                                                    (1.0 - std::cos(glen * R_cut));
+            auto glen = ctx_.gvec().gvec_len<index_domain_t::local>(igloc);
+            hartree_potential_->rg().f_pw_local(igloc) =
+                    (fourpi * rho.rg().f_pw_local(igloc) / std::pow(glen, 2)) * (1.0 - std::cos(glen * R_cut));
         }
     }
 
-    //if (ctx_.control().print_checksum_) {
-    //    auto z4 = mdarray<std::complex<double>, 1>(&vh->f_pw(0), ctx_.gvec().num_gvec()).checksum();
-    //    if (ctx_.comm().rank() == 0) {
-    //        DUMP("checksum(vh_pw): %20.14f %20.14f", z4.real(), z4.imag());
-    //    }
-    //}
+    // if (ctx_.control().print_checksum_) {
+    //     auto z4 = mdarray<std::complex<double>, 1>(&vh->f_pw(0), ctx_.gvec().num_gvec()).checksum();
+    //     if (ctx_.comm().rank() == 0) {
+    //         DUMP("checksum(vh_pw): %20.14f %20.14f", z4.real(), z4.imag());
+    //     }
+    // }
 
     /* boundary condition for muffin-tins */
     if (ctx_.full_potential()) {
         /* compute V_lm at the MT boundary */
-        auto vmtlm = ctx_.sum_fg_fl_yg(ctx_.lmax_pot(), &hartree_potential_->f_pw_local(0), sbessel_mt_, gvec_ylm_);
+        auto vmtlm =
+                sum_fg_fl_yg(ctx_, ctx_.lmax_pot(), &hartree_potential_->rg().f_pw_local(0), sbessel_mt_, gvec_ylm_);
 
         /* add boundary condition and convert to Rlm */
         PROFILE("sirius::Potential::poisson|bc");
-        sddk::mdarray<double, 3> rRl(unit_cell_.max_num_mt_points(), ctx_.lmax_pot() + 1, unit_cell_.num_atom_types());
+        mdarray<double, 3> rRl({unit_cell_.max_num_mt_points(), ctx_.lmax_pot() + 1, unit_cell_.num_atom_types()});
         for (int iat = 0; iat < unit_cell_.num_atom_types(); iat++) {
             int nmtp = unit_cell_.atom_type(iat).num_mt_points();
             double R = unit_cell_.atom_type(iat).mt_radius();
@@ -247,42 +254,40 @@ void Potential::poisson(Periodic_function<double> const& rho)
             }
         }
 
-        for (int ialoc = 0; ialoc < unit_cell_.spl_num_atoms().local_size(); ialoc++) {
-            int ia = unit_cell_.spl_num_atoms(ialoc);
-            int nmtp = unit_cell_.atom(ia).num_mt_points();
+        for (auto it : unit_cell_.spl_num_atoms()) {
+            int nmtp = unit_cell_.atom(it.i).num_mt_points();
 
             std::vector<double> vlm(ctx_.lmmax_pot());
-            SHT::convert(ctx_.lmax_pot(), &vmtlm(0, ia), &vlm[0]);
+            SHT::convert(ctx_.lmax_pot(), &vmtlm(0, it.i), &vlm[0]);
 
             #pragma omp parallel for default(shared)
             for (int lm = 0; lm < ctx_.lmmax_pot(); lm++) {
                 int l = l_by_lm_[lm];
 
                 for (int ir = 0; ir < nmtp; ir++) {
-                    hartree_potential_->f_mt<sddk::index_domain_t::local>(lm, ir, ialoc) +=
-                        vlm[lm] * rRl(ir, l, unit_cell_.atom(ia).type_id());
+                    hartree_potential_->mt()[it.i](lm, ir) += vlm[lm] * rRl(ir, l, unit_cell_.atom(it.i).type_id());
                 }
             }
             /* save electronic part of the potential at the point of origin */
 #ifdef __VHA_AUX
-            vh_el_(ia) = y00 * hartree_potential_->f_mt<index_domain_t::local>(0, 0, ialoc) +
-                unit_cell_.atom(ia).zn() / unit_cell_.atom(ia).radial_grid(0);
+            vh_el_(it.i) = y00 * hartree_potential_->mt()[it.i](0, 0) +
+                           unit_cell_.atom(it.i).zn() / unit_cell_.atom(it.i).radial_grid(0);
 #else
-            vh_el_(ia) = y00 * hartree_potential_->f_mt<sddk::index_domain_t::local>(0, 0, ialoc);
+            vh_el_(it.i) = y00 * hartree_potential_->mt()[it.i](0, 0);
 #endif
         }
-        ctx_.comm().allgather(vh_el_.at(sddk::memory_t::host), unit_cell_.spl_num_atoms().local_size(),
-                unit_cell_.spl_num_atoms().global_offset());
+        ctx_.comm().allgather(vh_el_.at(memory_t::host), unit_cell_.spl_num_atoms().local_size(),
+                              unit_cell_.spl_num_atoms().global_offset());
     }
 
     /* transform Hartree potential to real space */
-    hartree_potential_->fft_transform(1);
+    hartree_potential_->rg().fft_transform(1);
 
-    if (ctx_.cfg().control().print_checksum()) {
-        auto cs = hartree_potential_->checksum_rg();
-        auto cs1 = hartree_potential_->checksum_pw();
-        utils::print_checksum("vha_rg", cs, ctx_.out());
-        utils::print_checksum("vha_pw", cs1, ctx_.out());
+    if (env::print_checksum()) {
+        auto cs  = hartree_potential_->rg().checksum_rg();
+        auto cs1 = hartree_potential_->rg().checksum_pw();
+        print_checksum("vha_rg", cs, ctx_.out());
+        print_checksum("vha_pw", cs1, ctx_.out());
     }
 
     /* compute contribution from the smooth part of Hartree potential */
@@ -292,14 +297,13 @@ void Potential::poisson(Periodic_function<double> const& rho)
     /* add nucleus potential and contribution to Hartree energy */
     if (ctx_.full_potential()) {
         double evha_nuc{0};
-        for (int ialoc = 0; ialoc < unit_cell_.spl_num_atoms().local_size(); ialoc++) {
-            int ia = unit_cell_.spl_num_atoms(ialoc);
-            auto& atom = unit_cell_.atom(ia);
+        for (auto it : unit_cell_.spl_num_atoms()) {
+            auto& atom = unit_cell_.atom(it.i);
             Spline<double> srho(atom.radial_grid());
             for (int ir = 0; ir < atom.num_mt_points(); ir++) {
                 double r = atom.radial_grid(ir);
-                hartree_potential_->f_mt<sddk::index_domain_t::local>(0, ir, ialoc) -= atom.zn() / r / y00;
-                srho(ir) = rho.f_mt<sddk::index_domain_t::local>(0, ir, ialoc) * r;
+                hartree_potential_->mt()[it.i](0, ir) -= atom.zn() / r / y00;
+                srho(ir) = rho.mt()[it.i](0, ir) * r;
             }
             evha_nuc -= atom.zn() * srho.interpolate().integrate(0) / y00;
         }
@@ -309,9 +313,9 @@ void Potential::poisson(Periodic_function<double> const& rho)
 #endif
 
     /* check values at MT boundary */
-    //if (true) {
-    //    /* compute V_lm at the MT boundary */
-    //    auto vmtlm = ctx_.sum_fg_fl_yg(ctx_.lmax_pot(), &hartree_potential_->f_pw_local(0), sbessel_mt_, gvec_ylm_);
+    // if (true) {
+    //     /* compute V_lm at the MT boundary */
+    //     auto vmtlm = ctx_.sum_fg_fl_yg(ctx_.lmax_pot(), &hartree_potential_->f_pw_local(0), sbessel_mt_, gvec_ylm_);
 
     //    for (int ialoc = 0; ialoc < unit_cell_.spl_num_atoms().local_size(); ialoc++) {
     //        int ia = unit_cell_.spl_num_atoms(ialoc);
