@@ -76,13 +76,15 @@ class Spline : public Radial_grid<U>
 {
   private:
     /// Array of spline coefficients.
-    sddk::mdarray<T, 2> coeffs_;
+    mdarray<T, 2> coeffs_;
     /* forbid copy constructor */
     Spline(Spline<T, U> const& src__) = delete;
     /* forbid assignment operator */
-    Spline<T, U>& operator=(Spline<T, U> const& src__) = delete;
+    Spline<T, U>&
+    operator=(Spline<T, U> const& src__) = delete;
     /// Solver tridiagonal system of linear equaitons.
-    int solve(T* dl, T* d, T* du, T* b, int n)
+    int
+    solve(T* dl, T* d, T* du, T* b, int n)
     {
         for (int i = 0; i < n - 1; i++) {
             if (std::abs(dl[i]) == 0) {
@@ -124,11 +126,12 @@ class Spline : public Radial_grid<U>
         return 0;
     }
     /// Init the underlying radial grid.
-    void init_grid(Radial_grid<U> const& radial_grid__)
+    void
+    init_grid(Radial_grid<U> const& radial_grid__)
     {
         /* copy the grid points */
-        this->x_ = sddk::mdarray<U, 1>(radial_grid__.num_points());
-        radial_grid__.x() >> this->x_;
+        this->x_ = mdarray<U, 1>({radial_grid__.num_points()});
+        copy(radial_grid__.x(), this->x_);
         this->init();
     }
 
@@ -141,14 +144,14 @@ class Spline : public Radial_grid<U>
     Spline(Radial_grid<U> const& radial_grid__)
     {
         init_grid(radial_grid__);
-        coeffs_ = sddk::mdarray<T, 2>(this->num_points(), 4);
+        coeffs_ = mdarray<T, 2>({this->num_points(), 4});
         coeffs_.zero();
     }
     /// Constructor of a spline from a function.
     Spline(Radial_grid<U> const& radial_grid__, std::function<T(U)> f__)
     {
         init_grid(radial_grid__);
-        coeffs_ = sddk::mdarray<T, 2>(this->num_points(), 4);
+        coeffs_ = mdarray<T, 2>({this->num_points(), 4});
         for (int i = 0; i < this->num_points(); i++) {
             coeffs_(i, 0) = f__(this->x(i));
         }
@@ -159,7 +162,7 @@ class Spline : public Radial_grid<U>
     {
         init_grid(radial_grid__);
         assert(static_cast<int>(y__.size()) <= this->num_points());
-        coeffs_ = sddk::mdarray<T, 2>(this->num_points(), 4);
+        coeffs_ = mdarray<T, 2>({this->num_points(), 4});
         coeffs_.zero();
         int i{0};
         for (auto e : y__) {
@@ -170,9 +173,11 @@ class Spline : public Radial_grid<U>
 
     Spline(Spline<T, U>&& src__) = default;
 
-    Spline<T, U>& operator=(Spline<T, U>&& src__) = default;
+    Spline<T, U>&
+    operator=(Spline<T, U>&& src__) = default;
 
-    Spline<T, U>& operator=(std::function<T(U)> f__)
+    Spline<T, U>&
+    operator=(std::function<T(U)> f__)
     {
         for (int ir = 0; ir < this->num_points(); ir++) {
             coeffs_(ir, 0) = f__(this->x(ir));
@@ -180,7 +185,8 @@ class Spline : public Radial_grid<U>
         return interpolate();
     }
 
-    Spline<T, U>& operator=(std::vector<T> const& y__)
+    Spline<T, U>&
+    operator=(std::vector<T> const& y__)
     {
         assert(static_cast<int>(y__.size()) <= this->num_points());
         coeffs_.zero();
@@ -192,19 +198,22 @@ class Spline : public Radial_grid<U>
     }
 
     /// Get the reference to a value at the point x[i].
-    inline T& operator()(const int i)
+    inline T&
+    operator()(const int i)
     {
         return coeffs_(i, 0);
     }
 
     /// Get value at the point x[i].
-    inline T const& operator()(const int i) const
+    inline T const&
+    operator()(const int i) const
     {
         return coeffs_(i, 0);
     }
 
     /// Get value at the point x[i] + dx.
-    inline T operator()(const int i, U dx) const
+    inline T
+    operator()(const int i, U dx) const
     {
         assert(i >= 0);
         assert(i < this->num_points() - 1);
@@ -213,7 +222,8 @@ class Spline : public Radial_grid<U>
     }
 
     /// Compute value at any point.
-    inline T at_point(U x) const
+    inline T
+    at_point(U x) const
     {
         int j = this->index_of(x);
         if (j == -1) {
@@ -228,7 +238,8 @@ class Spline : public Radial_grid<U>
         return (*this)(j, dx);
     }
 
-    inline T deriv(const int dm, const int i, const U dx) const
+    inline T
+    deriv(const int dm, const int i, const U dx) const
     {
         assert(i >= 0);
         assert(i < this->num_points() - 1);
@@ -260,7 +271,8 @@ class Spline : public Radial_grid<U>
         return result;
     }
 
-    inline T deriv(int dm, int i) const
+    inline T
+    deriv(int dm, int i) const
     {
         assert(i >= 0);
         assert(i < this->num_points());
@@ -272,18 +284,19 @@ class Spline : public Radial_grid<U>
         }
     }
 
-    Spline<T, U>& interpolate()
+    Spline<T, U>&
+    interpolate()
     {
         /* number of segments; in principle we have n-1 segments, but the equations for spline are built in such a
            way that one extra m_i coefficient of a segment is necessary */
         int ns = this->num_points();
         assert(ns >= 4);
         /* lower diagonal (use coeffs as temporary storage) */
-        T* dl = coeffs_.at(sddk::memory_t::host, 0, 1);
+        T* dl = coeffs_.at(memory_t::host, 0, 1);
         /* main diagonal */
-        T* d = coeffs_.at(sddk::memory_t::host, 0, 2);
+        T* d = coeffs_.at(memory_t::host, 0, 2);
         /* upper diagonal */
-        T* du = coeffs_.at(sddk::memory_t::host, 0, 3);
+        T* du = coeffs_.at(memory_t::host, 0, 3);
         /* m_i = 2 c_i */
         std::vector<T> m(ns);
         /* derivatives of function */
@@ -357,69 +370,70 @@ class Spline : public Radial_grid<U>
     /**
     Derivation for r^2 prefactor is based on the following Mathematica notebook:
     \verbatim
-            In[26]:= result = 
+            In[26]:= result =
          FullSimplify[
           Integrate[
-           x^(2)*(a0 + a1*(x - x0) + a2*(x - x0)^2 + a3*(x - x0)^3), {x, x0, 
+           x^(2)*(a0 + a1*(x - x0) + a2*(x - x0)^2 + a3*(x - x0)^3), {x, x0,
             x1}],
           Assumptions -> {Element[{x0, x1}, Reals], x1 > x0 > 0}]
-        
-        
-        Out[26]= 1/60 (20 a0 (-x0^3 + x1^3) + 
-           5 a1 (x0^4 - 4 x0 x1^3 + 3 x1^4) + (x0 - 
-              x1)^3 (-2 a2 (x0^2 + 3 x0 x1 + 6 x1^2) + 
+
+
+        Out[26]= 1/60 (20 a0 (-x0^3 + x1^3) +
+           5 a1 (x0^4 - 4 x0 x1^3 + 3 x1^4) + (x0 -
+              x1)^3 (-2 a2 (x0^2 + 3 x0 x1 + 6 x1^2) +
               a3 (x0 - x1) (x0^2 + 4 x0 x1 + 10 x1^2)))
-        
+
         In[27]:= r = Expand[result] /. {x1 -> x0 + dx}
-        
+
         Out[27]= -((a0 x0^3)/3) + (a1 x0^4)/12 - (a2 x0^5)/30 + (
-         a3 x0^6)/60 + 1/3 a0 (dx + x0)^3 - 1/3 a1 x0 (dx + x0)^3 + 
-         1/3 a2 x0^2 (dx + x0)^3 - 1/3 a3 x0^3 (dx + x0)^3 + 
-         1/4 a1 (dx + x0)^4 - 1/2 a2 x0 (dx + x0)^4 + 
-         3/4 a3 x0^2 (dx + x0)^4 + 1/5 a2 (dx + x0)^5 - 
+         a3 x0^6)/60 + 1/3 a0 (dx + x0)^3 - 1/3 a1 x0 (dx + x0)^3 +
+         1/3 a2 x0^2 (dx + x0)^3 - 1/3 a3 x0^3 (dx + x0)^3 +
+         1/4 a1 (dx + x0)^4 - 1/2 a2 x0 (dx + x0)^4 +
+         3/4 a3 x0^2 (dx + x0)^4 + 1/5 a2 (dx + x0)^5 -
          3/5 a3 x0 (dx + x0)^5 + 1/6 a3 (dx + x0)^6
-        
+
         In[34]:= Collect[r, dx, Simplify]
-        
-        Out[34]= (a3 dx^6)/6 + a0 dx x0^2 + 1/2 dx^2 x0 (2 a0 + a1 x0) + 
-         1/5 dx^5 (a2 + 2 a3 x0) + 1/3 dx^3 (a0 + x0 (2 a1 + a2 x0)) + 
+
+        Out[34]= (a3 dx^6)/6 + a0 dx x0^2 + 1/2 dx^2 x0 (2 a0 + a1 x0) +
+         1/5 dx^5 (a2 + 2 a3 x0) + 1/3 dx^3 (a0 + x0 (2 a1 + a2 x0)) +
          1/4 dx^4 (a1 + x0 (2 a2 + a3 x0))
-        
+
         In[28]:= r1 = Collect[r/dx, dx, Simplify]
-        
-        Out[28]= (a3 dx^5)/6 + a0 x0^2 + 1/2 dx x0 (2 a0 + a1 x0) + 
-         1/5 dx^4 (a2 + 2 a3 x0) + 1/3 dx^2 (a0 + x0 (2 a1 + a2 x0)) + 
+
+        Out[28]= (a3 dx^5)/6 + a0 x0^2 + 1/2 dx x0 (2 a0 + a1 x0) +
+         1/5 dx^4 (a2 + 2 a3 x0) + 1/3 dx^2 (a0 + x0 (2 a1 + a2 x0)) +
          1/4 dx^3 (a1 + x0 (2 a2 + a3 x0))
-        
+
         In[29]:= r2 = Collect[(r1 - a0*x0^2)/dx, dx, Simplify]
-        
-        Out[29]= (a3 dx^4)/6 + 1/2 x0 (2 a0 + a1 x0) + 
-         1/5 dx^3 (a2 + 2 a3 x0) + 1/3 dx (a0 + x0 (2 a1 + a2 x0)) + 
+
+        Out[29]= (a3 dx^4)/6 + 1/2 x0 (2 a0 + a1 x0) +
+         1/5 dx^3 (a2 + 2 a3 x0) + 1/3 dx (a0 + x0 (2 a1 + a2 x0)) +
          1/4 dx^2 (a1 + x0 (2 a2 + a3 x0))
-        
+
         In[30]:= r3 = Collect[(r2 - 1/2 x0 (2 a0 + a1 x0))/dx, dx, Simplify]
-        
-        Out[30]= (a3 dx^3)/6 + 1/5 dx^2 (a2 + 2 a3 x0) + 
+
+        Out[30]= (a3 dx^3)/6 + 1/5 dx^2 (a2 + 2 a3 x0) +
          1/3 (a0 + x0 (2 a1 + a2 x0)) + 1/4 dx (a1 + x0 (2 a2 + a3 x0))
-        
-        In[31]:= r4 = 
+
+        In[31]:= r4 =
          Collect[(r3 - 1/3 (a0 + x0 (2 a1 + a2 x0)))/dx, dx, Simplify]
-        
-        Out[31]= (a3 dx^2)/6 + 1/5 dx (a2 + 2 a3 x0) + 
+
+        Out[31]= (a3 dx^2)/6 + 1/5 dx (a2 + 2 a3 x0) +
          1/4 (a1 + x0 (2 a2 + a3 x0))
-        
-        In[32]:= r5 = 
+
+        In[32]:= r5 =
          Collect[(r4 - 1/4 (a1 + x0 (2 a2 + a3 x0)))/dx, dx, Simplify]
-        
+
         Out[32]= (a3 dx)/6 + 1/5 (a2 + 2 a3 x0)
-        
+
         In[33]:= r6 = Collect[(r5 - 1/5 (a2 + 2 a3 x0))/dx, dx, Simplify]
-        
+
         Out[33]= a3/6
 
     \endverbatim
     */
-    T integrate(std::vector<T>& g__, int m__) const
+    T
+    integrate(std::vector<T>& g__, int m__) const
     {
         g__    = std::vector<T>(this->num_points());
         g__[0] = 0.0;
@@ -429,7 +443,10 @@ class Spline : public Radial_grid<U>
                 T t = 1.0 / 3.0;
                 for (int i = 0; i < this->num_points() - 1; i++) {
                     U dx       = this->dx(i);
-                    g__[i + 1] = g__[i] + (((coeffs_(i, 3) * dx * 0.25 + coeffs_(i, 2) * t) * dx + coeffs_(i, 1) * 0.5) * dx + coeffs_(i, 0)) * dx;
+                    g__[i + 1] = g__[i] +
+                                 (((coeffs_(i, 3) * dx * 0.25 + coeffs_(i, 2) * t) * dx + coeffs_(i, 1) * 0.5) * dx +
+                                  coeffs_(i, 0)) *
+                                         dx;
                 }
                 break;
             }
@@ -443,8 +460,10 @@ class Spline : public Radial_grid<U>
                     T a3 = coeffs_(i, 3);
 
                     T val = dx * (dx * (dx * (dx * (dx * (dx * a3 / 6.0 + (a2 + 2.0 * a3 * x0) / 5.0) +
-                        (a1 + x0 * (2.0 * a2 + a3 * x0)) / 4.0) + (a0 + x0 * (2.0 * a1 + a2 * x0)) / 3.0) +
-                        x0 * (2.0 * a0 + a1 * x0) / 2.0) + a0 * x0 * x0);
+                                                    (a1 + x0 * (2.0 * a2 + a3 * x0)) / 4.0) +
+                                              (a0 + x0 * (2.0 * a1 + a2 * x0)) / 3.0) +
+                                        x0 * (2.0 * a0 + a1 * x0) / 2.0) +
+                                  a0 * x0 * x0);
 
                     g__[i + 1] = g__[i] + val;
                 }
@@ -463,7 +482,9 @@ class Spline : public Radial_grid<U>
                     // obtained with the following Mathematica code:
                     //   FullSimplify[Integrate[x^(-1)*(a0+a1*(x-x0)+a2*(x-x0)^2+a3*(x-x0)^3),{x,x0,x1}],
                     //                          Assumptions->{Element[{x0,x1},Reals],x1>x0>0}]
-                    g__[i + 1] = g__[i] + (dx / 6.0) * (6.0 * a1 + x0 * (-9.0 * a2 + 11.0 * a3 * x0) + x1 * (3.0 * a2 - 7.0 * a3 * x0 + 2.0 * a3 * x1)) +
+                    g__[i + 1] = g__[i] +
+                                 (dx / 6.0) * (6.0 * a1 + x0 * (-9.0 * a2 + 11.0 * a3 * x0) +
+                                               x1 * (3.0 * a2 - 7.0 * a3 * x0 + 2.0 * a3 * x1)) +
                                  (-a0 + x0 * (a1 + x0 * (-a2 + a3 * x0))) * std::log(x0 / x1);
                 }
                 break;
@@ -485,10 +506,10 @@ class Spline : public Radial_grid<U>
                     //             a3 * (2.0 * std::pow(x0, 2) + 5.0 * x0 * x1 - std::pow(x1, 2)))) +
                     //             2.0 * x0 * (a1 + x0 * (-2.0 * a2 + 3.0 * a3 * x0)) * x1 * std::log(x1 / x0)) /
                     //             (2.0 * x0 * x1));
-                    g__[i + 1] =
-                        g__[i] +
-                        (a2 * dx - 5.0 * a3 * x0 * dx / 2.0 - a1 * (dx / x1) + a0 * (dx / x0 / x1) + (x0 / x1) * dx * (a2 - a3 * x0) + a3 * x1 * dx / 2.0) +
-                        (a1 + x0 * (-2.0 * a2 + 3.0 * a3 * x0)) * std::log(x1 / x0);
+                    g__[i + 1] = g__[i] +
+                                 (a2 * dx - 5.0 * a3 * x0 * dx / 2.0 - a1 * (dx / x1) + a0 * (dx / x0 / x1) +
+                                  (x0 / x1) * dx * (a2 - a3 * x0) + a3 * x1 * dx / 2.0) +
+                                 (a1 + x0 * (-2.0 * a2 + 3.0 * a3 * x0)) * std::log(x1 / x0);
                 }
                 break;
             }
@@ -507,15 +528,15 @@ class Spline : public Radial_grid<U>
                     //                          Assumptions->{Element[{x0,x1},Reals],x1>x0>0}]
                     // g__[i + 1] = g__[i] + (-((x0 - x1) * (a0 * (x0 + x1) + x0 * (a1 * (-x0 + x1) +
                     //             x0 * (a2 * x0 - a3 * std::pow(x0, 2) - 3.0 * a2 * x1 + 5.0 * a3 * x0 * x1 +
-                    //             2.0 * a3 * std::pow(x1, 2)))) + 2.0 * std::pow(x0, 2) * (a2 - 3.0 * a3 * x0) * std::pow(x1, 2) *
-                    //             std::log(x0 / x1)) / (2.0 * std::pow(x0, 2) * std::pow(x1, 2)));
-                    g__[i + 1] =
-                        g__[i] +
-                        dx *
-                            (a0 * (x0 + x1) +
-                             x0 * (a1 * dx + x0 * (a2 * x0 - a3 * std::pow(x0, 2) - 3.0 * a2 * x1 + 5.0 * a3 * x0 * x1 + 2.0 * a3 * std::pow(x1, 2)))) /
-                            std::pow(x0 * x1, 2) / 2.0 +
-                        (-a2 + 3.0 * a3 * x0) * std::log(x0 / x1);
+                    //             2.0 * a3 * std::pow(x1, 2)))) + 2.0 * std::pow(x0, 2) * (a2 - 3.0 * a3 * x0) *
+                    //             std::pow(x1, 2) * std::log(x0 / x1)) / (2.0 * std::pow(x0, 2) * std::pow(x1, 2)));
+                    g__[i + 1] = g__[i] +
+                                 dx *
+                                         (a0 * (x0 + x1) +
+                                          x0 * (a1 * dx + x0 * (a2 * x0 - a3 * std::pow(x0, 2) - 3.0 * a2 * x1 +
+                                                                5.0 * a3 * x0 * x1 + 2.0 * a3 * std::pow(x1, 2)))) /
+                                         std::pow(x0 * x1, 2) / 2.0 +
+                                 (-a2 + 3.0 * a3 * x0) * std::log(x0 / x1);
                 }
                 break;
             }
@@ -534,15 +555,18 @@ class Spline : public Radial_grid<U>
                     //                          Assumptions->{Element[{x0,x1},Reals],x1>x0>0}]
                     // g__[i + 1] = g__[i] + ((2.0 * a0 * (-std::pow(x0, 3) + std::pow(x1, 3)) +
                     //             x0 * (x0 - x1) * (a1 * (x0 - x1) * (2.0 * x0 + x1) +
-                    //             x0 * (-2.0 * a2 * std::pow(x0 - x1, 2) + a3 * x0 * (2.0 * std::pow(x0, 2) - 7.0 * x0 * x1 +
-                    //             11.0 * std::pow(x1, 2)))) + 6.0 * a3 * std::pow(x0 * x1, 3) * std::log(x1 / x0)) /
-                    //             (6.0 * std::pow(x0 * x1, 3)));
+                    //             x0 * (-2.0 * a2 * std::pow(x0 - x1, 2) + a3 * x0 * (2.0 * std::pow(x0, 2) - 7.0 * x0
+                    //             * x1 + 11.0 * std::pow(x1, 2)))) + 6.0 * a3 * std::pow(x0 * x1, 3) * std::log(x1 /
+                    //             x0)) / (6.0 * std::pow(x0 * x1, 3)));
                     g__[i + 1] = g__[i] +
                                  (2.0 * a0 * (-std::pow(x0, 3) + std::pow(x1, 3)) -
                                   x0 * dx *
-                                      (-a1 * dx * (2.0 * x0 + x1) +
-                                       x0 * (-2.0 * a2 * std::pow(dx, 2) + a3 * x0 * (2.0 * std::pow(x0, 2) - 7.0 * x0 * x1 + 11.0 * std::pow(x1, 2))))) /
-                                     std::pow(x0 * x1, 3) / 6.0 +
+                                          (-a1 * dx * (2.0 * x0 + x1) +
+                                           x0 * (-2.0 * a2 * std::pow(dx, 2) +
+                                                 a3 * x0 *
+                                                         (2.0 * std::pow(x0, 2) - 7.0 * x0 * x1 +
+                                                          11.0 * std::pow(x1, 2))))) /
+                                         std::pow(x0 * x1, 3) / 6.0 +
                                  a3 * std::log(x1 / x0);
                 }
                 break;
@@ -560,13 +584,16 @@ class Spline : public Radial_grid<U>
                     //   FullSimplify[Integrate[x^(m)*(a0+a1*(x-x0)+a2*(x-x0)^2+a3*(x-x0)^3),{x,x0,x1}],
                     //                          Assumptions->{Element[{x0,x1},Reals],x1>x0>0}]
                     g__[i + 1] =
-                        g__[i] +
-                        (std::pow(x0, 1 + m__) * (-(a0 * double((2 + m__) * (3 + m__) * (4 + m__))) +
-                                                  x0 * (a1 * double((3 + m__) * (4 + m__)) - 2.0 * a2 * double(4 + m__) * x0 + 6.0 * a3 * std::pow(x0, 2)))) /
-                            double((1 + m__) * (2 + m__) * (3 + m__) * (4 + m__)) +
-                        std::pow(x1, 1 + m__) *
-                            ((a0 - x0 * (a1 + x0 * (-a2 + a3 * x0))) / double(1 + m__) + ((a1 + x0 * (-2.0 * a2 + 3.0 * a3 * x0)) * x1) / double(2 + m__) +
-                             ((a2 - 3.0 * a3 * x0) * std::pow(x1, 2)) / double(3 + m__) + (a3 * std::pow(x1, 3)) / double(4 + m__));
+                            g__[i] +
+                            (std::pow(x0, 1 + m__) *
+                             (-(a0 * double((2 + m__) * (3 + m__) * (4 + m__))) +
+                              x0 * (a1 * double((3 + m__) * (4 + m__)) - 2.0 * a2 * double(4 + m__) * x0 +
+                                    6.0 * a3 * std::pow(x0, 2)))) /
+                                    double((1 + m__) * (2 + m__) * (3 + m__) * (4 + m__)) +
+                            std::pow(x1, 1 + m__) * ((a0 - x0 * (a1 + x0 * (-a2 + a3 * x0))) / double(1 + m__) +
+                                                     ((a1 + x0 * (-2.0 * a2 + 3.0 * a3 * x0)) * x1) / double(2 + m__) +
+                                                     ((a2 - 3.0 * a3 * x0) * std::pow(x1, 2)) / double(3 + m__) +
+                                                     (a3 * std::pow(x1, 3)) / double(4 + m__));
                 }
                 break;
             }
@@ -576,13 +603,15 @@ class Spline : public Radial_grid<U>
     }
 
     /// Integrate spline with r^m prefactor.
-    T integrate(int m__) const
+    T
+    integrate(int m__) const
     {
         std::vector<T> g;
         return integrate(g, m__);
     }
 
-    inline void scale(double a__)
+    inline void
+    scale(double a__)
     {
         for (int i = 0; i < this->num_points(); i++) {
             coeffs_(i, 0) *= a__;
@@ -592,24 +621,20 @@ class Spline : public Radial_grid<U>
         }
     }
 
-    inline std::array<T, 4> coeffs(int i__) const
+    inline std::array<T, 4>
+    coeffs(int i__) const
     {
         return {coeffs_(i__, 0), coeffs_(i__, 1), coeffs_(i__, 2), coeffs_(i__, 3)};
     }
 
-    inline sddk::mdarray<T, 2> const& coeffs() const
+    inline mdarray<T, 2> const&
+    coeffs() const
     {
         return coeffs_;
     }
 
-    //void copy_to_device()
-    //{
-    //    // Radial_grid<U>::copy_to_device();
-    //    coeffs_.allocate(sddk::memory_t::device);
-    //    coeffs_.template copy<memory_t::host, memory_t::device>();
-    //}
-
-    std::vector<T> values() const
+    auto
+    values() const
     {
         std::vector<T> val(this->num_points());
         for (int ir = 0; ir < this->num_points(); ir++) {
@@ -620,20 +645,22 @@ class Spline : public Radial_grid<U>
 };
 
 template <typename T, typename U = double>
-inline Spline<T, U> operator*(Spline<T, U> const& a__, Spline<T, U> const& b__)
+inline Spline<T, U>
+operator*(Spline<T, U> const& a__, Spline<T, U> const& b__)
 {
     Spline<T> s12(reinterpret_cast<Radial_grid<U> const&>(a__));
 
     auto& coeffs_a = a__.coeffs();
     auto& coeffs_b = b__.coeffs();
-    auto& coeffs   = const_cast<sddk::mdarray<T, 2>&>(s12.coeffs());
+    auto& coeffs   = const_cast<mdarray<T, 2>&>(s12.coeffs());
 
     for (int ir = 0; ir < a__.num_points(); ir++) {
         coeffs(ir, 0) = coeffs_a(ir, 0) * coeffs_b(ir, 0);
         coeffs(ir, 1) = coeffs_a(ir, 1) * coeffs_b(ir, 0) + coeffs_a(ir, 0) * coeffs_b(ir, 1);
-        coeffs(ir, 2) = coeffs_a(ir, 2) * coeffs_b(ir, 0) + coeffs_a(ir, 1) * coeffs_b(ir, 1) + coeffs_a(ir, 0) * coeffs_b(ir, 2);
-        coeffs(ir, 3) =
-            coeffs_a(ir, 3) * coeffs_b(ir, 0) + coeffs_a(ir, 2) * coeffs_b(ir, 1) + coeffs_a(ir, 1) * coeffs_b(ir, 2) + coeffs_a(ir, 0) * coeffs_b(ir, 3);
+        coeffs(ir, 2) = coeffs_a(ir, 2) * coeffs_b(ir, 0) + coeffs_a(ir, 1) * coeffs_b(ir, 1) +
+                        coeffs_a(ir, 0) * coeffs_b(ir, 2);
+        coeffs(ir, 3) = coeffs_a(ir, 3) * coeffs_b(ir, 0) + coeffs_a(ir, 2) * coeffs_b(ir, 1) +
+                        coeffs_a(ir, 1) * coeffs_b(ir, 2) + coeffs_a(ir, 0) * coeffs_b(ir, 3);
     }
 
     return s12;
@@ -649,12 +676,14 @@ inline Spline<T, U> operator*(Spline<T, U> const& a__, Spline<T, U> const& b__)
 //                                              double*       h_buf__,
 //                                              int           stream_id__);
 //
-extern "C" void spline_inner_product_gpu_v3(int const* idx_ri__, int num_ri__, int num_points__, double const* x__, double const* dx__, double const* f__,
-                                            double const* g__, double* result__);
+extern "C" void
+spline_inner_product_gpu_v3(int const* idx_ri__, int num_ri__, int num_points__, double const* x__, double const* dx__,
+                            double const* f__, double const* g__, double* result__);
 #endif
 
 template <typename T>
-T inner(Spline<T> const& f__, Spline<T> const& g__, int m__, int num_points__)
+T
+inner(Spline<T> const& f__, Spline<T> const& g__, int m__, int num_points__)
 {
     T result = 0;
 
@@ -675,7 +704,10 @@ T inner(Spline<T> const& f__, Spline<T> const& g__, int m__, int num_points__)
                 T k4 = f[1] * g[3] + f[2] * g[2] + f[3] * g[1];
                 T k5 = f[2] * g[3] + f[3] * g[2];
 
-                result += dx * (faga + dx * (k1 / 2.0 + dx * (k2 / 3.0 + dx * (k3 / 4.0 + dx * (k4 / 5.0 + dx * (k5 / 6.0 + dx * fdgd / 7.0))))));
+                result += dx *
+                          (faga + dx * (k1 / 2.0 +
+                                        dx * (k2 / 3.0 +
+                                              dx * (k3 / 4.0 + dx * (k4 / 5.0 + dx * (k5 / 6.0 + dx * fdgd / 7.0))))));
             }
             break;
         }
@@ -696,12 +728,13 @@ T inner(Spline<T> const& f__, Spline<T> const& g__, int m__, int num_points__)
                 T k4 = f[1] * g[3] + f[2] * g[2] + f[3] * g[1];
                 T k5 = f[2] * g[3] + f[3] * g[2];
 
-                result +=
-                    dx * ((faga * x0) +
-                          dx * ((faga + k1 * x0) / 2.0 +
-                                dx * ((k1 + k2 * x0) / 3.0 +
-                                      dx * ((k2 + k3 * x0) / 4.0 +
-                                            dx * ((k3 + k4 * x0) / 5.0 + dx * ((k4 + k5 * x0) / 6.0 + dx * ((k5 + fdgd * x0) / 7.0 + dx * fdgd / 8.0)))))));
+                result += dx * ((faga * x0) +
+                                dx * ((faga + k1 * x0) / 2.0 +
+                                      dx * ((k1 + k2 * x0) / 3.0 +
+                                            dx * ((k2 + k3 * x0) / 4.0 +
+                                                  dx * ((k3 + k4 * x0) / 5.0 +
+                                                        dx * ((k4 + k5 * x0) / 6.0 +
+                                                              dx * ((k5 + fdgd * x0) / 7.0 + dx * fdgd / 8.0)))))));
             }
             break;
         }
@@ -783,7 +816,8 @@ T inner(Spline<T> const& f__, Spline<T> const& g__, int m__, int num_points__)
 }
 
 template <typename T>
-T inner(Spline<T> const& f__, Spline<T> const& g__, int m__)
+T
+inner(Spline<T> const& f__, Spline<T> const& g__, int m__)
 {
     return inner(f__, g__, m__, f__.num_points());
 }
