@@ -29,7 +29,8 @@
 
 namespace sirius {
 
-inline void check_gvec(fft::Gvec const& gvec__, Crystal_symmetry const& sym__)
+inline void
+check_gvec(fft::Gvec const& gvec__, Crystal_symmetry const& sym__)
 {
     PROFILE("sirius::check_gvec");
 
@@ -38,7 +39,7 @@ inline void check_gvec(fft::Gvec const& gvec__, Crystal_symmetry const& sym__)
         auto sm = sym__[isym].spg_op.R;
 
         for (int igloc = 0; igloc < gvec__.count(); igloc++) {
-            auto gv = gvec__.gvec<sddk::index_domain_t::local>(igloc);
+            auto gv = gvec__.gvec<index_domain_t::local>(igloc);
             /* apply symmetry operation to the G-vector */
             auto gv_rot = dot(gv, sm);
 
@@ -78,13 +79,14 @@ inline void check_gvec(fft::Gvec const& gvec__, Crystal_symmetry const& sym__)
                   << "rotated G-vector: " << gv_rot << std::endl
                   << "rotated G-vector index: " << ig_rot << std::endl
                   << "number of G-vectors: " << gvec__.num_gvec();
-                  RTE_THROW(s);
+                RTE_THROW(s);
             }
         }
     }
 }
 
-inline void check_gvec(fft::Gvec_shells const& gvec_shells__, Crystal_symmetry const& sym__)
+inline void
+check_gvec(fft::Gvec_shells const& gvec_shells__, Crystal_symmetry const& sym__)
 {
     /* check G-vector symmetries */
     for (int igloc = 0; igloc < gvec_shells__.gvec_count_remapped(); igloc++) {
@@ -119,6 +121,6 @@ inline void check_gvec(fft::Gvec_shells const& gvec_shells__, Crystal_symmetry c
     }
 }
 
-} // namespace
+} // namespace sirius
 
 #endif // __CHECK_GVEC_HPP__
