@@ -25,10 +25,10 @@
 #ifndef __ATOM_HPP__
 #define __ATOM_HPP__
 
-#include "sht/gaunt.hpp"
+#include "core/sht/gaunt.hpp"
+#include "core/profiler.hpp"
 #include "atom_symmetry_class.hpp"
 #include "function3d/spheric_function.hpp"
-#include "utils/profiler.hpp"
 
 namespace sirius {
 
@@ -49,28 +49,25 @@ class Atom
     r3::vector<double> vector_field_;
 
     /// Muffin-tin potential.
-    sddk::mdarray<double, 2> veff_;
+    mdarray<double, 2> veff_;
 
     /// Radial integrals of the Hamiltonian.
-    sddk::mdarray<double, 3> h_radial_integrals_;
+    mdarray<double, 3> h_radial_integrals_;
 
     /// Muffin-tin magnetic field.
-    sddk::mdarray<double, 2> beff_[3];
+    mdarray<double, 2> beff_[3];
 
     /// Radial integrals of the effective magnetic field.
-    sddk::mdarray<double, 4> b_radial_integrals_;
+    mdarray<double, 4> b_radial_integrals_;
 
     /// Maximum l for potential and magnetic field.
     int lmax_pot_{-1};
 
-    /// Offset in the block of local orbitals of the Hamiltonian and overlap matrices and in the eigen-vectors.
-    int offset_lo_{-1}; // TODO: better name for this
-
     /// Unsymmetrized (sampled over IBZ) occupation matrix of the L(S)DA+U method.
-    sddk::mdarray<std::complex<double>, 4> occupation_matrix_;
+    mdarray<std::complex<double>, 4> occupation_matrix_;
 
     /// U,J correction matrix of the L(S)DA+U method
-    sddk::mdarray<std::complex<double>, 4> uj_correction_matrix_;
+    mdarray<std::complex<double>, 4> uj_correction_matrix_;
 
     /// True if UJ correction is applied for the current atom.
     bool apply_uj_correction_{false};
@@ -86,7 +83,7 @@ class Atom
      *
      *  The ionic part of the D-operator matrix is added in the D_operator class, when it is initialized.
      */
-    sddk::mdarray<double, 3> d_mtrx_;
+    mdarray<double, 3> d_mtrx_;
 
   public:
     /// Constructor.
@@ -98,33 +95,32 @@ class Atom
     }
 
     /// Initialize atom.
-    inline void init(int offset_lo__)
+    inline void
+    init()
     {
-        offset_lo_ = offset_lo__;
-
         lmax_pot_ = type().parameters().lmax_pot();
 
         if (type().parameters().full_potential()) {
-            int lmmax = utils::lmmax(lmax_pot_);
+            int lmmax = sf::lmmax(lmax_pot_);
             int nrf   = type().indexr().size();
 
-            h_radial_integrals_ = sddk::mdarray<double, 3>(lmmax, nrf, nrf);
+            h_radial_integrals_ = mdarray<double, 3>({lmmax, nrf, nrf});
             h_radial_integrals_.zero();
 
             if (type().parameters().num_mag_dims()) {
-                b_radial_integrals_ = sddk::mdarray<double, 4>(lmmax, nrf, nrf, type().parameters().num_mag_dims());
+                b_radial_integrals_ = mdarray<double, 4>({lmmax, nrf, nrf, type().parameters().num_mag_dims()});
                 b_radial_integrals_.zero();
             }
 
-            occupation_matrix_ = sddk::mdarray<std::complex<double>, 4>(16, 16, 2, 2);
+            occupation_matrix_ = mdarray<std::complex<double>, 4>({16, 16, 2, 2});
 
-            uj_correction_matrix_ = sddk::mdarray<std::complex<double>, 4>(16, 16, 2, 2);
+            uj_correction_matrix_ = mdarray<std::complex<double>, 4>({16, 16, 2, 2});
         }
 
         if (!type().parameters().full_potential()) {
-            int nbf = type().mt_lo_basis_size();
-            d_mtrx_ = sddk::mdarray<double, 3>(nbf, nbf, type().parameters().num_mag_dims() + 1, sddk::memory_t::host,
-                                               "Atom::d_mtrx_");
+            int nbf = type().mt_basis_size();
+            d_mtrx_ = mdarray<double, 3>({nbf, nbf, type().parameters().num_mag_dims() + 1},
+                                         mdarray_label("Atom::d_mtrx_"));
             d_mtrx_.zero();
         }
     }
@@ -143,11 +139,12 @@ class Atom
      *        V_{\ell m}(r) & \ell > 0 \end{array} \right.
      *  \f]
      */
-    inline void generate_radial_integrals(sddk::device_t pu__, mpi::Communicator const& comm__)
+    inline void
+    generate_radial_integrals(device_t pu__, mpi::Communicator const& comm__)
     {
         PROFILE("sirius::Atom::generate_radial_integrals");
 
-        int lmmax        = utils::lmmax(lmax_pot_);
+        int lmmax        = sf::lmmax(lmax_pot_);
         int nmtp         = type().num_mt_points();
         int nrf          = type().indexr().size();
         int num_mag_dims = type().parameters().num_mag_dims();
@@ -156,9 +153,9 @@ class Atom
             RTE_THROW("not yet mpi parallel");
         }
 
-        sddk::splindex<sddk::splindex_t::block> spl_lm(lmmax, comm__.size(), comm__.rank());
+        splindex_block<> spl_lm(lmmax, n_blocks(comm__.size()), block_id(comm__.rank()));
 
-        auto l_by_lm = utils::l_by_lm(lmax_pot_);
+        auto l_by_lm = sf::l_by_lm(lmax_pot_);
 
         h_radial_integrals_.zero();
         if (num_mag_dims) {
@@ -197,9 +194,9 @@ class Atom
 
         auto& idx_ri = type().idx_radial_integrals();
 
-        sddk::mdarray<double, 1> result(idx_ri.size(1));
+        mdarray<double, 1> result({idx_ri.size(1)});
 
-        if (pu__ == sddk::device_t::GPU) {
+        if (pu__ == device_t::GPU) {
 #ifdef SIRIUS_GPU
             auto& rgrid    = type().radial_grid();
             auto& rf_coef  = type().rf_coef();
@@ -211,9 +208,8 @@ class Atom
                 #pragma omp for
                 for (int i = 0; i < nrf; i++) {
                     rf_spline[i].interpolate();
-                    std::copy(rf_spline[i].coeffs().at(sddk::memory_t::host),
-                              rf_spline[i].coeffs().at(sddk::memory_t::host) + nmtp * 4,
-                              rf_coef.at(sddk::memory_t::host, 0, 0, i));
+                    std::copy(rf_spline[i].coeffs().at(memory_t::host),
+                              rf_spline[i].coeffs().at(memory_t::host) + nmtp * 4, rf_coef.at(memory_t::host, 0, 0, i));
                     // cuda_async_copy_to_device(rf_coef.at<GPU>(0, 0, i), rf_coef.at<CPU>(0, 0, i), nmtp * 4 *
                     // sizeof(double), tid);
                 }
@@ -222,7 +218,7 @@ class Atom
                     v_spline[i].interpolate();
                 }
             }
-            rf_coef.copy_to(sddk::memory_t::device, stream_id(-1));
+            rf_coef.copy_to(memory_t::device, acc::stream_id(-1));
 
             #pragma omp parallel for
             for (int lm = 0; lm < lmmax; lm++) {
@@ -230,32 +226,32 @@ class Atom
                     for (int j = 0; j < num_mag_dims + 1; j++) {
                         int idx         = lm + lmmax * i + lmmax * nrf * j;
                         vrf_spline[idx] = rf_spline[i] * v_spline[lm + j * lmmax];
-                        std::memcpy(vrf_coef.at(sddk::memory_t::host, 0, 0, idx), vrf_spline[idx].coeffs().at(sddk::memory_t::host),
+                        std::memcpy(vrf_coef.at(memory_t::host, 0, 0, idx), vrf_spline[idx].coeffs().at(memory_t::host),
                                     nmtp * 4 * sizeof(double));
                         // cuda_async_copy_to_device(vrf_coef.at<GPU>(0, 0, idx), vrf_coef.at<CPU>(0, 0, idx), nmtp * 4
                         // *sizeof(double), tid);
                     }
                 }
             }
-            vrf_coef.copy_to(sddk::memory_t::device);
+            vrf_coef.copy_to(memory_t::device);
             PROFILE_STOP("sirius::Atom::generate_radial_integrals|interp");
 
-            result.allocate(sddk::memory_t::device);
-            spline_inner_product_gpu_v3(idx_ri.at(sddk::memory_t::device), (int)idx_ri.size(1), nmtp,
-                                        rgrid.x().at(sddk::memory_t::device), rgrid.dx().at(sddk::memory_t::device),
-                                        rf_coef.at(sddk::memory_t::device), vrf_coef.at(sddk::memory_t::device),
-                                        result.at(sddk::memory_t::device));
+            result.allocate(memory_t::device);
+            spline_inner_product_gpu_v3(idx_ri.at(memory_t::device), (int)idx_ri.size(1), nmtp,
+                                        rgrid.x().at(memory_t::device), rgrid.dx().at(memory_t::device),
+                                        rf_coef.at(memory_t::device), vrf_coef.at(memory_t::device),
+                                        result.at(memory_t::device));
             acc::sync();
-            //if (type().parameters().control().print_performance_) {
-            //    double tval = t2.stop();
-            //    DUMP("spline GPU integration performance: %12.6f GFlops",
-            //         1e-9 * double(idx_ri.size(1)) * nmtp * 85 / tval);
-            //}
-            result.copy_to(sddk::memory_t::host);
-            result.deallocate(sddk::memory_t::device);
+            // if (type().parameters().control().print_performance_) {
+            //     double tval = t2.stop();
+            //     DUMP("spline GPU integration performance: %12.6f GFlops",
+            //          1e-9 * double(idx_ri.size(1)) * nmtp * 85 / tval);
+            // }
+            result.copy_to(memory_t::host);
+            result.deallocate(memory_t::device);
 #endif
         }
-        if (pu__ == sddk::device_t::CPU) {
+        if (pu__ == device_t::CPU) {
             PROFILE_START("sirius::Atom::generate_radial_integrals|interp");
             #pragma omp parallel
             {
@@ -284,11 +280,11 @@ class Atom
             for (int j = 0; j < (int)idx_ri.size(1); j++) {
                 result(j) = inner(rf_spline[idx_ri(0, j)], vrf_spline[idx_ri(1, j)], 2);
             }
-            //if (type().parameters().control().print_performance_) {
-            //    double tval = t2.stop();
-            //    DUMP("spline CPU integration performance: %12.6f GFlops",
-            //         1e-9 * double(idx_ri.size(1)) * nmtp * 85 / tval);
-            //}
+            // if (type().parameters().control().print_performance_) {
+            //     double tval = t2.stop();
+            //     DUMP("spline CPU integration performance: %12.6f GFlops",
+            //          1e-9 * double(idx_ri.size(1)) * nmtp * 85 / tval);
+            // }
         }
 
         int n{0};
@@ -296,9 +292,9 @@ class Atom
             int l = l_by_lm[lm];
 
             for (int i2 = 0; i2 < type().indexr().size(); i2++) {
-                int l2 = type().indexr(i2).l;
+                int l2 = type().indexr(i2).am.l();
                 for (int i1 = 0; i1 <= i2; i1++) {
-                    int l1 = type().indexr(i1).l;
+                    int l1 = type().indexr(i1).am.l();
                     if ((l + l1 + l2) % 2 == 0) {
                         if (lm) {
                             h_radial_integrals_(lm, i1, i2) = h_radial_integrals_(lm, i2, i1) = result(n++);
@@ -314,55 +310,63 @@ class Atom
             }
         }
 
-        //if (type().parameters().control().print_checksum_) {
-        //    DUMP("checksum(h_radial_integrals): %18.10f", h_radial_integrals_.checksum());
-        //}
+        // if (type().parameters().control().print_checksum_) {
+        //     DUMP("checksum(h_radial_integrals): %18.10f", h_radial_integrals_.checksum());
+        // }
     }
 
     /// Return const reference to corresponding atom type object.
-    inline Atom_type const& type() const
+    inline Atom_type const&
+    type() const
     {
         return type_;
     }
 
     /// Return reference to corresponding atom symmetry class.
-    inline Atom_symmetry_class& symmetry_class()
+    inline Atom_symmetry_class&
+    symmetry_class()
     {
         return (*symmetry_class_);
     }
 
     /// Return const referenced to atom symmetry class.
-    inline Atom_symmetry_class const& symmetry_class() const
+    inline Atom_symmetry_class const&
+    symmetry_class() const
     {
         return (*symmetry_class_);
     }
 
     /// Return atom type id.
-    inline int type_id() const
+    inline int
+    type_id() const
     {
         return type_.id();
     }
 
     /// Return atom position in fractional coordinates.
-    inline r3::vector<double> const& position() const
+    inline r3::vector<double> const&
+    position() const
     {
         return position_;
     }
 
     /// Set atom position in fractional coordinates.
-    inline void set_position(r3::vector<double> position__)
+    inline void
+    set_position(r3::vector<double> position__)
     {
         position_ = position__;
     }
 
     /// Return vector field.
-    inline auto vector_field() const
+    inline auto
+    vector_field() const
     {
         return vector_field_;
     }
 
     /// Return id of the symmetry class.
-    inline int symmetry_class_id() const
+    inline int
+    symmetry_class_id() const
     {
         if (symmetry_class_ != nullptr) {
             return symmetry_class_->id();
@@ -371,50 +375,51 @@ class Atom
     }
 
     /// Set symmetry class of the atom.
-    inline void set_symmetry_class(std::shared_ptr<Atom_symmetry_class> symmetry_class__)
+    inline void
+    set_symmetry_class(std::shared_ptr<Atom_symmetry_class> symmetry_class__)
     {
         symmetry_class_ = std::move(symmetry_class__);
     }
 
     /// Set muffin-tin potential and magnetic field.
-    inline void set_nonspherical_potential(double* veff__, double* beff__[3])
+    inline void
+    set_nonspherical_potential(double* veff__, double* beff__[3])
     {
-        veff_ = sddk::mdarray<double, 2>(veff__, utils::lmmax(lmax_pot_), type().num_mt_points());
+        veff_ = mdarray<double, 2>({sf::lmmax(lmax_pot_), type().num_mt_points()}, veff__);
         for (int j = 0; j < 3; j++) {
-            beff_[j] = sddk::mdarray<double, 2>(beff__[j], utils::lmmax(lmax_pot_), type().num_mt_points());
+            beff_[j] = mdarray<double, 2>({sf::lmmax(lmax_pot_), type().num_mt_points()}, beff__[j]);
         }
     }
 
-    inline void sync_radial_integrals(mpi::Communicator const& comm__, int const rank__)
+    inline void
+    sync_radial_integrals(mpi::Communicator const& comm__, int const rank__)
     {
-        comm__.bcast(h_radial_integrals_.at(sddk::memory_t::host), (int)h_radial_integrals_.size(), rank__);
+        comm__.bcast(h_radial_integrals_.at(memory_t::host), (int)h_radial_integrals_.size(), rank__);
         if (type().parameters().num_mag_dims()) {
-            comm__.bcast(b_radial_integrals_.at(sddk::memory_t::host), (int)b_radial_integrals_.size(), rank__);
+            comm__.bcast(b_radial_integrals_.at(memory_t::host), (int)b_radial_integrals_.size(), rank__);
         }
     }
 
-    inline void sync_occupation_matrix(mpi::Communicator const& comm__, int const rank__)
+    inline void
+    sync_occupation_matrix(mpi::Communicator const& comm__, int const rank__)
     {
-        comm__.bcast(occupation_matrix_.at(sddk::memory_t::host), (int)occupation_matrix_.size(), rank__);
+        comm__.bcast(occupation_matrix_.at(memory_t::host), (int)occupation_matrix_.size(), rank__);
     }
 
-    inline int offset_lo() const
-    {
-        assert(offset_lo_ >= 0);
-        return offset_lo_;
-    }
-
-    inline double const* h_radial_integrals(int idxrf1, int idxrf2) const
+    inline double const*
+    h_radial_integrals(int idxrf1, int idxrf2) const
     {
         return &h_radial_integrals_(0, idxrf1, idxrf2);
     }
 
-    inline double* h_radial_integrals(int idxrf1, int idxrf2)
+    inline double*
+    h_radial_integrals(int idxrf1, int idxrf2)
     {
         return &h_radial_integrals_(0, idxrf1, idxrf2);
     }
 
-    inline double const* b_radial_integrals(int idxrf1, int idxrf2, int x) const
+    inline double const*
+    b_radial_integrals(int idxrf1, int idxrf2, int x) const
     {
         return &b_radial_integrals_(0, idxrf1, idxrf2, x);
     }
@@ -453,14 +458,16 @@ class Atom
                 }
                 case spin_block_t::ud: {
                     /* Bx - i By */
-                    zsum += gnt__[i].coef * std::complex<double>(b_radial_integrals_(gnt__[i].lm3, idxrf1__, idxrf2__, 1),
-                                                           -b_radial_integrals_(gnt__[i].lm3, idxrf1__, idxrf2__, 2));
+                    zsum += gnt__[i].coef *
+                            std::complex<double>(b_radial_integrals_(gnt__[i].lm3, idxrf1__, idxrf2__, 1),
+                                                 -b_radial_integrals_(gnt__[i].lm3, idxrf1__, idxrf2__, 2));
                     break;
                 }
                 case spin_block_t::du: {
                     /* Bx + i By */
-                    zsum += gnt__[i].coef * std::complex<double>(b_radial_integrals_(gnt__[i].lm3, idxrf1__, idxrf2__, 1),
-                                                           b_radial_integrals_(gnt__[i].lm3, idxrf1__, idxrf2__, 2));
+                    zsum += gnt__[i].coef *
+                            std::complex<double>(b_radial_integrals_(gnt__[i].lm3, idxrf1__, idxrf2__, 1),
+                                                 b_radial_integrals_(gnt__[i].lm3, idxrf1__, idxrf2__, 2));
                     break;
                 }
             }
@@ -468,100 +475,118 @@ class Atom
         return zsum;
     }
 
-    inline int num_mt_points() const
+    inline int
+    num_mt_points() const
     {
         return type_.num_mt_points();
     }
 
-    inline Radial_grid<double> const& radial_grid() const
+    inline Radial_grid<double> const&
+    radial_grid() const
     {
         return type_.radial_grid();
     }
 
-    inline double radial_grid(int idx) const
+    inline double
+    radial_grid(int idx) const
     {
         return type_.radial_grid(idx);
     }
 
-    inline double mt_radius() const
+    inline double
+    mt_radius() const
     {
         return type_.mt_radius();
     }
 
-    inline int zn() const
+    inline int
+    zn() const
     {
         return type_.zn();
     }
 
-    inline int mt_basis_size() const
+    inline int
+    mt_basis_size() const
     {
         return type_.mt_basis_size();
     }
 
-    inline int mt_aw_basis_size() const
+    inline int
+    mt_aw_basis_size() const
     {
         return type_.mt_aw_basis_size();
     }
 
-    inline int mt_lo_basis_size() const
+    inline int
+    mt_lo_basis_size() const
     {
         return type_.mt_lo_basis_size();
     }
 
-    inline void set_occupation_matrix(const std::complex<double>* source)
+    inline void
+    set_occupation_matrix(const std::complex<double>* source)
     {
-        std::memcpy(occupation_matrix_.at(sddk::memory_t::host), source, 16 * 16 * 2 * 2 * sizeof(std::complex<double>));
+        std::memcpy(occupation_matrix_.at(memory_t::host), source, 16 * 16 * 2 * 2 * sizeof(std::complex<double>));
         apply_uj_correction_ = false;
     }
 
-    inline void get_occupation_matrix(std::complex<double>* destination)
+    inline void
+    get_occupation_matrix(std::complex<double>* destination)
     {
-        std::memcpy(destination, occupation_matrix_.at(sddk::memory_t::host), 16 * 16 * 2 * 2 * sizeof(std::complex<double>));
+        std::memcpy(destination, occupation_matrix_.at(memory_t::host), 16 * 16 * 2 * 2 * sizeof(std::complex<double>));
     }
 
-    inline void set_uj_correction_matrix(const int l, const std::complex<double>* source)
+    inline void
+    set_uj_correction_matrix(const int l, const std::complex<double>* source)
     {
         uj_correction_l_ = l;
-        std::memcpy(uj_correction_matrix_.at(sddk::memory_t::host), source, 16 * 16 * 2 * 2 * sizeof(std::complex<double>));
+        std::memcpy(uj_correction_matrix_.at(memory_t::host), source, 16 * 16 * 2 * 2 * sizeof(std::complex<double>));
         apply_uj_correction_ = true;
     }
 
-    inline bool apply_uj_correction()
+    inline bool
+    apply_uj_correction()
     {
         return apply_uj_correction_;
     }
 
-    inline int uj_correction_l()
+    inline int
+    uj_correction_l()
     {
         return uj_correction_l_;
     }
 
-    inline auto uj_correction_matrix(int lm1, int lm2, int ispn1, int ispn2)
+    inline auto
+    uj_correction_matrix(int lm1, int lm2, int ispn1, int ispn2)
     {
         return uj_correction_matrix_(lm1, lm2, ispn1, ispn2);
     }
 
-    inline double& d_mtrx(int xi1, int xi2, int iv)
+    inline double&
+    d_mtrx(int xi1, int xi2, int iv)
     {
         return d_mtrx_(xi1, xi2, iv);
     }
 
-    inline double const& d_mtrx(int xi1, int xi2, int iv) const
+    inline double const&
+    d_mtrx(int xi1, int xi2, int iv) const
     {
         return d_mtrx_(xi1, xi2, iv);
     }
 
-    inline auto const& d_mtrx() const
+    inline auto const&
+    d_mtrx() const
     {
         return d_mtrx_;
     }
 
-    inline auto& d_mtrx()
+    inline auto&
+    d_mtrx()
     {
         return d_mtrx_;
     }
 };
 
-} // namespace
+} // namespace sirius
 
 #endif // __ATOM_H__
