@@ -23,7 +23,7 @@
  */
 
 #include "atom_symmetry_class.hpp"
-#include "linalg/eigensolver.hpp"
+#include "core/la/eigensolver.hpp"
 
 namespace sirius {
 
@@ -35,39 +35,39 @@ Atom_symmetry_class::Atom_symmetry_class(int id__, Atom_type const& atom_type__)
         RTE_THROW("atom type is not initialized");
     }
 
-    surface_derivatives_ = sddk::mdarray<double, 2>(3, atom_type_.mt_radial_basis_size());
+    int nl        = atom_type_.indexr().lmax() + 1;
+    int max_order = atom_type_.indexr().max_order();
+    int nrf       = atom_type_.mt_radial_basis_size();
 
-    radial_functions_ = sddk::mdarray<double, 3>(atom_type_.num_mt_points(), atom_type_.mt_radial_basis_size(), 2);
+    surface_derivatives_ = mdarray<double, 2>({3, nrf});
 
-    h_spherical_integrals_ = sddk::mdarray<double, 2>(atom_type_.mt_radial_basis_size(), atom_type_.mt_radial_basis_size());
+    radial_functions_ = mdarray<double, 3>({atom_type_.num_mt_points(), nrf, 2});
+
+    h_spherical_integrals_ = mdarray<double, 2>({nrf, nrf});
     h_spherical_integrals_.zero();
 
-    o_radial_integrals_ = sddk::mdarray<double, 3>(atom_type_.indexr().lmax() + 1, atom_type_.indexr().max_num_rf(),
-                                             atom_type_.indexr().max_num_rf());
+    o_radial_integrals_ = mdarray<double, 3>({nl, max_order, max_order});
     o_radial_integrals_.zero();
 
-    so_radial_integrals_ = sddk::mdarray<double, 3>(atom_type_.indexr().lmax() + 1, atom_type_.indexr().max_num_rf(),
-                                              atom_type_.indexr().max_num_rf());
+    so_radial_integrals_ = mdarray<double, 3>({nl, max_order, max_order});
     so_radial_integrals_.zero();
 
     if (atom_type_.parameters().valence_relativity() == relativity_t::iora) {
-        o1_radial_integrals_ = sddk::mdarray<double, 2>(atom_type_.mt_radial_basis_size(), atom_type_.mt_radial_basis_size());
+        o1_radial_integrals_ = mdarray<double, 2>({nrf, nrf});
         o1_radial_integrals_.zero();
     }
 
     /* copy descriptors because enu is different between atom classes */
-    aw_descriptors_.resize(atom_type_.num_aw_descriptors());
-    for (int i = 0; i < num_aw_descriptors(); i++) {
-        aw_descriptors_[i] = atom_type_.aw_descriptor(i);
+    for (int i = 0; i < atom_type_.num_aw_descriptors(); i++) {
+        aw_descriptors_.push_back(atom_type_.aw_descriptor(i));
     }
 
-    lo_descriptors_.resize(atom_type_.num_lo_descriptors());
-    for (int i = 0; i < num_lo_descriptors(); i++) {
-        lo_descriptors_[i] = atom_type_.lo_descriptor(i);
+    for (int i = 0; i < atom_type_.num_lo_descriptors(); i++) {
+        lo_descriptors_.push_back(atom_type_.lo_descriptor(i));
     }
 
     ae_core_charge_density_.resize(atom_type_.num_mt_points());
-    std::memset(&ae_core_charge_density_[0], 0, atom_type_.num_mt_points() * sizeof(double));
+    std::fill(ae_core_charge_density_.begin(), ae_core_charge_density_.end(), 0);
 }
 
 void
@@ -81,8 +81,8 @@ Atom_symmetry_class::generate_aw_radial_functions(relativity_t rel__)
     {
         Spline<double> s(atom_type_.radial_grid());
 
-        std::vector<double>   p;
-        std::vector<double>   rdudr;
+        std::vector<double> p;
+        std::vector<double> rdudr;
         std::array<double, 2> uderiv;
 
         #pragma omp for schedule(dynamic, 1)
@@ -90,7 +90,7 @@ Atom_symmetry_class::generate_aw_radial_functions(relativity_t rel__)
             for (int order = 0; order < (int)aw_descriptor(l).size(); order++) {
                 auto rsd = aw_descriptor(l)[order];
 
-                int idxrf = atom_type_.indexr().index_by_l_order(l, order);
+                auto idxrf = atom_type_.indexr().index_of(angular_momentum(l), order);
 
                 solver.solve(rel__, rsd.dme, rsd.l, rsd.enu, p, rdudr, uderiv);
 
@@ -111,7 +111,7 @@ Atom_symmetry_class::generate_aw_radial_functions(relativity_t rel__)
 
                 /* orthogonalize to previous radial functions */
                 for (int order1 = 0; order1 < order; order1++) {
-                    int idxrf1 = atom_type_.indexr().index_by_l_order(l, order1);
+                    auto idxrf1 = atom_type_.indexr().index_of(angular_momentum(l), order1);
 
                     for (int ir = 0; ir < nmtp; ir++) {
                         s(ir) = radial_functions_(ir, idxrf, 0) * radial_functions_(ir, idxrf1, 0);
@@ -157,7 +157,7 @@ Atom_symmetry_class::generate_aw_radial_functions(relativity_t rel__)
             } // order
             /* divide by r */
             for (int order = 0; order < (int)aw_descriptor(l).size(); order++) {
-                int idxrf = atom_type_.indexr().index_by_l_order(l, order);
+                auto idxrf = atom_type_.indexr().index_of(angular_momentum(l), order);
                 for (int ir = 0; ir < nmtp; ir++) {
                     radial_functions_(ir, idxrf, 0) *= atom_type_.radial_grid().x_inv(ir);
                 }
@@ -185,7 +185,7 @@ Atom_symmetry_class::generate_lo_radial_functions(relativity_t rel__)
 
         std::vector<std::vector<double>> p(num_rs);
         std::vector<std::vector<double>> rdudr(num_rs);
-        std::array<double, 2>            uderiv;
+        std::array<double, 2> uderiv;
 
         for (int order = 0; order < num_rs; order++) {
             auto rsd = lo_descriptor(idxlo).rsd_set[order];
@@ -213,7 +213,7 @@ Atom_symmetry_class::generate_lo_radial_functions(relativity_t rel__)
             a[order][1] = uderiv[0];
             a[order][2] = uderiv[1];
 
-            for (int i: {0, 1, 2}) {
+            for (int i : {0, 1, 2}) {
                 rderiv[order][i] = a[order][i];
             }
         }
@@ -234,13 +234,13 @@ Atom_symmetry_class::generate_lo_radial_functions(relativity_t rel__)
             s << std::endl;
             s << "atom: " << atom_type_.label() << std::endl
               << "zn: " << atom_type_.zn() << std::endl
-              << "l: " << lo_descriptor(idxlo).l << std::endl;
+              << "l: " << lo_descriptor(idxlo).am.l() << std::endl;
             s << "gesv returned " << info;
             RTE_THROW(s);
         }
 
         /* index of local orbital radial function */
-        int idxrf = atom_type_.indexr().index_by_idxlo(idxlo);
+        auto idxrf = atom_type_.indexr().index_of(rf_lo_index(idxlo));
         /* zero surface derivatives */
         for (int i : {0, 1, 2}) {
             surface_derivatives_(i, idxrf) = 0;
@@ -286,7 +286,7 @@ Atom_symmetry_class::generate_lo_radial_functions(relativity_t rel__)
             }
             s << std::endl;
             s << "surface derivative : " << surface_derivatives_(0, idxrf);
-            WARNING(s);
+            RTE_WARNING(s);
         }
     }
 
@@ -303,9 +303,9 @@ Atom_symmetry_class::orthogonalize_radial_functions()
     /* orthogonalize local orbitals */
     for (int l = 0; l <= atom_type().indexr().lmax_lo(); l++) {
         for (int j = 0; j < atom_type().indexr().num_lo(l); j++) {
-            int idxrf = atom_type().indexr().index_by_l_order(l, j + atom_type_.aw_order(l));
+            int idxrf = atom_type().indexr().index_of(angular_momentum(l), j + atom_type_.aw_order(l));
             for (int j1 = 0; j1 < j; j1++) {
-                int idxrf1 = atom_type().indexr().index_by_l_order(l, j1 + atom_type().aw_order(l));
+                int idxrf1 = atom_type().indexr().index_of(angular_momentum(l), j1 + atom_type().aw_order(l));
 
                 for (int ir = 0; ir < nmtp; ir++) {
                     s(ir) = radial_functions_(ir, idxrf, 0) * radial_functions_(ir, idxrf1, 0);
@@ -352,18 +352,18 @@ Atom_symmetry_class::check_lo_linear_independence(double tol__)
 {
     int nmtp = atom_type_.num_mt_points();
 
-    Spline<double>  s(atom_type_.radial_grid());
+    Spline<double> s(atom_type_.radial_grid());
     la::dmatrix<double> loprod(num_lo_descriptors(), num_lo_descriptors());
     loprod.zero();
     for (int idxlo1 = 0; idxlo1 < num_lo_descriptors(); idxlo1++) {
 
-        int idxrf1 = atom_type_.indexr().index_by_idxlo(idxlo1);
+        int idxrf1 = atom_type_.indexr().index_of(rf_lo_index(idxlo1));
 
         for (int idxlo2 = 0; idxlo2 < num_lo_descriptors(); idxlo2++) {
 
-            int idxrf2 = atom_type_.indexr().index_by_idxlo(idxlo2);
+            int idxrf2 = atom_type_.indexr().index_of(rf_lo_index(idxlo2));
 
-            if (lo_descriptor(idxlo1).l == lo_descriptor(idxlo2).l) {
+            if (lo_descriptor(idxlo1).am == lo_descriptor(idxlo2).am) {
 
                 for (int ir = 0; ir < nmtp; ir++) {
                     s(ir) = radial_functions_(ir, idxrf1, 0) * radial_functions_(ir, idxrf2, 0);
@@ -373,8 +373,8 @@ Atom_symmetry_class::check_lo_linear_independence(double tol__)
         }
     }
 
-    sddk::mdarray<double, 2> ovlp(num_lo_descriptors(), num_lo_descriptors());
-    loprod >> ovlp;
+    mdarray<double, 2> ovlp({num_lo_descriptors(), num_lo_descriptors()});
+    copy(loprod, ovlp);
 
     auto stdevp = la::Eigensolver_factory("lapack");
 
@@ -443,7 +443,7 @@ Atom_symmetry_class::dump_lo()
     for (int ir = 0; ir < atom_type_.num_mt_points(); ir++) {
         fprintf(fout, "%f ", atom_type_.radial_grid(ir));
         for (int idxlo = 0; idxlo < num_lo_descriptors(); idxlo++) {
-            int idxrf = atom_type_.indexr().index_by_idxlo(idxlo);
+            int idxrf = atom_type_.indexr().index_of(rf_lo_index(idxlo));
             fprintf(fout, "%f ", radial_functions_(ir, idxrf, 0));
         }
         fprintf(fout, "\n");
@@ -457,7 +457,7 @@ Atom_symmetry_class::dump_lo()
     for (int ir = 0; ir < atom_type_.num_mt_points(); ir++) {
         fprintf(fout, "%f ", atom_type_.radial_grid(ir));
         for (int idxlo = 0; idxlo < num_lo_descriptors(); idxlo++) {
-            int idxrf = atom_type_.indexr().index_by_idxlo(idxlo);
+            int idxrf = atom_type_.indexr().index_of(rf_lo_index(idxlo));
             fprintf(fout, "%f ", radial_functions_(ir, idxrf, 1));
         }
         fprintf(fout, "\n");
@@ -474,19 +474,19 @@ Atom_symmetry_class::set_spherical_potential(std::vector<double> const& vs__)
 
     spherical_potential_ = vs__;
 
-    //HDF5_tree fout("mt_potential.h5", true);
-    //fout.write("potential", spherical_potential_);
+    // HDF5_tree fout("mt_potential.h5", true);
+    // fout.write("potential", spherical_potential_);
 
     ///* write spherical potential */
-    //std::stringstream sstr;
-    //sstr << "mt_spheric_potential_" << id_ << ".dat";
-    //FILE* fout = fopen(sstr.str().c_str(), "w");
+    // std::stringstream sstr;
+    // sstr << "mt_spheric_potential_" << id_ << ".dat";
+    // FILE* fout = fopen(sstr.str().c_str(), "w");
 
-    //for (int ir = 0; ir < atom_type_.num_mt_points(); ir++) {
-    //    double r = atom_type_.radial_grid(ir);
-    //    fprintf(fout, "%20.10f %20.10f \n", r, spherical_potential_[ir] + atom_type_.zn() / r);
-    //}
-    //fclose(fout);
+    // for (int ir = 0; ir < atom_type_.num_mt_points(); ir++) {
+    //     double r = atom_type_.radial_grid(ir);
+    //     fprintf(fout, "%20.10f %20.10f \n", r, spherical_potential_[ir] + atom_type_.zn() / r);
+    // }
+    // fclose(fout);
 }
 
 void
@@ -521,8 +521,10 @@ Atom_symmetry_class::find_enu(relativity_t rel__)
 
     #pragma omp parallel for
     for (size_t i = 0; i < rs_with_auto_enu.size(); i++) {
-        auto   rsd     = rs_with_auto_enu[i];
-        double new_enu = Enu_finder(rel__, atom_type_.zn(), rsd->n, rsd->l, atom_type_.radial_grid(), spherical_potential_, rsd->enu).enu();
+        auto rsd       = rs_with_auto_enu[i];
+        double new_enu = Enu_finder(rel__, atom_type_.zn(), rsd->n, rsd->l, atom_type_.radial_grid(),
+                                    spherical_potential_, rsd->enu)
+                                 .enu();
         /* update linearization energy only if its change is above a threshold */
         if (std::abs(new_enu - rsd->enu) > atom_type_.parameters().cfg().settings().auto_enu_tol()) {
             rsd->enu           = new_enu;
@@ -547,14 +549,9 @@ Atom_symmetry_class::generate_radial_functions(relativity_t rel__)
     generate_lo_radial_functions(rel__);
 
     if (atom_type().parameters().cfg().control().ortho_rf()) {
-       orthogonalize_radial_functions();
+        orthogonalize_radial_functions();
     }
 
-//= #ifdef __PRINT_OBJECT_CHECKSUM
-//=     DUMP("checksum(spherical_potential): %18.10f", mdarray<double, 1>(spherical_potential_.data(), atom_type_.num_mt_points()).checksum());
-//=     DUMP("checksum(radial_functions): %18.10f", radial_functions_.checksum());
-//= #endif
-
     //** if (true)
     //** {
     //**     std::stringstream s;
@@ -580,19 +577,19 @@ Atom_symmetry_class::sync_radial_functions(mpi::Communicator const& comm__, int
 {
     /* don't broadcast Hamiltonian radial functions, because they are used locally */
     int size = (int)(radial_functions_.size(0) * radial_functions_.size(1));
-    comm__.bcast(radial_functions_.at(sddk::memory_t::host), size, rank__);
-    comm__.bcast(surface_derivatives_.at(sddk::memory_t::host), (int)surface_derivatives_.size(), rank__);
+    comm__.bcast(radial_functions_.at(memory_t::host), size, rank__);
+    comm__.bcast(surface_derivatives_.at(memory_t::host), (int)surface_derivatives_.size(), rank__);
     // TODO: sync enu to pass to Exciting / Elk
 }
 
 void
 Atom_symmetry_class::sync_radial_integrals(mpi::Communicator const& comm__, int const rank__)
 {
-    comm__.bcast(h_spherical_integrals_.at(sddk::memory_t::host), (int)h_spherical_integrals_.size(), rank__);
-    comm__.bcast(o_radial_integrals_.at(sddk::memory_t::host), (int)o_radial_integrals_.size(), rank__);
-    comm__.bcast(so_radial_integrals_.at(sddk::memory_t::host), (int)so_radial_integrals_.size(), rank__);
+    comm__.bcast(h_spherical_integrals_.at(memory_t::host), (int)h_spherical_integrals_.size(), rank__);
+    comm__.bcast(o_radial_integrals_.at(memory_t::host), (int)o_radial_integrals_.size(), rank__);
+    comm__.bcast(so_radial_integrals_.at(memory_t::host), (int)so_radial_integrals_.size(), rank__);
     if (atom_type_.parameters().valence_relativity() == relativity_t::iora) {
-        comm__.bcast(o1_radial_integrals_.at(sddk::memory_t::host), (int)o1_radial_integrals_.size(), rank__);
+        comm__.bcast(o1_radial_integrals_.at(memory_t::host), (int)o1_radial_integrals_.size(), rank__);
     }
 }
 
@@ -626,16 +623,17 @@ Atom_symmetry_class::generate_radial_integrals(relativity_t rel__)
         for (int i1 = 0; i1 < atom_type_.mt_radial_basis_size(); i1++) {
             for (int i2 = 0; i2 < atom_type_.mt_radial_basis_size(); i2++) {
                 /* for spherical part of potential integrals are diagonal in l */
-                if (atom_type_.indexr(i1).l == atom_type_.indexr(i2).l) {
-                    int ll = atom_type_.indexr(i1).l * (atom_type_.indexr(i1).l + 1);
+                if (atom_type_.indexr(i1).am.l() == atom_type_.indexr(i2).am.l()) {
+                    int ll = atom_type_.indexr(i1).am.l() * (atom_type_.indexr(i1).am.l() + 1);
                     for (int ir = 0; ir < nmtp; ir++) {
                         double Minv = 1.0 / (1 - spherical_potential_[ir] * sq_alpha_half);
                         /* u_1(r) * u_2(r) */
                         double t0 = radial_functions_(ir, i1, 0) * radial_functions_(ir, i2, 0);
                         /* r*u'_1(r) * r*u'_2(r) */
                         double t1 = radial_functions_(ir, i1, 1) * radial_functions_(ir, i2, 1);
-                        s(ir)     = 0.5 * t1 * Minv + t0 * (0.5 * ll * Minv + spherical_potential_[ir] *
-                            std::pow(atom_type_.radial_grid(ir), 2));
+                        s(ir)     = 0.5 * t1 * Minv +
+                                t0 * (0.5 * ll * Minv +
+                                      spherical_potential_[ir] * std::pow(atom_type_.radial_grid(ir), 2));
                     }
                     h_spherical_integrals_(i1, i2) = s.interpolate().integrate(0) / y00;
                 }
@@ -649,12 +647,12 @@ Atom_symmetry_class::generate_radial_integrals(relativity_t rel__)
         Spline<double> s(atom_type_.radial_grid());
         #pragma omp for
         for (int l = 0; l <= atom_type_.indexr().lmax(); l++) {
-            int nrf = atom_type_.indexr().num_rf(l);
+            int nrf = atom_type_.indexr().max_order(l);
 
             for (int order1 = 0; order1 < nrf; order1++) {
-                int idxrf1 = atom_type_.indexr().index_by_l_order(l, order1);
+                auto idxrf1 = atom_type_.indexr().index_of(angular_momentum(l), order1);
                 for (int order2 = 0; order2 < nrf; order2++) {
-                    int idxrf2 = atom_type_.indexr().index_by_l_order(l, order2);
+                    auto idxrf2 = atom_type_.indexr().index_of(angular_momentum(l), order2);
                     if (order1 == order2) {
                         o_radial_integrals_(l, order1, order2) = 1.0;
                     } else {
@@ -674,8 +672,8 @@ Atom_symmetry_class::generate_radial_integrals(relativity_t rel__)
             Spline<double> s(atom_type_.radial_grid());
             for (int i2 = 0; i2 < atom_type_.mt_radial_basis_size(); i2++) {
                 /* for spherical part of potential integrals are diagonal in l */
-                if (atom_type_.indexr(i1).l == atom_type_.indexr(i2).l) {
-                    int ll = atom_type_.indexr(i1).l * (atom_type_.indexr(i1).l + 1);
+                if (atom_type_.indexr(i1).am.l() == atom_type_.indexr(i2).am.l()) {
+                    int ll = atom_type_.indexr(i1).am.l() * (atom_type_.indexr(i1).am.l() + 1);
                     for (int ir = 0; ir < nmtp; ir++) {
                         double Minv = std::pow(1 - spherical_potential_[ir] * sq_alpha_half, -2);
                         /* u_1(r) * u_2(r) */
@@ -704,22 +702,22 @@ Atom_symmetry_class::generate_radial_integrals(relativity_t rel__)
 
         so_radial_integrals_.zero();
         for (int l = 0; l <= atom_type_.indexr().lmax(); l++) {
-            int nrf = atom_type_.indexr().num_rf(l);
+            int nrf = atom_type_.indexr().max_order(l);
 
             for (int order1 = 0; order1 < nrf; order1++) {
-                int idxrf1 = atom_type_.indexr().index_by_l_order(l, order1);
+                auto idxrf1 = atom_type_.indexr().index_of(angular_momentum(l), order1);
                 for (int order2 = 0; order2 < nrf; order2++) {
-                    int idxrf2 = atom_type_.indexr().index_by_l_order(l, order2);
+                    auto idxrf2 = atom_type_.indexr().index_of(angular_momentum(l), order2);
 
                     for (int ir = 0; ir < nmtp; ir++) {
                         double M = 1.0 - 2 * soc * spherical_potential_[ir];
                         /* first part <f| dVe / dr |f'> */
-                        s(ir) = radial_functions_(ir, idxrf1, 0) * radial_functions_(ir, idxrf2, 0) *
-                                soc * ve.deriv(1, ir) / pow(M, 2);
+                        s(ir) = radial_functions_(ir, idxrf1, 0) * radial_functions_(ir, idxrf2, 0) * soc *
+                                ve.deriv(1, ir) / pow(M, 2);
 
                         /* second part <f| d(z/r) / dr |f'> */
-                        s1(ir) = radial_functions_(ir, idxrf1, 0) * radial_functions_(ir, idxrf2, 0) *
-                                 soc * atom_type_.zn() / pow(M, 2);
+                        s1(ir) = radial_functions_(ir, idxrf1, 0) * radial_functions_(ir, idxrf2, 0) * soc *
+                                 atom_type_.zn() / pow(M, 2);
                     }
                     s.interpolate();
                     s1.interpolate();
@@ -740,7 +738,7 @@ Atom_symmetry_class::write_enu(mpi::pstdout& pout) const
         for (size_t order = 0; order < aw_descriptor(l).size(); order++) {
             auto& rsd = aw_descriptor(l)[order];
             if (rsd.auto_enu) {
-                pout << rsd; //printf("n = %2i   l = %2i   order = %i   enu = %12.6f", rsd.n, rsd.l, order, rsd.enu);
+                pout << rsd; // printf("n = %2i   l = %2i   order = %i   enu = %12.6f", rsd.n, rsd.l, order, rsd.enu);
                 if (rsd.new_enu_found) {
                     pout << "  +";
                 }
@@ -754,7 +752,7 @@ Atom_symmetry_class::write_enu(mpi::pstdout& pout) const
         for (size_t order = 0; order < lo_descriptor(idxlo).rsd_set.size(); order++) {
             auto& rsd = lo_descriptor(idxlo).rsd_set[order];
             if (rsd.auto_enu) {
-                //pout.printf("n = %2i   l = %2i   order = %i   enu = %12.6f", rsd.n, rsd.l, order, rsd.enu);
+                // pout.printf("n = %2i   l = %2i   order = %i   enu = %12.6f", rsd.n, rsd.l, order, rsd.enu);
                 pout << rsd;
                 if (rsd.new_enu_found) {
                     pout << "  +";
@@ -836,7 +834,7 @@ Atom_symmetry_class::generate_core_charge_density(relativity_t core_rel__)
         level_energy[ist] = -1.0 * atom_type_.zn() / 2 / std::pow(double(atom_type_.atomic_level(ist).n), 2);
     }
 
-    sddk::mdarray<double, 2> rho_t(rgrid.num_points(), atom_type_.num_atomic_levels());
+    mdarray<double, 2> rho_t({rgrid.num_points(), atom_type_.num_atomic_levels()});
     rho_t.zero();
     #pragma omp parallel for
     for (int ist = 0; ist < atom_type_.num_atomic_levels(); ist++) {
@@ -869,7 +867,8 @@ Atom_symmetry_class::generate_core_charge_density(relativity_t core_rel__)
     //    #pragma omp for
     //    for (int ist = 0; ist < atom_type_.num_atomic_levels(); ist++) {
     //        if (atom_type_.atomic_level(ist).core) {
-    //            Bound_state bs(core_rel__, atom_type_.zn(), atom_type_.atomic_level(ist).n, atom_type_.atomic_level(ist).l,
+    //            Bound_state bs(core_rel__, atom_type_.zn(), atom_type_.atomic_level(ist).n,
+    //            atom_type_.atomic_level(ist).l,
     //                           atom_type_.atomic_level(ist).k, rgrid, veff, level_energy[ist]);
 
     //            auto& rho = bs.rho();
@@ -907,4 +906,3 @@ Atom_symmetry_class::generate_core_charge_density(relativity_t core_rel__)
 }
 
 } // namespace sirius
-
