@@ -23,11 +23,12 @@
  */
 
 #include "atom_type.hpp"
+#include "core/ostream_tools.hpp"
 
 namespace sirius {
 
 void
-Atom_type::init(int offset_lo__)
+Atom_type::init()
 {
     PROFILE("sirius::Atom_type::init");
 
@@ -36,8 +37,6 @@ Atom_type::init(int offset_lo__)
         RTE_THROW("can't initialize twice");
     }
 
-    offset_lo_ = offset_lo__;
-
     /* read data from file if it exists */
     read_input(file_name_);
 
@@ -91,42 +90,69 @@ Atom_type::init(int offset_lo__)
         if (max_aw_order_ > 3) {
             RTE_THROW("maximum aw order > 3");
         }
+        /* build radial function index */
+        for (auto aw : aw_descriptors_) {
+            RTE_ASSERT(aw.size() <= 3);
+            for (auto e : aw) {
+                indexr_.add(angular_momentum(e.l));
+            }
+        }
+        for (auto e : lo_descriptors_) {
+            indexr_.add_lo(e.am);
+        }
+    } else {
+        for (int i = 0; i < this->num_beta_radial_functions(); i++) {
+            auto idxrf = rf_index(i);
+            if (this->spin_orbit_coupling()) {
+                if (this->beta_radial_function(idxrf).first.l() == 0) {
+                    indexr_.add(this->beta_radial_function(idxrf).first);
+                } else {
+                    indexr_.add(this->beta_radial_function(idxrf).first,
+                                this->beta_radial_function(rf_index(i + 1)).first);
+                    i++;
+                }
+            } else {
+                indexr_.add(this->beta_radial_function(idxrf).first);
+            }
+        }
+        /* check inner consistency of the index */
+        for (auto e : this->indexr_) {
+            if (e.am != this->beta_radial_function(e.idxrf).first) {
+                RTE_THROW("wrong order of beta radial functions");
+            }
+        }
     }
 
-    /* initialize index of radial functions */
-    indexr_.init(aw_descriptors_, lo_descriptors_);
-
     /* initialize index of muffin-tin basis functions */
-    indexb_.init(indexr_);
-
+    indexb_ = basis_functions_index(indexr_, false);
 
     /* initialize index for wave functions */
     if (ps_atomic_wfs_.size()) {
-        for (size_t i = 0; i < ps_atomic_wfs_.size(); i++) {
-            indexr_wfs_.add(ps_atomic_wfs_[i].am);
+        for (auto& e : ps_atomic_wfs_) {
+            indexr_wfs_.add(e.am);
         }
-        indexb_wfs_ = sirius::experimental::basis_functions_index(indexr_wfs_, false);
+        indexb_wfs_ = basis_functions_index(indexr_wfs_, false);
         if (static_cast<int>(ps_atomic_wfs_.size()) != indexr_wfs_.size()) {
             RTE_THROW("wrong size of atomic orbital list");
         }
     }
 
     if (hubbard_correction_) {
-        indexb_hub_ = sirius::experimental::basis_functions_index(indexr_hub_, false);
+        indexb_hub_ = basis_functions_index(indexr_hub_, false);
     }
 
     if (!parameters_.full_potential()) {
         RTE_ASSERT(mt_radial_basis_size() == num_beta_radial_functions());
-        RTE_ASSERT(lmax_beta() == indexr().lmax());
+        // RTE_ASSERT(lmax_beta() == indexr1().lmax());
     }
 
     /* get number of valence electrons */
     num_valence_electrons_ = zn_ - num_core_electrons_;
 
-    int lmmax_pot = utils::lmmax(parameters_.lmax_pot());
+    int lmmax_pot = sf::lmmax(parameters_.lmax_pot());
 
     if (parameters_.full_potential()) {
-        auto l_by_lm = utils::l_by_lm(parameters_.lmax_pot());
+        auto l_by_lm = sf::l_by_lm(parameters_.lmax_pot());
 
         /* index the non-zero radial integrals */
         std::vector<std::pair<int, int>> non_zero_elements;
@@ -135,9 +161,9 @@ Atom_type::init(int offset_lo__)
             int l = l_by_lm[lm];
 
             for (int i2 = 0; i2 < indexr().size(); i2++) {
-                int l2 = indexr(i2).l;
+                int l2 = indexr(i2).am.l();
                 for (int i1 = 0; i1 <= i2; i1++) {
-                    int l1 = indexr(i1).l;
+                    int l1 = indexr(i1).am.l();
                     if ((l + l1 + l2) % 2 == 0) {
                         if (lm) {
                             non_zero_elements.push_back(std::pair<int, int>(i2, lm + lmmax_pot * i1));
@@ -150,24 +176,24 @@ Atom_type::init(int offset_lo__)
                 }
             }
         }
-        idx_radial_integrals_ = sddk::mdarray<int, 2>(2, non_zero_elements.size());
+        idx_radial_integrals_ = mdarray<int, 2>({2, non_zero_elements.size()});
         for (int j = 0; j < (int)non_zero_elements.size(); j++) {
             idx_radial_integrals_(0, j) = non_zero_elements[j].first;
             idx_radial_integrals_(1, j) = non_zero_elements[j].second;
         }
     }
 
-    if (parameters_.processing_unit() == sddk::device_t::GPU && parameters_.full_potential()) {
-        idx_radial_integrals_.allocate(sddk::memory_t::device).copy_to(sddk::memory_t::device);
-        rf_coef_ =
-            sddk::mdarray<double, 3>(num_mt_points(), 4, indexr().size(), sddk::memory_t::host_pinned, "Atom_type::rf_coef_");
+    if (parameters_.processing_unit() == device_t::GPU && parameters_.full_potential()) {
+        idx_radial_integrals_.allocate(memory_t::device).copy_to(memory_t::device);
+        rf_coef_ = mdarray<double, 3>({num_mt_points(), 4, indexr().size()}, memory_t::host_pinned,
+                                      mdarray_label("Atom_type::rf_coef_"));
         vrf_coef_ =
-            sddk::mdarray<double, 3>(num_mt_points(), 4, lmmax_pot * indexr().size() * (parameters_.num_mag_dims() + 1),
-                               sddk::memory_t::host_pinned, "Atom_type::vrf_coef_");
-        rf_coef_.allocate(sddk::memory_t::device);
-        vrf_coef_.allocate(sddk::memory_t::device);
+                mdarray<double, 3>({num_mt_points(), 4, lmmax_pot * indexr().size() * (parameters_.num_mag_dims() + 1)},
+                                   memory_t::host_pinned, mdarray_label("Atom_type::vrf_coef_"));
+        rf_coef_.allocate(memory_t::device);
+        vrf_coef_.allocate(memory_t::device);
     }
-    if (parameters_.processing_unit() == sddk::device_t::GPU) {
+    if (parameters_.processing_unit() == device_t::GPU) {
         radial_grid_.copy_to_device();
     }
 
@@ -182,9 +208,9 @@ Atom_type::init(int offset_lo__)
         if (num_beta_radial_functions() != num_ae_paw_wf()) {
             RTE_THROW("wrong number of all-electron wave-functions for PAW");
         }
-        ae_paw_wfs_array_ = sddk::mdarray<double, 2>(num_mt_points(), num_beta_radial_functions());
+        ae_paw_wfs_array_ = mdarray<double, 2>({num_mt_points(), num_beta_radial_functions()});
         ae_paw_wfs_array_.zero();
-        ps_paw_wfs_array_ = sddk::mdarray<double, 2>(num_mt_points(), num_beta_radial_functions());
+        ps_paw_wfs_array_ = mdarray<double, 2>({num_mt_points(), num_beta_radial_functions()});
         ps_paw_wfs_array_.zero();
 
         for (int i = 0; i < num_beta_radial_functions(); i++) {
@@ -252,7 +278,7 @@ void
 Atom_type::print_info(std::ostream& out__) const
 {
     out__ << "label          : " << label() << std::endl
-          << utils::hbar(80, '-') << std::endl
+          << hbar(80, '-') << std::endl
           << "symbol         : " << symbol_ << std::endl
           << "name           : " << name_ << std::endl
           << "zn             : " << zn_ << std::endl
@@ -268,7 +294,7 @@ Atom_type::print_info(std::ostream& out__) const
     if (parameters_.full_potential()) {
         out__ << std::endl;
         out__ << "atomic levels" << std::endl;
-        for (auto& e: atomic_levels_) {
+        for (auto& e : atomic_levels_) {
             out__ << "n: " << e.n << ", l: " << e.l << ", k: " << e.k << ", occ: " << e.occupancy
                   << ", core: " << e.core << std::endl;
         }
@@ -303,7 +329,7 @@ Atom_type::print_info(std::ostream& out__) const
     out__ << std::endl;
     out__ << "total number of radial functions : " << indexr().size() << std::endl
           << "lmax of radial functions         : " << indexr().lmax() << std::endl
-          << "max. number of radial functions  : " << indexr().max_num_rf() << std::endl
+          << "max. number of radial functions  : " << indexr().max_order() << std::endl
           << "total number of basis functions  : " << indexb().size() << std::endl
           << "number of aw basis functions     : " << indexb().size_aw() << std::endl
           << "number of lo basis functions     : " << indexb().size_lo() << std::endl
@@ -311,12 +337,13 @@ Atom_type::print_info(std::ostream& out__) const
     if (!parameters_.full_potential()) {
         out__ << "lmax of beta-projectors          : " << this->lmax_beta() << std::endl
               << "number of ps wavefunctions       : " << this->indexr_wfs().size() << std::endl
-              << "charge augmentation              : " << utils::boolstr(this->augment()) << std::endl
-              << "vloc is set                      : " << utils::boolstr(!this->local_potential().empty()) << std::endl
-              << "ps_rho_core is set               : " << utils::boolstr(!this->ps_core_charge_density().empty()) << std::endl
-              << "ps_rho_total is set              : " << utils::boolstr(!this->ps_total_charge_density().empty()) << std::endl;
+              << "charge augmentation              : " << boolstr(this->augment()) << std::endl
+              << "vloc is set                      : " << boolstr(!this->local_potential().empty()) << std::endl
+              << "ps_rho_core is set               : " << boolstr(!this->ps_core_charge_density().empty()) << std::endl
+              << "ps_rho_total is set              : " << boolstr(!this->ps_total_charge_density().empty())
+              << std::endl;
     }
-    out__ << "Hubbard correction               : " << utils::boolstr(this->hubbard_correction()) << std::endl;
+    out__ << "Hubbard correction               : " << boolstr(this->hubbard_correction()) << std::endl;
     if (parameters_.hubbard_correction() && this->hubbard_correction_) {
         out__ << "  angular momentum                   : " << lo_descriptors_hub_[0].l() << std::endl
               << "  principal quantum number           : " << lo_descriptors_hub_[0].n() << std::endl
@@ -331,23 +358,25 @@ Atom_type::print_info(std::ostream& out__) const
             }
             out__ << lo_descriptors_hub_[i];
         }
+
+        bool orthogonalize_ = parameters_.cfg().hubbard().hubbard_subspace_method() == "orthogonalize";
+        bool full_orthogonalization_ =
+                parameters_.cfg().hubbard().hubbard_subspace_method() == "full_orthogonalization";
+        bool normalize_ = parameters_.cfg().hubbard().hubbard_subspace_method() == "normalize";
         out__ << std::endl;
-        out__ << "  orthogonalize                      : "
-              << utils::boolstr(parameters_.cfg().hubbard().orthogonalize()) << std::endl
-              << "  normalize                          : "
-              << utils::boolstr(parameters_.cfg().hubbard().normalize()) << std::endl
-              << "  full_orthogonalization             : "
-              << utils::boolstr(parameters_.cfg().hubbard().full_orthogonalization()) << std::endl
-              << "  simplified                         : "
-              << utils::boolstr(parameters_.cfg().hubbard().simplified()) << std::endl;
+        out__ << "  orthogonalize                      : " << boolstr(orthogonalize_) << std::endl
+              << "  normalize                          : " << boolstr(normalize_) << std::endl
+              << "  full_orthogonalization             : " << boolstr(full_orthogonalization_) << std::endl
+              << "  simplified                         : " << boolstr(parameters_.cfg().hubbard().simplified())
+              << std::endl;
     }
-    out__ << "spin-orbit coupling              : " << utils::boolstr(this->spin_orbit_coupling()) << std::endl;
+    out__ << "spin-orbit coupling              : " << boolstr(this->spin_orbit_coupling()) << std::endl;
     out__ << "atomic wave-functions            : ";
-    for (int i = 0; i < indexr_wfs_.size(); i++) {
-        if (i) {
+    for (auto e : indexr_wfs_) {
+        if (e.idxrf) {
             out__ << ", ";
         }
-        out__ << indexr_wfs_.am(i);
+        out__ << e.am;
     }
     out__ << std::endl;
     out__ << std::endl;
@@ -377,7 +406,7 @@ Atom_type::read_input_core(nlohmann::json const& parser)
             if (n <= 0 || iss.fail()) {
                 std::stringstream s;
                 s << "wrong principal quantum number : " << std::string(1, c1);
-                TERMINATE(s);
+                RTE_THROW(s);
             }
 
             switch (c2) {
@@ -400,7 +429,7 @@ Atom_type::read_input_core(nlohmann::json const& parser)
                 default: {
                     std::stringstream s;
                     s << "wrong angular momentum label : " << std::string(1, c2);
-                    TERMINATE(s);
+                    RTE_THROW(s);
                 }
             }
 
@@ -459,8 +488,8 @@ Atom_type::read_input_lo(nlohmann::json const& parser)
     for (size_t j = 0; j < parser["lo"].size(); j++) {
         l = parser["lo"][j]["l"].get<int>();
 
-        local_orbital_descriptor lod;
-        lod.l = l;
+        angular_momentum am(l);
+        local_orbital_descriptor lod(am);
         rsd.l = l;
         rsd_set.clear();
         for (size_t order = 0; order < parser["lo"][j]["basis"].size(); order++) {
@@ -488,7 +517,7 @@ Atom_type::read_pseudo_uspp(nlohmann::json const& parser)
 
     auto rgrid = parser["pseudo_potential"]["radial_grid"].get<std::vector<double>>();
     if (static_cast<int>(rgrid.size()) != nmtp) {
-        TERMINATE("wrong mesh size");
+        RTE_THROW("wrong mesh size");
     }
     /* set the radial grid */
     set_radial_grid(nmtp, rgrid.data());
@@ -496,7 +525,7 @@ Atom_type::read_pseudo_uspp(nlohmann::json const& parser)
     local_potential(parser["pseudo_potential"]["local_potential"].get<std::vector<double>>());
 
     ps_core_charge_density(
-        parser["pseudo_potential"].value("core_charge_density", std::vector<double>(rgrid.size(), 0)));
+            parser["pseudo_potential"].value("core_charge_density", std::vector<double>(rgrid.size(), 0)));
 
     ps_total_charge_density(parser["pseudo_potential"]["total_charge_density"].get<std::vector<double>>());
 
@@ -504,7 +533,7 @@ Atom_type::read_pseudo_uspp(nlohmann::json const& parser)
         ps_total_charge_density().size() != rgrid.size()) {
         std::cout << local_potential().size() << " " << ps_core_charge_density().size() << " "
                   << ps_total_charge_density().size() << std::endl;
-        TERMINATE("wrong array size");
+        RTE_THROW("wrong array size");
     }
 
     if (parser["pseudo_potential"]["header"].count("spin_orbit")) {
@@ -520,7 +549,7 @@ Atom_type::read_pseudo_uspp(nlohmann::json const& parser)
             s << "wrong size of beta functions for atom type " << symbol_ << " (label: " << label_ << ")" << std::endl
               << "size of beta radial functions in the file: " << beta.size() << std::endl
               << "radial grid size: " << num_mt_points();
-            TERMINATE(s);
+            RTE_THROW(s);
         }
         int l = parser["pseudo_potential"]["beta_projectors"][i]["angular_momentum"].get<int>();
         if (spin_orbit_coupling_) {
@@ -532,10 +561,19 @@ Atom_type::read_pseudo_uspp(nlohmann::json const& parser)
                 l *= -1;
             }
         }
-        add_beta_radial_function(l, beta);
+        // add_beta_radial_function(l, beta);
+        if (spin_orbit_coupling_) {
+            if (l >= 0) {
+                add_beta_radial_function(angular_momentum(l, 1), beta);
+            } else {
+                add_beta_radial_function(angular_momentum(-l, -1), beta);
+            }
+        } else {
+            add_beta_radial_function(angular_momentum(l), beta);
+        }
     }
 
-    sddk::mdarray<double, 2> d_mtrx(nbf, nbf);
+    mdarray<double, 2> d_mtrx({nbf, nbf});
     d_mtrx.zero();
     auto v = parser["pseudo_potential"]["D_ion"].get<std::vector<double>>();
 
@@ -548,13 +586,12 @@ Atom_type::read_pseudo_uspp(nlohmann::json const& parser)
 
     if (parser["pseudo_potential"].count("augmentation")) {
         for (size_t k = 0; k < parser["pseudo_potential"]["augmentation"].size(); k++) {
-            int i = parser["pseudo_potential"]["augmentation"][k]["i"].get<int>();
-            int j = parser["pseudo_potential"]["augmentation"][k]["j"].get<int>();
-            // int idx  = j * (j + 1) / 2 + i;
+            int i    = parser["pseudo_potential"]["augmentation"][k]["i"].get<int>();
+            int j    = parser["pseudo_potential"]["augmentation"][k]["j"].get<int>();
             int l    = parser["pseudo_potential"]["augmentation"][k]["angular_momentum"].get<int>();
             auto qij = parser["pseudo_potential"]["augmentation"][k]["radial_function"].get<std::vector<double>>();
             if ((int)qij.size() != num_mt_points()) {
-                TERMINATE("wrong size of qij");
+                RTE_THROW("wrong size of qij");
             }
             add_q_radial_function(i, j, l, qij);
         }
@@ -604,7 +641,7 @@ Atom_type::read_pseudo_uspp(nlohmann::json const& parser)
                 }
                 k += 1;
             }
-            add_ps_atomic_wf(n, sirius::experimental::angular_momentum(l), v, occ);
+            add_ps_atomic_wf(n, angular_momentum(l), v, occ);
         }
     }
 }
@@ -627,7 +664,7 @@ Atom_type::read_pseudo_paw(nlohmann::json const& parser)
 
     /* read core density and potential */
     paw_ae_core_charge_density(
-        parser["pseudo_potential"]["paw_data"]["ae_core_charge_density"].get<std::vector<double>>());
+            parser["pseudo_potential"]["paw_data"]["ae_core_charge_density"].get<std::vector<double>>());
 
     /* read occupations */
     paw_wf_occ(parser["pseudo_potential"]["paw_data"]["occupations"].get<std::vector<double>>());
@@ -645,7 +682,7 @@ Atom_type::read_pseudo_paw(nlohmann::json const& parser)
             s << "wrong size of ae_wfc functions for atom type " << symbol_ << " (label: " << label_ << ")" << std::endl
               << "size of ae_wfc radial functions in the file: " << wfc.size() << std::endl
               << "radial grid size: " << num_mt_points();
-            TERMINATE(s);
+            RTE_THROW(s);
         }
 
         add_ae_paw_wf(std::vector<double>(wfc.begin(), wfc.begin() + cutoff_radius_index));
@@ -657,7 +694,7 @@ Atom_type::read_pseudo_paw(nlohmann::json const& parser)
             s << "wrong size of ps_wfc functions for atom type " << symbol_ << " (label: " << label_ << ")" << std::endl
               << "size of ps_wfc radial functions in the file: " << wfc.size() << std::endl
               << "radial grid size: " << num_mt_points();
-            TERMINATE(s);
+            RTE_THROW(s);
         }
 
         add_ps_paw_wf(std::vector<double>(wfc.begin(), wfc.begin() + cutoff_radius_index));
@@ -667,7 +704,7 @@ Atom_type::read_pseudo_paw(nlohmann::json const& parser)
 void
 Atom_type::read_input(std::string const& str__)
 {
-    auto parser = utils::read_json_from_file_or_string(str__);
+    auto parser = read_json_from_file_or_string(str__);
 
     if (parser.empty()) {
         return;
@@ -728,16 +765,16 @@ Atom_type::generate_f_coefficients()
 
     // number of beta projectors
     int nbf         = this->mt_basis_size();
-    f_coefficients_ = sddk::mdarray<std::complex<double>, 4>(nbf, nbf, 2, 2);
+    f_coefficients_ = mdarray<std::complex<double>, 4>({nbf, nbf, 2, 2});
     f_coefficients_.zero();
 
     for (int xi2 = 0; xi2 < nbf; xi2++) {
-        const int l2    = this->indexb(xi2).l;
-        const double j2 = this->indexb(xi2).j;
+        const int l2    = this->indexb(xi2).am.l();
+        const double j2 = this->indexb(xi2).am.j();
         const int m2    = this->indexb(xi2).m;
         for (int xi1 = 0; xi1 < nbf; xi1++) {
-            const int l1    = this->indexb(xi1).l;
-            const double j1 = this->indexb(xi1).j;
+            const int l1    = this->indexb(xi1).am.l();
+            const double j1 = this->indexb(xi1).am.j();
             const int m1    = this->indexb(xi1).m;
 
             if ((l2 == l1) && (std::abs(j1 - j2) < 1e-8)) {
@@ -801,7 +838,9 @@ Atom_type::read_hubbard_input()
             for (int s = 0; s < 6; s++) {
                 coeff[s] /= ha2ev;
             }
+
             std::vector<double> initial_occupancy;
+
             if (ho.contains("initial_occupancy")) {
                 initial_occupancy = ho.initial_occupancy();
 
@@ -821,7 +860,7 @@ Atom_type::read_hubbard_input()
         }
     }
 
-    if (parameters_.cfg().hubbard().full_orthogonalization()) {
+    if (parameters_.cfg().hubbard().hubbard_subspace_method() == "full_orthogonalization") {
         this->hubbard_correction_ = true;
         if (lo_descriptors_hub_.empty()) {
             for (int s = 0; s < (int)ps_atomic_wfs_.size(); s++) {
@@ -894,7 +933,7 @@ Atom_type::add_hubbard_orbital(int n__, int l__, double occ__, double U, double
     }
 
     /* add a record in radial function index */
-    indexr_hub_.add(sirius::experimental::angular_momentum(l__));
+    indexr_hub_.add(angular_momentum(l__));
     /* add Hubbard orbital descriptor to a list */
     lo_descriptors_hub_.emplace_back(n__, l__, -1, occ__, J, U, hub_coef__, alpha__, beta__, J0__, initial_occupancy__,
                                      std::move(s.interpolate()), use_for_calculations__, idx_rf);
